@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2015-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Alexander Heinecke, Hans Pabst (Intel Corp.)
 ******************************************************************************/
@@ -52,10 +32,14 @@ LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_wigemm_descriptor_init(libxsmm_desc
   libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
   int alpha, int beta, int flags, int prefetch);
-LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_wsgemm_descriptor_init(libxsmm_descriptor_blob* blob,
+LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_bigemm_descriptor_init(libxsmm_descriptor_blob* blob,
   libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
-  float alpha, float beta, int flags, int prefetch);
+  int alpha, int beta, int flags, int prefetch);
+LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_bbgemm_descriptor_init(libxsmm_descriptor_blob* blob,
+  libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
+  libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
+  int alpha, int beta, int flags, int prefetch);
 LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_bsgemm_descriptor_init(libxsmm_descriptor_blob* blob,
   libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
@@ -135,6 +119,10 @@ LIBXSMM_EXTERN_C typedef struct libxsmm_generated_code {
                                *  0: no error occurred
                                * >0: error code
                                */
+  unsigned int arch;          /* target arch for the current code generation task */
+  unsigned int sf_size;       /* offset of RSP to the beginning of the stack frame
+                               * we track this value to have RBP availbale for general compute
+                               */
 } libxsmm_generated_code;
 
 /** function to translate LIBXSMM Generator error codes to error messages */
@@ -155,11 +143,9 @@ void libxsmm_generator_gemm_directasm(const char*                    i_file_out,
                                       const libxsmm_gemm_descriptor* i_xgemm_desc,
                                       const char*                    i_arch );
 
-/* @TODO change int based architecture value */
 LIBXSMM_API
 void libxsmm_generator_gemm_kernel(libxsmm_generated_code*        io_generated_code,
-                                   const libxsmm_gemm_descriptor* i_xgemm_desc,
-                                   const char*                    i_arch );
+                                   const libxsmm_gemm_descriptor* i_xgemm_desc );
 
 /* @TODO change int based architecture value */
 LIBXSMM_API
@@ -204,7 +190,8 @@ void libxsmm_generator_spgemm_csr_soa_kernel(libxsmm_generated_code*        io_g
                                              const char*                    i_arch,
                                              const unsigned int*            i_row_idx,
                                              const unsigned int*            i_column_idx,
-                                             const void*                    i_values);
+                                             const void*                    i_values,
+                                             const unsigned int             i_packed_width );
 
 /* @TODO change int based architecture value */
 LIBXSMM_API
@@ -213,16 +200,19 @@ void libxsmm_generator_spgemm_csc_soa_kernel( libxsmm_generated_code*        io_
                                               const char*                    i_arch,
                                               const unsigned int*            i_row_idx,
                                               const unsigned int*            i_column_idx,
-                                              const void*                    i_values );
+                                              const void*                    i_values,
+                                              const unsigned int             i_packed_width );
 
 /* @TODO change int based architecture value */
 LIBXSMM_API void libxsmm_generator_packed_gemm_ac_rm( libxsmm_generated_code*         io_generated_code,
                                                       const libxsmm_gemm_descriptor*  i_xgemm_desc,
+                                                      const unsigned int              i_packed_width,
                                                       const char*                     i_arch );
 
 /* @TODO change int based architecture value */
 LIBXSMM_API void libxsmm_generator_packed_gemm_bc_rm( libxsmm_generated_code*         io_generated_code,
                                                       const libxsmm_gemm_descriptor*  i_xgemm_desc,
+                                                      const unsigned int              i_packed_width,
                                                       const char*                     i_arch );
 
 LIBXSMM_API
@@ -256,28 +246,12 @@ void libxsmm_generator_transpose_kernel( libxsmm_generated_code*          io_gen
                                          const libxsmm_trans_descriptor*  i_trans_desc,
                                          int                              i_arch );
 
-/* @TODO change int based architecture value */
-LIBXSMM_API
-void libxsmm_generator_convolution_forward_kernel(libxsmm_generated_code*           io_generated_code,
-                                                  const libxsmm_convolution_forward_descriptor* i_conv_desc,
-                                                  const char*                       i_arch);
-
-/* @TODO change int based architecture value */
-LIBXSMM_API
-void libxsmm_generator_convolution_backward_kernel(libxsmm_generated_code*           io_generated_code,
-                                                   const libxsmm_convolution_backward_descriptor* i_conv_desc,
-                                                   const char*                       i_arch);
-
-/* @TODO change int based architecture value */
-LIBXSMM_API
-void libxsmm_generator_convolution_weight_update_kernel(libxsmm_generated_code*           io_generated_code,
-                                                        const libxsmm_convolution_weight_update_descriptor* i_conv_desc,
-                                                        const char*                       i_arch);
-
 /** Initialization counter that can be used to check whether the library is initialized (!=0) or not (==0). */
-LIBXSMM_APIVAR_ALIGNED(unsigned int libxsmm_ninit);
+LIBXSMM_APIVAR_PUBLIC(unsigned int libxsmm_ninit);
+/** Target architecture (libxsmm_get_target_archid, libxsmm_set_target_archid). */
+LIBXSMM_APIVAR_PUBLIC(int libxsmm_target_archid);
 /** Verbosity level (0: quiet, 1: errors, 2: warnings, 3: info, neg.: all/dump). */
-LIBXSMM_APIVAR_ALIGNED(int libxsmm_verbosity);
+LIBXSMM_APIVAR_PUBLIC(int libxsmm_verbosity);
 
 #endif /*LIBXSMM_GENERATOR_H*/
 
