@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2013-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Hans Pabst (Intel Corp.)
 ******************************************************************************/
@@ -36,6 +16,7 @@
 /** Parameters the library was built for. */
 #define LIBXSMM_CACHELINE LIBXSMM_CONFIG_CACHELINE
 #define LIBXSMM_ALIGNMENT LIBXSMM_CONFIG_ALIGNMENT
+#define LIBXSMM_MALLOC LIBXSMM_CONFIG_MALLOC
 #define LIBXSMM_ILP64 LIBXSMM_CONFIG_ILP64
 #define LIBXSMM_SYNC LIBXSMM_CONFIG_SYNC
 #define LIBXSMM_JIT LIBXSMM_CONFIG_JIT
@@ -51,14 +32,32 @@
 #define LIBXSMM_ALPHA LIBXSMM_CONFIG_ALPHA
 #define LIBXSMM_BETA LIBXSMM_CONFIG_BETA
 
+/**
+ * Use "make PLATFORM=1" to disable platform checks.
+ * The platform check is to bail-out with an error
+ * message for an attempt to build an upstream package
+ * and subsequently to list LIBXSMM as "broken" on
+ * that platform.
+ * Note: successful compilation on an unsupported
+ * platform is desired, but only fall-back code is
+ * present at best.
+ */
+#if !defined(LIBXSMM_PLATFORM_FORCE) && 0
+# define LIBXSMM_PLATFORM_FORCE
+#endif
+
+#if !defined(LIBXSMM_PLATFORM_X86) && ( \
+    (defined(__x86_64__) && 0 != (__x86_64__)) || \
+    (defined(__amd64__) && 0 != (__amd64__)) || \
+    (defined(_M_X64) || defined(_M_AMD64)) || \
+    (defined(__i386__) && 0 != (__i386__)) || \
+    (defined(_M_IX86)))
+# define LIBXSMM_PLATFORM_X86
+#endif
 #if !defined(LIBXSMM_PLATFORM_SUPPORTED)
-# if  (defined(__x86_64__) && 0 != (__x86_64__)) || \
-      (defined(__amd64__) && 0 != (__amd64__)) || \
-      (defined(_M_X64) || defined(_M_AMD64)) || \
-      (defined(__i386__) && 0 != (__i386__)) || \
-      (defined(_M_IX86))
+# if defined(LIBXSMM_PLATFORM_X86)
 #   define LIBXSMM_PLATFORM_SUPPORTED
-# else /* JIT-generated code (among other issues) is not supported! */
+# elif !defined(LIBXSMM_PLATFORM_FORCE)
 #   error Intel Architecture or compatible CPU required!
 # endif
 #endif
@@ -68,11 +67,14 @@
       (defined(__x86_64__) && 0 != (__x86_64__)) || \
       (defined(__amd64__) && 0 != (__amd64__)) || \
       (defined(_M_X64) || defined(_M_AMD64)) || \
-      (defined(_WIN64))
+      (defined(_WIN64)) || \
+      (defined(__powerpc64))
+#   define LIBXSMM_UNLIMITED 0xFFFFFFFFFFFFFFFF
 #   define LIBXSMM_BITS 64
-# elif defined(NDEBUG) /* not for production use! */
+# elif !defined(LIBXSMM_PLATFORM_FORCE) && defined(NDEBUG)
 #   error LIBXSMM is only supported on a 64-bit platform!
 # else /* JIT-generated code (among other issues) is not supported! */
+#   define LIBXSMM_UNLIMITED 0xFFFFFFFF
 #   define LIBXSMM_BITS 32
 # endif
 #endif
@@ -80,10 +82,10 @@
 #define LIBXSMM_STRINGIFY2(SYMBOL) #SYMBOL
 #define LIBXSMM_STRINGIFY(SYMBOL) LIBXSMM_STRINGIFY2(SYMBOL)
 #define LIBXSMM_TOSTRING(SYMBOL) LIBXSMM_STRINGIFY(SYMBOL)
-#define LIBXSMM_CONCATENATE1(A, B) A##B
-#define LIBXSMM_CONCATENATE(A, B) LIBXSMM_CONCATENATE1(A, B)
-#define LIBXSMM_CONCATENATE2(A, B, C) LIBXSMM_CONCATENATE(LIBXSMM_CONCATENATE(A, B), C)
-#define LIBXSMM_CONCATENATE3(A, B, C, D) LIBXSMM_CONCATENATE(LIBXSMM_CONCATENATE2(A, B, C), D)
+#define LIBXSMM_CONCATENATE2(A, B) A##B
+#define LIBXSMM_CONCATENATE3(A, B, C) LIBXSMM_CONCATENATE(LIBXSMM_CONCATENATE(A, B), C)
+#define LIBXSMM_CONCATENATE4(A, B, C, D) LIBXSMM_CONCATENATE(LIBXSMM_CONCATENATE3(A, B, C), D)
+#define LIBXSMM_CONCATENATE(A, B) LIBXSMM_CONCATENATE2(A, B)
 #define LIBXSMM_FSYMBOL(SYMBOL) LIBXSMM_CONCATENATE(SYMBOL, _)
 #define LIBXSMM_UNIQUE(NAME) LIBXSMM_CONCATENATE(NAME, __LINE__)
 #define LIBXSMM_EXPAND(...) __VA_ARGS__
@@ -91,18 +93,86 @@
 
 #define LIBXSMM_VERSION2(MAJOR, MINOR) ((MAJOR) * 10000 + (MINOR) * 100)
 #define LIBXSMM_VERSION3(MAJOR, MINOR, UPDATE) (LIBXSMM_VERSION2(MAJOR, MINOR) + (UPDATE))
-#define LIBXSMM_VERSION4(MAJOR, MINOR, UPDATE, PATCH) \
-  (((MAJOR/*7b*/) << 24) | ((MINOR/*5b*/) << 19) | ((UPDATE/*5b*/) << 14) | (PATCH/*14b*/))
+#define LIBXSMM_VERSION4(MAJOR, MINOR, UPDATE, PATCH) (int) \
+  (((0x7F & (MAJOR)) << 24) | ((0x1F & (MINOR)) << 19) | ((0x1F & (UPDATE)) << 14) | (0x3FFF & (PATCH)))
 #define LIBXSMM_VERSION41(VERSION) (((VERSION) >> 24))
 #define LIBXSMM_VERSION42(VERSION) (((VERSION) >> 19) & 0x1F)
 #define LIBXSMM_VERSION43(VERSION) (((VERSION) >> 14) & 0x1F)
 #define LIBXSMM_VERSION44(VERSION) (((VERSION)) & 0x3FFF)
 
+#if !defined(LIBXSMM_UNPACKED) && (defined(_CRAYC) || defined(LIBXSMM_OFFLOAD_BUILD) || \
+  (0 == LIBXSMM_SYNC)/*Windows: missing pack(pop) error*/)
+# define LIBXSMM_UNPACKED
+#endif
+#if defined(_WIN32) && !defined(__GNUC__) && !defined(__clang__)
+# define LIBXSMM_ATTRIBUTE(A) __declspec(A)
+# if defined(__cplusplus)
+#   define LIBXSMM_INLINE_ALWAYS __forceinline
+# else
+#   define LIBXSMM_INLINE_ALWAYS static __forceinline
+# endif
+# define LIBXSMM_ALIGNED(DECL, N) LIBXSMM_ATTRIBUTE(align(N)) DECL
+# if !defined(LIBXSMM_UNPACKED)
+#   define LIBXSMM_PACKED(TYPE) LIBXSMM_PRAGMA(pack(1)) TYPE
+# endif
+# define LIBXSMM_CDECL __cdecl
+#elif (defined(__GNUC__) || defined(__clang__) || defined(__PGI))
+# define LIBXSMM_ATTRIBUTE(A) __attribute__((A))
+# define LIBXSMM_INLINE_ALWAYS LIBXSMM_ATTRIBUTE(always_inline) LIBXSMM_INLINE
+# define LIBXSMM_ALIGNED(DECL, N) LIBXSMM_ATTRIBUTE(aligned(N)) DECL
+# if !defined(LIBXSMM_UNPACKED)
+#   define LIBXSMM_PACKED(TYPE) TYPE LIBXSMM_ATTRIBUTE(__packed__)
+# endif
+# define LIBXSMM_CDECL LIBXSMM_ATTRIBUTE(cdecl)
+#else
+# define LIBXSMM_ATTRIBUTE(A)
+# define LIBXSMM_INLINE_ALWAYS LIBXSMM_INLINE
+# define LIBXSMM_ALIGNED(DECL, N) DECL
+# define LIBXSMM_CDECL
+#endif
+#if !defined(LIBXSMM_PACKED)
+# define LIBXSMM_PACKED(TYPE) TYPE
+# if !defined(LIBXSMM_UNPACKED)
+#   define LIBXSMM_UNPACKED
+# endif
+#endif
+#if defined(LIBXSMM_UNPACKED)
+# define LIBXSMM_PAD(EXPR)
+#else /* no braces around EXPR */
+# define LIBXSMM_PAD(EXPR) EXPR;
+#endif
+
+#if defined(__INTEL_COMPILER)
+# if !defined(__INTEL_COMPILER_UPDATE)
+#   define LIBXSMM_INTEL_COMPILER __INTEL_COMPILER
+# else
+#   define LIBXSMM_INTEL_COMPILER (__INTEL_COMPILER + __INTEL_COMPILER_UPDATE)
+# endif
+#elif defined(__INTEL_COMPILER_BUILD_DATE)
+# define LIBXSMM_INTEL_COMPILER ((__INTEL_COMPILER_BUILD_DATE / 10000 - 2000) * 100)
+#endif
+
+/* LIBXSMM_ATTRIBUTE_USED: mark library functions as used to avoid warning */
+#if defined(__GNUC__) || defined(__clang__) || (defined(__INTEL_COMPILER) && !defined(_WIN32))
+# if !defined(__cplusplus) || !defined(__clang__)
+#   define LIBXSMM_ATTRIBUTE_COMMON LIBXSMM_ATTRIBUTE(common)
+# else
+#   define LIBXSMM_ATTRIBUTE_COMMON
+# endif
+# define LIBXSMM_ATTRIBUTE_MALLOC LIBXSMM_ATTRIBUTE(malloc)
+# define LIBXSMM_ATTRIBUTE_UNUSED LIBXSMM_ATTRIBUTE(unused)
+# define LIBXSMM_ATTRIBUTE_USED LIBXSMM_ATTRIBUTE(used)
+#else
+# define LIBXSMM_ATTRIBUTE_COMMON
+# define LIBXSMM_ATTRIBUTE_MALLOC
+# define LIBXSMM_ATTRIBUTE_UNUSED
+# define LIBXSMM_ATTRIBUTE_USED
+#endif
+
 #if defined(__cplusplus)
 # define LIBXSMM_VARIADIC ...
-# define LIBXSMM_EXTERN_KEYWORD extern "C"
-# define LIBXSMM_EXTERN LIBXSMM_EXTERN_KEYWORD
-# define LIBXSMM_EXTERN_C LIBXSMM_EXTERN_KEYWORD
+# define LIBXSMM_EXTERN extern "C"
+# define LIBXSMM_EXTERN_C extern "C"
 # define LIBXSMM_INLINE_KEYWORD inline
 # define LIBXSMM_INLINE LIBXSMM_INLINE_KEYWORD
 # if defined(__GNUC__) || defined(_CRAYC)
@@ -113,10 +183,9 @@
 # else
 #   define LIBXSMM_CALLER __FUNCNAME__
 # endif
-#else
+#else /* C */
 # define LIBXSMM_VARIADIC
-# define LIBXSMM_EXTERN_KEYWORD extern
-# define LIBXSMM_EXTERN LIBXSMM_EXTERN_KEYWORD
+# define LIBXSMM_EXTERN extern
 # define LIBXSMM_EXTERN_C
 # if defined(__STDC_VERSION__) && (199901L <= __STDC_VERSION__) /*C99*/
 #   define LIBXSMM_PRAGMA(DIRECTIVE) _Pragma(LIBXSMM_STRINGIFY(DIRECTIVE))
@@ -135,7 +204,8 @@
 #   define LIBXSMM_INLINE_KEYWORD
 #   define LIBXSMM_INLINE_FIXUP
 # endif
-# define LIBXSMM_INLINE static LIBXSMM_INLINE_KEYWORD
+/* LIBXSMM_ATTRIBUTE_USED: increases compile-time of header-only by a large factor */
+# define LIBXSMM_INLINE static LIBXSMM_INLINE_KEYWORD LIBXSMM_ATTRIBUTE_UNUSED
 #endif /*__cplusplus*/
 #if !defined(LIBXSMM_CALLER)
 # define LIBXSMM_CALLER NULL
@@ -151,53 +221,6 @@
 # endif
 #endif
 
-#if !defined(LIBXSMM_UNPACKED) && (defined(_CRAYC) || defined(LIBXSMM_OFFLOAD_BUILD))
-# define LIBXSMM_UNPACKED
-#endif
-
-#if defined(_WIN32) && !defined(__GNUC__)
-# define LIBXSMM_ATTRIBUTE(A) __declspec(A)
-# if defined(__cplusplus)
-#   define LIBXSMM_INLINE_ALWAYS __forceinline
-# else
-#   define LIBXSMM_INLINE_ALWAYS static __forceinline
-# endif
-# define LIBXSMM_ALIGNED(DECL, N) LIBXSMM_ATTRIBUTE(align(N)) DECL
-# if !defined(LIBXSMM_UNPACKED)
-#   define LIBXSMM_PACKED(TYPE) LIBXSMM_PRAGMA(pack(1)) TYPE
-# endif
-# define LIBXSMM_CDECL __cdecl
-#elif (defined(__GNUC__) || defined(__clang__) || defined(__PGI))
-# define LIBXSMM_ATTRIBUTE(A) __attribute__((A))
-# define LIBXSMM_INLINE_ALWAYS LIBXSMM_ATTRIBUTE(always_inline) LIBXSMM_INLINE
-# define LIBXSMM_ALIGNED(DECL, N) DECL LIBXSMM_ATTRIBUTE(aligned(N))
-# if !defined(LIBXSMM_UNPACKED)
-#   define LIBXSMM_PACKED(TYPE) TYPE LIBXSMM_ATTRIBUTE(__packed__)
-# endif
-# define LIBXSMM_CDECL LIBXSMM_ATTRIBUTE(cdecl)
-#else
-# define LIBXSMM_ATTRIBUTE(A)
-# define LIBXSMM_INLINE_ALWAYS LIBXSMM_INLINE
-# define LIBXSMM_ALIGNED(DECL, N) DECL
-# define LIBXSMM_CDECL
-#endif
-#if !defined(LIBXSMM_PACKED)
-# define LIBXSMM_PACKED(TYPE) TYPE
-# if !defined(LIBXSMM_UNPACKED)
-#   define LIBXSMM_UNPACKED
-# endif
-#endif
-
-#if defined(__INTEL_COMPILER)
-# if !defined(__INTEL_COMPILER_UPDATE)
-#   define LIBXSMM_INTEL_COMPILER __INTEL_COMPILER
-# else
-#   define LIBXSMM_INTEL_COMPILER (__INTEL_COMPILER + __INTEL_COMPILER_UPDATE)
-# endif
-#elif defined(__INTEL_COMPILER_BUILD_DATE)
-# define LIBXSMM_INTEL_COMPILER ((__INTEL_COMPILER_BUILD_DATE / 10000 - 2000) * 100)
-#endif
-
 #if defined(LIBXSMM_OFFLOAD_BUILD) && \
   defined(__INTEL_OFFLOAD) && (!defined(_WIN32) || (1400 <= LIBXSMM_INTEL_COMPILER))
 # define LIBXSMM_OFFLOAD(A) LIBXSMM_ATTRIBUTE(target(A))
@@ -240,59 +263,59 @@
 # define LIBXSMM_VISIBILITY_IMPORT LIBXSMM_ATTRIBUTE(dllimport)
 #endif
 #if !defined(LIBXSMM_VISIBILITY_EXPORT)
-# define LIBXSMM_VISIBILITY_EXPORT LIBXSMM_EXTERN_C LIBXSMM_VISIBILITY_PUBLIC
+# define LIBXSMM_VISIBILITY_EXPORT LIBXSMM_VISIBILITY_PUBLIC
 #endif
 #if !defined(LIBXSMM_VISIBILITY_IMPORT)
-# define LIBXSMM_VISIBILITY_IMPORT LIBXSMM_EXTERN_KEYWORD LIBXSMM_VISIBILITY_PUBLIC
+# define LIBXSMM_VISIBILITY_IMPORT LIBXSMM_VISIBILITY_PUBLIC
 #endif
 
-#if defined(LIBXSMM_API) /* header-only mode */
-# define LIBXSMM_API_INTERN LIBXSMM_API
-# define LIBXSMM_API_EXPORT LIBXSMM_API
-# define LIBXSMM_APIVAR_PUBLIC(DECL) LIBXSMM_EXTERN_C LIBXSMM_RETARGETABLE DECL; LIBXSMM_RETARGETABLE DECL
-# define LIBXSMM_APIVAR(DECL) LIBXSMM_APIVAR_PUBLIC(DECL)
-# define LIBXSMM_APIEXT_INTERN LIBXSMM_API_INTERN
-# define LIBXSMM_APIEXT LIBXSMM_APIEXT_INTERN
+#if defined(LIBXSMM_SOURCE_H) /* header-only mode */
+# define LIBXSMM_API_VISIBILITY_EXPORT
+# define LIBXSMM_API_VISIBILITY_IMPORT
+# define LIBXSMM_API_VISIBILITY_INTERN
+# define LIBXSMM_API_COMMON LIBXSMM_RETARGETABLE LIBXSMM_ATTRIBUTE_COMMON
+# define LIBXSMM_API_TARGET LIBXSMM_API_INLINE
+# define LIBXSMM_API_EXTERN LIBXSMM_EXTERN_C
 #else /* classic ABI */
 # if defined(LIBXSMM_BUILD_EXT)
-#   define LIBXSMM_API LIBXSMM_VISIBILITY_IMPORT LIBXSMM_RETARGETABLE
-#   define LIBXSMM_API_INTERN LIBXSMM_VISIBILITY_PRIVATE LIBXSMM_RETARGETABLE
-#   define LIBXSMM_API_EXPORT LIBXSMM_EXTERN_C LIBXSMM_API
-#   define LIBXSMM_APIVAR_PUBLIC(DECL) LIBXSMM_API DECL; LIBXSMM_API DECL
-#   define LIBXSMM_APIVAR(DECL) LIBXSMM_EXTERN_C LIBXSMM_VISIBILITY_PRIVATE LIBXSMM_RETARGETABLE DECL; \
-                                LIBXSMM_EXTERN_C LIBXSMM_VISIBILITY_PRIVATE LIBXSMM_RETARGETABLE DECL
-#   define LIBXSMM_APIEXT LIBXSMM_VISIBILITY_EXPORT LIBXSMM_RETARGETABLE
-#   define LIBXSMM_APIEXT_INTERN LIBXSMM_API_INTERN
+#   define LIBXSMM_API_VISIBILITY_EXPORT LIBXSMM_VISIBILITY_IMPORT
+#   define LIBXSMM_API_VISIBILITY_IMPORT LIBXSMM_VISIBILITY_EXPORT
+#   define LIBXSMM_API_VISIBILITY_INTERN LIBXSMM_VISIBILITY_PRIVATE
 # elif defined(LIBXSMM_BUILD)
-#   define LIBXSMM_API LIBXSMM_VISIBILITY_EXPORT LIBXSMM_RETARGETABLE
-#   define LIBXSMM_API_INTERN LIBXSMM_VISIBILITY_PRIVATE LIBXSMM_RETARGETABLE
-#   define LIBXSMM_API_EXPORT LIBXSMM_API
-#   define LIBXSMM_APIVAR_PUBLIC(DECL) LIBXSMM_VISIBILITY_EXPORT LIBXSMM_RETARGETABLE DECL; \
-                                       LIBXSMM_VISIBILITY_EXPORT LIBXSMM_RETARGETABLE DECL
-#   define LIBXSMM_APIVAR(DECL) LIBXSMM_EXTERN_C LIBXSMM_VISIBILITY_PRIVATE LIBXSMM_RETARGETABLE DECL; \
-                                LIBXSMM_EXTERN_C LIBXSMM_VISIBILITY_PRIVATE LIBXSMM_RETARGETABLE DECL
-#   define LIBXSMM_APIEXT LIBXSMM_VISIBILITY_IMPORT LIBXSMM_RETARGETABLE
+#   define LIBXSMM_API_VISIBILITY_EXPORT LIBXSMM_VISIBILITY_EXPORT
+#   define LIBXSMM_API_VISIBILITY_IMPORT LIBXSMM_VISIBILITY_IMPORT
+#   define LIBXSMM_API_VISIBILITY_INTERN LIBXSMM_VISIBILITY_PRIVATE
 # else /* import */
-#   define LIBXSMM_API LIBXSMM_VISIBILITY_IMPORT LIBXSMM_RETARGETABLE
-#   define LIBXSMM_API_INTERN LIBXSMM_API
-#   define LIBXSMM_API_EXPORT LIBXSMM_API
-#   define LIBXSMM_APIVAR_PUBLIC(DECL) LIBXSMM_VISIBILITY_IMPORT LIBXSMM_RETARGETABLE DECL; \
-                                       LIBXSMM_VISIBILITY_IMPORT LIBXSMM_RETARGETABLE DECL
-#   define LIBXSMM_APIVAR(DECL) LIBXSMM_EXTERN_C LIBXSMM_VISIBILITY_PRIVATE LIBXSMM_RETARGETABLE DECL; \
-                                LIBXSMM_EXTERN_C LIBXSMM_VISIBILITY_PRIVATE LIBXSMM_RETARGETABLE DECL
-#   define LIBXSMM_APIEXT LIBXSMM_API
+#   define LIBXSMM_API_VISIBILITY_EXPORT LIBXSMM_VISIBILITY_IMPORT
+#   define LIBXSMM_API_VISIBILITY_IMPORT LIBXSMM_VISIBILITY_IMPORT
+#   define LIBXSMM_API_VISIBILITY_INTERN
 # endif
+# define LIBXSMM_API_COMMON LIBXSMM_RETARGETABLE
+# define LIBXSMM_API_TARGET LIBXSMM_RETARGETABLE
+# define LIBXSMM_API_EXTERN LIBXSMM_EXTERN
 #endif
-#if !defined(_WIN32)
-# define LIBXSMM_APIVAR_PRIVATE(DECL) LIBXSMM_APIVAR(LIBXSMM_ALIGNED(DECL, LIBXSMM_CONFIG_CACHELINE))
-# define LIBXSMM_APIVAR_ALIGNED(DECL) LIBXSMM_APIVAR_PUBLIC(LIBXSMM_ALIGNED(DECL, LIBXSMM_CONFIG_CACHELINE))
-# define LIBXSMM_APIVAR_ARRAY(DECL, N) LIBXSMM_APIVAR(LIBXSMM_ALIGNED(DECL[N], LIBXSMM_CONFIG_CACHELINE))
-#else /* Windows */
-# define LIBXSMM_APIVAR_PRIVATE(DECL) LIBXSMM_APIVAR(DECL)
-# define LIBXSMM_APIVAR_ALIGNED(DECL) LIBXSMM_APIVAR_PUBLIC(DECL)
-# define LIBXSMM_APIVAR_ARRAY(DECL, N) LIBXSMM_APIVAR(DECL[N])
-#endif
-#define LIBXSMM_API_INLINE LIBXSMM_EXTERN_C LIBXSMM_INLINE LIBXSMM_RETARGETABLE
+
+#define LIBXSMM_API_VISIBILITY(VISIBILITY) LIBXSMM_CONCATENATE(LIBXSMM_API_VISIBILITY_, VISIBILITY)
+#define LIBXSMM_APIVAR(DECL, VISIBILITY, EXTERN) EXTERN LIBXSMM_API_COMMON LIBXSMM_API_VISIBILITY(VISIBILITY) DECL
+#define LIBXSMM_API_INLINE LIBXSMM_INLINE LIBXSMM_RETARGETABLE
+#define LIBXSMM_API_DEF
+
+/** Public visible variable declaration (without definition) located in header file. */
+#define LIBXSMM_APIVAR_PUBLIC(DECL) LIBXSMM_APIVAR(DECL, EXPORT, LIBXSMM_API_EXTERN)
+/** Public visible variable definition (complements declaration) located in source file. */
+#define LIBXSMM_APIVAR_PUBLIC_DEF(DECL) LIBXSMM_ALIGNED(LIBXSMM_APIVAR(DECL, EXPORT, LIBXSMM_API_DEF), LIBXSMM_CONFIG_CACHELINE)
+/** Private variable declaration (without definition) located in header file. */
+#define LIBXSMM_APIVAR_PRIVATE(DECL) LIBXSMM_APIVAR(DECL, INTERN, LIBXSMM_API_EXTERN)
+/** Private variable definition (complements declaration) located in source file. */
+#define LIBXSMM_APIVAR_PRIVATE_DEF(DECL) LIBXSMM_ALIGNED(LIBXSMM_APIVAR(DECL, INTERN, LIBXSMM_API_DEF), LIBXSMM_CONFIG_CACHELINE)
+/** Private variable (declaration and definition) located in source file. */
+#define LIBXSMM_APIVAR_DEFINE(DECL) LIBXSMM_APIVAR_PRIVATE(DECL); LIBXSMM_APIVAR_PRIVATE_DEF(DECL)
+/** Function decoration used for private functions. */
+#define LIBXSMM_API_INTERN LIBXSMM_API_EXTERN LIBXSMM_API_TARGET LIBXSMM_API_VISIBILITY(INTERN)
+/** Function decoration used for public functions of LIBXSMMext library. */
+#define LIBXSMM_APIEXT LIBXSMM_API_EXTERN LIBXSMM_API_TARGET LIBXSMM_API_VISIBILITY(IMPORT)
+/** Function decoration used for public functions of LIBXSMM library. */
+#define LIBXSMM_API LIBXSMM_API_EXTERN LIBXSMM_API_TARGET LIBXSMM_API_VISIBILITY(EXPORT)
 
 #if !defined(LIBXSMM_RESTRICT)
 # if ((defined(__GNUC__) && !defined(__CYGWIN32__)) || defined(LIBXSMM_INTEL_COMPILER)) && !defined(_WIN32)
@@ -306,7 +329,7 @@
 
 #if !defined(LIBXSMM_PRAGMA)
 # if defined(LIBXSMM_INTEL_COMPILER) || defined(_MSC_VER)
-#   define LIBXSMM_PRAGMA(DIRECTIVE) __pragma(DIRECTIVE)
+#   define LIBXSMM_PRAGMA(DIRECTIVE) __pragma(LIBXSMM_EXPAND(DIRECTIVE))
 # else
 #   define LIBXSMM_PRAGMA(DIRECTIVE)
 # endif
@@ -358,6 +381,17 @@
 # define LIBXSMM_PRAGMA_UNROLL LIBXSMM_PRAGMA(unroll)
 # define LIBXSMM_PRAGMA_VALIGNED_VAR(A) LIBXSMM_ASSUME_ALIGNED(A, LIBXSMM_ALIGNMENT);
 /*# define LIBXSMM_UNUSED(VARIABLE) LIBXSMM_PRAGMA(unused(VARIABLE))*/
+#elif defined(__clang__)
+# define LIBXSMM_PRAGMA_NONTEMPORAL_VARS(A, ...)
+# define LIBXSMM_PRAGMA_NONTEMPORAL
+# define LIBXSMM_PRAGMA_VALIGNED_VAR(A)
+# define LIBXSMM_PRAGMA_VALIGNED
+# define LIBXSMM_PRAGMA_NOVECTOR LIBXSMM_PRAGMA(clang loop vectorize(disable))
+# define LIBXSMM_PRAGMA_FORCEINLINE
+# define LIBXSMM_PRAGMA_LOOP_COUNT(MIN, MAX, AVG) LIBXSMM_PRAGMA(unroll(AVG))
+# define LIBXSMM_PRAGMA_UNROLL_AND_JAM(N) LIBXSMM_PRAGMA(unroll(N))
+# define LIBXSMM_PRAGMA_UNROLL_N(N) LIBXSMM_PRAGMA(unroll(N))
+# define LIBXSMM_PRAGMA_UNROLL LIBXSMM_PRAGMA_UNROLL_N(4)
 #else
 # define LIBXSMM_PRAGMA_NONTEMPORAL_VARS(A, ...)
 # define LIBXSMM_PRAGMA_NONTEMPORAL
@@ -420,14 +454,16 @@
 #define LIBXSMM_SIZEOF(START, LAST) (((const char*)(LAST)) - ((const char*)(START)) + sizeof(*LAST))
 #define LIBXSMM_FEQ(A, B) ((A) == (B))
 #define LIBXSMM_NEQ(A, B) ((A) != (B))
+#define LIBXSMM_ISWAP(A, B) (((A) ^= (B)), ((B) ^= (A)), ((A) ^= (B)))
 #define LIBXSMM_ISNAN(A)  LIBXSMM_NEQ(A, A)
 #define LIBXSMM_NOTNAN(A) LIBXSMM_FEQ(A, A)
 #define LIBXSMM_ROUNDX(TYPE, A) ((TYPE)((long long)(0 <= (A) ? ((double)(A) + 0.5) : ((double)(A) - 0.5))))
+#define LIBXSMM_CONST_VOID_PTR(A) *((const void**)&(A))
 
 /** Makes some functions available independent of C99 support. */
 #if defined(__STDC_VERSION__) && (199901L <= __STDC_VERSION__) /*C99*/
 # if defined(__PGI)
-#   define LIBXSMM_POWF(A, B) ((float)pow((double)(A), (double)(B)))
+#   define LIBXSMM_POWF(A, B) ((float)pow((float)(A), (float)(B)))
 # else
 #   define LIBXSMM_POWF(A, B) powf(A, B)
 # endif
@@ -443,19 +479,18 @@
 # define LIBXSMM_EXPF(A) expf(A)
 # define LIBXSMM_LOGF(A) logf(A)
 #else
-# define LIBXSMM_POWF(A, B) ((float)pow((double)(A), (double)(B)))
-# define LIBXSMM_FREXPF(A, B) ((float)frexp((double)(A), B))
+# define LIBXSMM_POWF(A, B) ((float)pow((float)(A), (float)(B)))
+# define LIBXSMM_FREXPF(A, B) ((float)frexp((float)(A), B))
 # define LIBXSMM_ROUNDF(A) LIBXSMM_ROUNDX(float, A)
 # define LIBXSMM_ROUND(A) LIBXSMM_ROUNDX(double, A)
-# define LIBXSMM_TANHF(A) ((float)tanh((double)(A)))
-# define LIBXSMM_SQRTF(A) ((float)sqrt((double)(A)))
+# define LIBXSMM_TANHF(A) ((float)tanh((float)(A)))
+# define LIBXSMM_SQRTF(A) ((float)sqrt((float)(A)))
 # define LIBXSMM_EXP2F(A) LIBXSMM_POWF(2, A)
-# define LIBXSMM_LOG2F(A) ((float)LIBXSMM_LOG2((double)(A)))
-# define LIBXSMM_EXP2(A) pow(2.0, (double)(A))
+# define LIBXSMM_LOG2F(A) ((float)LIBXSMM_LOG2((float)(A)))
+# define LIBXSMM_EXP2(A) pow(2.0, A)
 # define LIBXSMM_LOG2(A) (log(A) * (1.0 / (M_LN2)))
-# define LIBXSMM_EXPF(A) ((float)exp((double)(A)))
-# define LIBXSMM_LOGF(A) ((float)log((double)(A)))
-
+# define LIBXSMM_EXPF(A) ((float)exp((float)(A)))
+# define LIBXSMM_LOGF(A) ((float)log((float)(A)))
 #endif
 
 #if defined(LIBXSMM_INTEL_COMPILER)
@@ -472,7 +507,7 @@
 # define LIBXSMM_ASSUME(EXPRESSION) assert(EXPRESSION)
 #endif
 
-#if defined(LIBXSMM_INTEL_COMPILER)
+#if defined(__INTEL_COMPILER)
 # define LIBXSMM_ASSUME_ALIGNED(A, N) __assume_aligned(A, N)
 #else
 # define LIBXSMM_ASSUME_ALIGNED(A, N) assert(0 == ((uintptr_t)(A)) % (N))
@@ -503,9 +538,11 @@
  * To ultimately disable VLA-support, define LIBXSMM_NO_VLA (make VLA=0).
  * VLA-support is signaled by LIBXSMM_VLA.
  */
-#if !defined(LIBXSMM_VLA) && !defined(LIBXSMM_NO_VLA) && !defined(__PGI) && ((defined(__STDC_VERSION__) && (199901L/*C99*/ == __STDC_VERSION__ || \
-   (!defined(__STDC_NO_VLA__) && 199901L/*C99*/ < __STDC_VERSION__))) || (defined(__GNUC__) && !defined(__STRICT_ANSI__) && !defined(__cplusplus)) /*|| \
-    (defined(LIBXSMM_INTEL_COMPILER) && !defined(_WIN32) && !defined(__cplusplus)) || (defined(__INTEL_COMPILER) && !defined(_WIN32))*/)
+#if !defined(LIBXSMM_VLA) && !defined(LIBXSMM_NO_VLA) && !defined(__PGI) && ( \
+    (defined(__STDC_VERSION__) && (199901L/*C99*/ == __STDC_VERSION__ || (!defined(__STDC_NO_VLA__) && 199901L/*C99*/ < __STDC_VERSION__))) || \
+    (defined(__GNUC__) && LIBXSMM_VERSION2(5, 0) <= LIBXSMM_VERSION2(__GNUC__, __GNUC_MINOR__) && !defined(__STRICT_ANSI__) && !defined(__cplusplus)) || \
+    (defined(LIBXSMM_INTEL_COMPILER) && !defined(_WIN32) && !defined(__cplusplus)) || \
+    (defined(__INTEL_COMPILER) && !defined(_WIN32)))
 # define LIBXSMM_VLA
 #endif
 
@@ -543,24 +580,25 @@
 # define LIBXSMM_VLA_POSTFIX _
 #endif
 #if defined(LIBXSMM_VLA)
-# define LIBXSMM_VLA_ACCESS(NDIMS, ARRAY, ...) LIBXSMM_VLA_ACCESS_Z(NDIMS, LIBXSMM_CONCATENATE(ARRAY, LIBXSMM_VLA_POSTFIX), LIBXSMM_VLA_ACCESS_X, __VA_ARGS__)
-# define LIBXSMM_VLA_ACCESS_X(S) + 0 * (S)
-# define LIBXSMM_VLA_ACCESS_Y(...)
-# define LIBXSMM_VLA_ACCESS_Z(NDIMS, ARRAY, XY, ...) LIBXSMM_CONCATENATE(LIBXSMM_VLA_ACCESS_, NDIMS)(ARRAY, XY, __VA_ARGS__)
-# define LIBXSMM_VLA_ACCESS_0(ARRAY, XY, ...) (ARRAY)/*scalar*/
-# define LIBXSMM_VLA_ACCESS_1(ARRAY, XY, ...) ((ARRAY)[LIBXSMM_SELECT_HEAD(__VA_ARGS__)])
-# define LIBXSMM_VLA_ACCESS_2(ARRAY, XY, I0, I1, ...) (((ARRAY) XY(__VA_ARGS__))[I0][I1])
-# define LIBXSMM_VLA_ACCESS_3(ARRAY, XY, I0, I1, I2, S1, ...) (((ARRAY) XY(S1) XY(__VA_ARGS__))[I0][I1][I2])
-# define LIBXSMM_VLA_ACCESS_4(ARRAY, XY, I0, I1, I2, I3, S1, S2, ...) (((ARRAY) XY(S1) XY(S2) XY(__VA_ARGS__))[I0][I1][I2][I3])
-# define LIBXSMM_VLA_ACCESS_5(ARRAY, XY, I0, I1, I2, I3, I4, S1, S2, S3, ...) (((ARRAY) XY(S1) XY(S2) XY(S3) XY(__VA_ARGS__))[I0][I1][I2][I3][I4])
-# define LIBXSMM_VLA_ACCESS_6(ARRAY, XY, I0, I1, I2, I3, I4, I5, S1, S2, S3, S4, ...) (((ARRAY) XY(S1) XY(S2) XY(S3) XY(S4) XY(__VA_ARGS__))[I0][I1][I2][I3][I4][I5])
-# define LIBXSMM_VLA_ACCESS_7(ARRAY, XY, I0, I1, I2, I3, I4, I5, I6, S1, S2, S3, S4, S5, ...) (((ARRAY) XY(S1) XY(S2) XY(S3) XY(S4) XY(S5) XY(__VA_ARGS__))[I0][I1][I2][I3][I4][I5][I6])
-# define LIBXSMM_VLA_ACCESS_8(ARRAY, XY, I0, I1, I2, I3, I4, I5, I6, I7, S1, S2, S3, S4, S5, S6, ...) (((ARRAY) XY(S1) XY(S2) XY(S3) XY(S4) XY(S5) XY(S6) XY(__VA_ARGS__))[I0][I1][I2][I3][I4][I5][I6][I7])
+LIBXSMM_API_INLINE int libxsmm_nonconst_int(int i) { return i; }
+# define LIBXSMM_VLA_ACCESS(NDIMS, ARRAY, ...) LIBXSMM_VLA_ACCESS_ND(NDIMS, LIBXSMM_CONCATENATE(ARRAY, LIBXSMM_VLA_POSTFIX), LIBXSMM_VLA_ACCESS_SINK, __VA_ARGS__)
+# define LIBXSMM_VLA_ACCESS_SINK(S) + 0 * (S)
+# define LIBXSMM_VLA_ACCESS_NONCONST(I) libxsmm_nonconst_int(I)
+# define LIBXSMM_VLA_ACCESS_ND(NDIMS, ARRAY, XY, ...) LIBXSMM_CONCATENATE3(LIBXSMM_VLA_ACCESS_, NDIMS, D)(ARRAY, XY, __VA_ARGS__)
+# define LIBXSMM_VLA_ACCESS_0D(ARRAY, XY, ...) (ARRAY)/*scalar*/
+# define LIBXSMM_VLA_ACCESS_1D(ARRAY, XY, ...) ((ARRAY)[LIBXSMM_VLA_ACCESS_NONCONST(LIBXSMM_SELECT_HEAD(__VA_ARGS__))])
+# define LIBXSMM_VLA_ACCESS_2D(ARRAY, XY, I0, I1, ...) (((ARRAY) XY(__VA_ARGS__))[I0][LIBXSMM_VLA_ACCESS_NONCONST(I1)])
+# define LIBXSMM_VLA_ACCESS_3D(ARRAY, XY, I0, I1, I2, S1, ...) (((ARRAY) XY(S1) XY(__VA_ARGS__))[I0][I1][LIBXSMM_VLA_ACCESS_NONCONST(I2)])
+# define LIBXSMM_VLA_ACCESS_4D(ARRAY, XY, I0, I1, I2, I3, S1, S2, ...) (((ARRAY) XY(S1) XY(S2) XY(__VA_ARGS__))[I0][I1][I2][LIBXSMM_VLA_ACCESS_NONCONST(I3)])
+# define LIBXSMM_VLA_ACCESS_5D(ARRAY, XY, I0, I1, I2, I3, I4, S1, S2, S3, ...) (((ARRAY) XY(S1) XY(S2) XY(S3) XY(__VA_ARGS__))[I0][I1][I2][I3][LIBXSMM_VLA_ACCESS_NONCONST(I4)])
+# define LIBXSMM_VLA_ACCESS_6D(ARRAY, XY, I0, I1, I2, I3, I4, I5, S1, S2, S3, S4, ...) (((ARRAY) XY(S1) XY(S2) XY(S3) XY(S4) XY(__VA_ARGS__))[I0][I1][I2][I3][I4][LIBXSMM_VLA_ACCESS_NONCONST(I5)])
+# define LIBXSMM_VLA_ACCESS_7D(ARRAY, XY, I0, I1, I2, I3, I4, I5, I6, S1, S2, S3, S4, S5, ...) (((ARRAY) XY(S1) XY(S2) XY(S3) XY(S4) XY(S5) XY(__VA_ARGS__))[I0][I1][I2][I3][I4][I5][LIBXSMM_VLA_ACCESS_NONCONST(I6)])
+# define LIBXSMM_VLA_ACCESS_8D(ARRAY, XY, I0, I1, I2, I3, I4, I5, I6, I7, S1, S2, S3, S4, S5, S6, ...) (((ARRAY) XY(S1) XY(S2) XY(S3) XY(S4) XY(S5) XY(S6) XY(__VA_ARGS__))[I0][I1][I2][I3][I4][I5][I6][LIBXSMM_VLA_ACCESS_NONCONST(I7)])
 # define LIBXSMM_VLA_DECL(NDIMS, ELEMENT_TYPE, ARRAY_VAR, .../*initial value, and bounds*/) \
-    ELEMENT_TYPE LIBXSMM_VLA_ACCESS_Z(LIBXSMM_SELECT_ELEMENT(NDIMS, 0, 1, 2, 3, 4, 5, 6, 7), *LIBXSMM_RESTRICT LIBXSMM_CONCATENATE(ARRAY_VAR, LIBXSMM_VLA_POSTFIX), \
-      LIBXSMM_VLA_ACCESS_Y, LIBXSMM_SELECT_TAIL(__VA_ARGS__, 0)/*bounds*/, LIBXSMM_SELECT_TAIL(__VA_ARGS__, 0)/*dummy*/) = \
-   (ELEMENT_TYPE LIBXSMM_VLA_ACCESS_Z(LIBXSMM_SELECT_ELEMENT(NDIMS, 0, 1, 2, 3, 4, 5, 6, 7), *, \
-      LIBXSMM_VLA_ACCESS_Y, LIBXSMM_SELECT_TAIL(__VA_ARGS__, 0)/*bounds*/, LIBXSMM_SELECT_TAIL(__VA_ARGS__, 0)/*dummy*/))LIBXSMM_SELECT_HEAD(__VA_ARGS__)
+    ELEMENT_TYPE LIBXSMM_VLA_ACCESS_ND(LIBXSMM_SELECT_ELEMENT(NDIMS, 0, 1, 2, 3, 4, 5, 6, 7), *LIBXSMM_RESTRICT LIBXSMM_CONCATENATE(ARRAY_VAR, LIBXSMM_VLA_POSTFIX), \
+      LIBXSMM_ELIDE, LIBXSMM_SELECT_TAIL(__VA_ARGS__, 0)/*bounds*/, LIBXSMM_SELECT_TAIL(__VA_ARGS__, 0)/*dummy*/) = \
+   (ELEMENT_TYPE LIBXSMM_VLA_ACCESS_ND(LIBXSMM_SELECT_ELEMENT(NDIMS, 0, 1, 2, 3, 4, 5, 6, 7), *, \
+      LIBXSMM_ELIDE, LIBXSMM_SELECT_TAIL(__VA_ARGS__, 0)/*bounds*/, LIBXSMM_SELECT_TAIL(__VA_ARGS__, 0)/*dummy*/))LIBXSMM_SELECT_HEAD(__VA_ARGS__)
 #else /* calculate linear index */
 # define LIBXSMM_VLA_ACCESS(NDIMS, ARRAY, ...) LIBXSMM_CONCATENATE(ARRAY, LIBXSMM_VLA_POSTFIX)[LIBXSMM_INDEX1(NDIMS, __VA_ARGS__)]
 # define LIBXSMM_VLA_DECL(NDIMS, ELEMENT_TYPE, ARRAY_VAR, .../*initial value, and bounds*/) \
@@ -584,13 +622,13 @@
 #else
 # define LIBXSMM_PRAGMA_OMP(...)
 #endif
-#if defined(_OPENMP) && defined(_MSC_VER) && !defined(__clang__) && !defined(LIBXSMM_INTEL_COMPILER)
+#if defined(_OPENMP) && defined(_MSC_VER) && !defined(LIBXSMM_INTEL_COMPILER)
 # define LIBXSMM_OMP_VAR(A) LIBXSMM_UNUSED(A) /* suppress warning about "unused" variable */
 #else
 # define LIBXSMM_OMP_VAR(A)
 #endif
 
-#if (defined(__GNUC__) || defined(__clang__)) && !defined(__CYGWIN__) && !defined(__MINGW32__)
+#if defined(LIBXSMM_BUILD) && (defined(__GNUC__) || defined(__clang__)) && !defined(__CYGWIN__) && !defined(__MINGW32__)
 # define LIBXSMM_ATTRIBUTE_WEAK_IMPORT LIBXSMM_ATTRIBUTE(weak_import)
 # define LIBXSMM_ATTRIBUTE_WEAK LIBXSMM_ATTRIBUTE(weak)
 #else
@@ -598,26 +636,24 @@
 # define LIBXSMM_ATTRIBUTE_WEAK_IMPORT
 #endif
 
-#if !defined(LIBXSMM_NO_CTOR) && defined(__GNUC__) && !defined(LIBXSMM_CTOR)
+#if !defined(LIBXSMM_NO_CTOR) && !defined(LIBXSMM_CTOR) && \
+    (defined(__STDC_VERSION__) && (199901L <= __STDC_VERSION__)) && \
+    (defined(LIBXSMM_BUILD) && !defined(__STATIC)) && \
+    (defined(__GNUC__) || defined(__clang__))
 # define LIBXSMM_ATTRIBUTE_CTOR LIBXSMM_ATTRIBUTE(constructor)
 # define LIBXSMM_ATTRIBUTE_DTOR LIBXSMM_ATTRIBUTE(destructor)
-# if defined(LIBXSMM_BUILD) && !defined(__STATIC)
-#   define LIBXSMM_CTOR
-# endif
+# define LIBXSMM_CTOR
 #else
 # define LIBXSMM_ATTRIBUTE_CTOR
 # define LIBXSMM_ATTRIBUTE_DTOR
 #endif
 
-#if defined(__GNUC__) || (defined(LIBXSMM_INTEL_COMPILER) && !defined(_WIN32))
-# define LIBXSMM_ATTRIBUTE_MALLOC LIBXSMM_ATTRIBUTE(malloc)
-# define LIBXSMM_ATTRIBUTE_UNUSED LIBXSMM_ATTRIBUTE(unused)
-# define LIBXSMM_ATTRIBUTE_USED LIBXSMM_ATTRIBUTE(used)
+#if defined(__GNUC__) && !defined(__PGI) && !defined(__ibmxl__)
+# define LIBXSMM_ATTRIBUTE_NO_TRACE LIBXSMM_ATTRIBUTE(no_instrument_function)
 #else
-# define LIBXSMM_ATTRIBUTE_MALLOC
-# define LIBXSMM_ATTRIBUTE_UNUSED
-# define LIBXSMM_ATTRIBUTE_USED
+# define LIBXSMM_ATTRIBUTE_NO_TRACE
 #endif
+
 #if defined(__GNUC__)
 # define LIBXSMM_MAY_ALIAS LIBXSMM_ATTRIBUTE(__may_alias__)
 #else
@@ -627,31 +663,6 @@
 #if !defined(LIBXSMM_MKTEMP_PATTERN)
 # define LIBXSMM_MKTEMP_PATTERN "XXXXXX"
 #endif
-#if defined(_WIN32) && 0
-# define LIBXSMM_SNPRINTF(S, N, ...) _snprintf_s(S, N, _TRUNCATE, __VA_ARGS__)
-# define setenv(NAME, VALUE, OVERWRITE) _putenv(NAME "=" VALUE)
-#elif defined(__STDC_VERSION__) && (199901L <= __STDC_VERSION__ || defined(__GNUC__))
-# define LIBXSMM_SNPRINTF(S, N, ...) snprintf(S, N, __VA_ARGS__)
-#else
-# define LIBXSMM_SNPRINTF(S, N, ...) sprintf((S) + /*unused*/(N) * 0, __VA_ARGS__)
-#endif
-#if (0 == LIBXSMM_SYNC)
-# define LIBXSMM_FLOCK(FILE)
-# define LIBXSMM_FUNLOCK(FILE)
-#elif defined(_WIN32)
-# define LIBXSMM_FLOCK(FILE) _lock_file(FILE)
-# define LIBXSMM_FUNLOCK(FILE) _unlock_file(FILE)
-#elif !defined(__CYGWIN__)
-# define LIBXSMM_FLOCK(FILE) flockfile(FILE)
-# define LIBXSMM_FUNLOCK(FILE) funlockfile(FILE)
-#else /* Only available with __CYGWIN__ *and* C++0x. */
-# define LIBXSMM_FLOCK(FILE)
-# define LIBXSMM_FUNLOCK(FILE)
-#endif
-
-/** Synchronize console output */
-#define LIBXSMM_STDIO_ACQUIRE() LIBXSMM_FLOCK(stdout); LIBXSMM_FLOCK(stderr)
-#define LIBXSMM_STDIO_RELEASE() LIBXSMM_FUNLOCK(stderr); LIBXSMM_FUNLOCK(stdout)
 
 /** Below group is to fix-up some platform/compiler specifics. */
 #if defined(_WIN32)
@@ -682,7 +693,7 @@
 #   endif
 # endif
 #endif
-#if !defined(_GNU_SOURCE) /*&& defined(__GNUC__)*/
+#if !defined(_GNU_SOURCE) && 0
 # define _GNU_SOURCE
 #endif
 #if !defined(__STDC_FORMAT_MACROS)
@@ -695,8 +706,16 @@
 # define inline LIBXSMM_INLINE_KEYWORD
 #endif
 
-#if (0 != LIBXSMM_SYNC) && !defined(_REENTRANT)
-# define _REENTRANT
+#if (0 != LIBXSMM_SYNC)
+# if !defined(_REENTRANT)
+#   define _REENTRANT
+# endif
+# if defined(__PGI)
+#   if defined(__GCC_ATOMIC_TEST_AND_SET_TRUEVAL)
+#     undef __GCC_ATOMIC_TEST_AND_SET_TRUEVAL
+#   endif
+#   define __GCC_ATOMIC_TEST_AND_SET_TRUEVAL 1
+# endif
 #endif
 
 #if !defined(__has_feature) && !defined(__clang__)
@@ -709,6 +728,14 @@
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
 #endif
+
+#if (0 != LIBXSMM_SYNC)
+# if defined(_WIN32) || defined(__CYGWIN__)
+#   include <windows.h>
+# else
+#   include <pthread.h>
+# endif
+#endif
 #if !defined(LIBXSMM_ASSERT)
 # include <assert.h>
 # if defined(NDEBUG)
@@ -720,49 +747,73 @@
 #if !defined(LIBXSMM_ASSERT_MSG)
 # define LIBXSMM_ASSERT_MSG(EXPR, MSG) assert((EXPR) && (0 != *(MSG)))
 #endif
-#if !defined(LIBXSMM_EXPECT)
-# if defined(NDEBUG)
-#   define LIBXSMM_EXPECT(RESULT, EXPR) (EXPR)
-# else
-#   define LIBXSMM_EXPECT(RESULT, EXPR) LIBXSMM_ASSERT((RESULT) == (EXPR))
-# endif
+#if !defined(LIBXSMM_EXPECT_ELIDE)
+# define LIBXSMM_EXPECT_ELIDE(RESULT, EXPR) do { \
+    /*const*/ int libxsmm_expect_result_ = (EXPR); \
+    LIBXSMM_UNUSED(libxsmm_expect_result_); \
+  } while(0)
 #endif
-#if !defined(LIBXSMM_EXPECT_NOT)
-# if defined(NDEBUG)
-#   define LIBXSMM_EXPECT_NOT(RESULT, EXPR) (EXPR)
-# else
-#   define LIBXSMM_EXPECT_NOT(RESULT, EXPR) LIBXSMM_ASSERT((RESULT) != (EXPR))
-# endif
+#if defined(NDEBUG)
+# define LIBXSMM_EXPECT LIBXSMM_EXPECT_ELIDE
+# define LIBXSMM_EXPECT_NOT LIBXSMM_EXPECT_ELIDE
+#else
+# define LIBXSMM_EXPECT(RESULT, EXPR) LIBXSMM_ASSERT((RESULT) == (EXPR))
+# define LIBXSMM_EXPECT_NOT(RESULT, EXPR) LIBXSMM_ASSERT((RESULT) != (EXPR))
 #endif
-#if defined(LIBXSMM_GLIBC_FPTYPES)
-# if defined(__cplusplus)
-#   undef __USE_MISC
-#   include <math.h>
-#   if !defined(_DEFAULT_SOURCE)
-#     define _DEFAULT_SOURCE
-#   endif
-#   if !defined(_BSD_SOURCE)
-#     define _BSD_SOURCE
-#   endif
-# elif !defined(__PURE_INTEL_C99_HEADERS__)
-#   define __PURE_INTEL_C99_HEADERS__
-# endif
+#if defined(_DEBUG)
+# define LIBXSMM_EXPECT_DEBUG LIBXSMM_EXPECT
+#else
+# define LIBXSMM_EXPECT_DEBUG LIBXSMM_EXPECT_ELIDE
 #endif
-#include <stddef.h>
+#if defined(_OPENMP) && defined(LIBXSMM_SYNC_OMP)
+# include <omp.h>
+#endif
+#include <inttypes.h>
 #include <stdint.h>
+#include <stddef.h>
+#include <stdlib.h>
+#include <string.h>
+#include <stdio.h>
+
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(pop)
 #endif
 
+#if defined(_WIN32) && 0
+# define LIBXSMM_SNPRINTF(S, N, ...) _snprintf_s(S, N, _TRUNCATE, __VA_ARGS__)
+#elif defined(__STDC_VERSION__) && (199901L <= __STDC_VERSION__ || defined(__GNUC__))
+# define LIBXSMM_SNPRINTF(S, N, ...) snprintf(S, N, __VA_ARGS__)
+#else
+# define LIBXSMM_SNPRINTF(S, N, ...) sprintf((S) + /*unused*/(N) * 0, __VA_ARGS__)
+#endif
+#if defined(__THROW) && defined(__cplusplus)
+# define LIBXSMM_THROW __THROW
+#endif
+#if !defined(LIBXSMM_THROW)
+# define LIBXSMM_THROW
+#endif
+#if defined(__GNUC__) && LIBXSMM_VERSION2(4, 2) == LIBXSMM_VERSION2(__GNUC__, __GNUC_MINOR__) && \
+  !defined(__clang__) && !defined(__PGI) && !defined(__INTEL_COMPILER) && !defined(_CRAYC)
+# define LIBXSMM_NOTHROW LIBXSMM_THROW
+#else
+# define LIBXSMM_NOTHROW
+#endif
+#if defined(_WIN32)
+# define LIBXSMM_PUTENV(A) _putenv(A)
+#else
+# define LIBXSMM_PUTENV(A) putenv(A)
+#endif
+
 /* block must be after including above header files */
-#if defined(__GLIBC__) && defined(__GLIBC_MINOR__) && LIBXSMM_VERSION2(__GLIBC__, __GLIBC_MINOR__) < LIBXSMM_VERSION2(2, 26)
+#if (defined(__GLIBC__) && defined(__GLIBC_MINOR__) && LIBXSMM_VERSION2(__GLIBC__, __GLIBC_MINOR__) < LIBXSMM_VERSION2(2, 26)) \
+  || (defined(LIBXSMM_INTEL_COMPILER) && (1802 >= LIBXSMM_INTEL_COMPILER) && !defined(__cplusplus) && defined(__linux__))
 /* _Float128 was introduced with GNU GCC 7.0. */
 # if !defined(_Float128) && !defined(__SIZEOF_FLOAT128__) && defined(__GNUC__) && !defined(__cplusplus) && defined(__linux__)
 #   define _Float128 __float128
 # endif
 # if !defined(LIBXSMM_GLIBC_FPTYPES) && defined(__GNUC__) && !defined(__cplusplus) && defined(__linux__) \
-  && (LIBXSMM_VERSION3(7, 0, 0) > LIBXSMM_VERSION3(__GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__) \
-  || defined(LIBXSMM_INTEL_COMPILER)) && 0 /* TODO */
+  && (LIBXSMM_VERSION3(7, 0, 0) > LIBXSMM_VERSION3(__GNUC__, __GNUC_MINOR__, __GNUC_PATCHLEVEL__) || \
+     (defined(LIBXSMM_INTEL_COMPILER) && (1802 >= LIBXSMM_INTEL_COMPILER)))
 #   define LIBXSMM_GLIBC_FPTYPES
 # endif
 # if !defined(_Float128X) && defined(LIBXSMM_GLIBC_FPTYPES)
@@ -782,5 +833,44 @@
 # endif
 #endif
 
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
+#endif
+#if defined(LIBXSMM_GLIBC_FPTYPES)
+# if defined(__cplusplus)
+#   undef __USE_MISC
+#   if !defined(_DEFAULT_SOURCE)
+#     define _DEFAULT_SOURCE
+#   endif
+#   if !defined(_BSD_SOURCE)
+#     define _BSD_SOURCE
+#   endif
+# else
+#   if !defined(__PURE_INTEL_C99_HEADERS__)
+#     define __PURE_INTEL_C99_HEADERS__
+#   endif
+# endif
+#endif
+#if !defined(LIBXSMM_NO_LIBM) && (!defined(__STDC_VERSION__) || (199901L > __STDC_VERSION__) || defined(__cplusplus))
+# if defined(LIBXSMM_INTEL_COMPILER)
+#   include <mathimf.h>
+# endif
+# include <math.h>
+#endif
+#if !defined(M_LN2)
+# define M_LN2 0.69314718055994530942
+# if !defined(__cplusplus)
+LIBXSMM_EXTERN double pow(double, double);
+LIBXSMM_EXTERN double frexp(double, int*);
+LIBXSMM_EXTERN double sqrt(double);
+LIBXSMM_EXTERN double tanh(double);
+LIBXSMM_EXTERN double exp(double);
+# endif
+
+#endif
+#if defined(LIBXSMM_OFFLOAD_TARGET)
+# pragma offload_attribute(pop)
+#endif
+
 #endif /*LIBXSMM_MACROS_H*/
 
