@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2014-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Hans Pabst (Intel Corp.)
 ******************************************************************************/
@@ -33,6 +13,10 @@
 
 #include "libxsmm_memory.h"
 
+/** Can be used with libxsmm_[get|set]_scratch_limit. */
+#define LIBXSMM_SCRATCH_UNLIMITED ((size_t)LIBXSMM_UNLIMITED)
+#define LIBXSMM_SCRATCH_DEFAULT 0
+
 
 /** Function types accepted for memory allocation (see libxsmm_*_allocator). */
 LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void* (*libxsmm_malloc_ctx)(size_t /*size*/, const void* /*context*/);
@@ -53,9 +37,10 @@ LIBXSMM_EXTERN_C typedef union LIBXSMM_RETARGETABLE libxsmm_free_function {
 /**
  * To setup the custom default memory allocator, either a malloc_fn and a free_fn
  * are given, or two NULL-pointers designate to reset the default allocator to a
- * library-internal default. If a context is given (non-NULL ), the context-based
+ * library-internal default. If a context is given (non-NULL), the context-based
  * form of the memory allocation is used.
- * It is supported to change the allocator while buffers are pending.
+ * Changing the allocator including the function for deallocation applies to
+ * upcoming allocation/deallocation and works correctly for pending buffers.
  */
 LIBXSMM_API int libxsmm_set_default_allocator(/* malloc_fn/free_fn must correspond */
   const void* context, libxsmm_malloc_function malloc_fn, libxsmm_free_function free_fn);
@@ -69,7 +54,8 @@ LIBXSMM_API int libxsmm_get_default_allocator(const void** context,
  * to be controlled otherwise. If two NULL-pointers are given, the allocator is reset
  * to the currently active default memory allocator. If a context is given (non-NULL),
  * the context-based form of the memory allocation is used.
- * It is supported to change the allocator while buffers are pending.
+ * Changing the allocator including the function for deallocation applies to
+ * upcoming allocation/deallocation and works correctly for pending buffers.
  */
 LIBXSMM_API int libxsmm_set_scratch_allocator(/* malloc_fn/free_fn must correspond */
   const void* context, libxsmm_malloc_function malloc_fn, libxsmm_free_function free_fn);
@@ -152,13 +138,29 @@ LIBXSMM_API int libxsmm_get_scratch_info(libxsmm_scratch_info* info);
 
 /**
  * Limit the total size (Bytes) of the scratch memory.
- * The environment variable LIBXSMM_SCRATCH_LIMIT takes
- * the following units: none (Bytes), k/K, m/M, and g/G.
+ * LIBXSMM_SCRATCH_UNLIMITED removes any limit, and
+ * LIBXSMM_SCRATCH_DEFAULT populates the default.
+ * The related environment variable LIBXSMM_SCRATCH_LIMIT
+ * allows units: <none>/b/B (Bytes), k/K, m/M, and g/G.
  */
 LIBXSMM_API void libxsmm_set_scratch_limit(size_t nbytes);
 /** Get the maximum size of the scratch memory domain. */
 LIBXSMM_API size_t libxsmm_get_scratch_limit(void);
 
+/**
+ * Intercepts malloc/free to use scratch memory allocator.
+ * (related environment variable LIBXSMM_MALLOC).
+ * Optionally set the range of malloc-sizes to be intercepted.
+ * The related environment variable LIBXSMM_MALLOC_LIMIT
+ * allows units: <none>/b/B (Bytes), k/K, m/M, and g/G.
+ */
+LIBXSMM_API void libxsmm_set_malloc(int enabled, const size_t* lo, const size_t* hi);
+/**
+ * Determines if malloc/free are (and can be) intercepted.
+ * Optionally gets the range of enabled malloc-sizes.
+ */
+LIBXSMM_API int libxsmm_get_malloc(size_t* lo, size_t* hi);
+
 /**
  * Calculate the linear offset of the n-dimensional (ndims) offset (can be NULL),
  * and the (optional) linear size of the corresponding shape.
