@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2017-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Hans Pabst (Intel Corp.)
 ******************************************************************************/
@@ -124,5 +104,35 @@ LIBXSMM_API float libxsmm_sexp2_i8(signed char x);
 /** Similar to libxsmm_sexp2_i8, but takes an integer as signed 8-bit value (check). */
 LIBXSMM_API float libxsmm_sexp2_i8i(int x);
 
+/** Inlineable fast tanh, such that a the compiler can potentially vectorize. */
+LIBXSMM_API_INLINE float libxsmm_stanh_pade78( float i_x ) {
+  const float l_c0       = 2027025.0f;
+  const float l_c1       = 270270.0f;
+  const float l_c2       = 6930.0f;
+  const float l_c3       = 36.0f;
+  const float l_c1_d     = 945945.0f;
+  const float l_c2_d     = 51975.0f;
+  const float l_c3_d     = 630.0f;
+  const float l_hi_bound = 4.97f;
+  const float l_lo_bound = -4.97f;
+  const float l_ones     = 1.0f;
+  const float l_neg_ones = -1.0f;
+
+  const float x2         = i_x * i_x;
+  const float t1_nom     = (l_c3 * x2) + l_c2;
+  const float t2_nom     = (t1_nom * x2) + l_c1;
+  const float t3_nom     = (t2_nom * x2) + l_c0;
+  const float nom        = t3_nom * i_x;
+  const float t1_denom   = x2 + l_c3_d;
+  const float t2_denom   = (t1_denom * x2) + l_c2_d;
+  const float t3_denom   = (t2_denom * x2) + l_c1_d;
+  const float denom      = (t3_denom * x2) + l_c0;
+  float result           = nom/denom ;
+  result = ( result > l_hi_bound ) ? l_ones : result;
+  result = ( result < l_lo_bound ) ? l_neg_ones : result;
+
+  return result;
+}
+
 #endif /*LIBXSMM_MATH_H*/
 
