@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2018-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Kunal Banerjee (Intel Corp.)
 ******************************************************************************/
@@ -45,858 +25,18 @@
 # pragma offload_attribute(pop)
 #endif
 
-#define CHKERR_LIBXSMM_DNN(A) if ( A != LIBXSMM_DNN_SUCCESS ) fprintf(stderr, "%s\n", libxsmm_dnn_get_error(A) );
+/* include c-based dnn library */
+#include "../common/dnn_common.h"
+
+#define CHKERR_LIBXSMM_DNN(A) { const int chkerr_libxsmm_dnn_ = A; if (LIBXSMM_DNN_SUCCESS != chkerr_libxsmm_dnn_) { \
+  fprintf(stderr, "%s\n", libxsmm_dnn_get_error(chkerr_libxsmm_dnn_)); global_status = chkerr_libxsmm_dnn_; } \
+}
 
-#if 0
-#define TWO_GEMMS
-#endif
-#if 0
-#define PROFILE
-#endif
 #if defined(PROFILE)
 unsigned long long Gbl_blas_start, Gbl_blas_end, Gbl_eltwise_start, Gbl_eltwise_end, Gbl_conv_start, Gbl_conv_end, Gbl_copy_bias_start, Gbl_copy_bias_end;
 double Gbl_blas_total, Gbl_eltwise_total, Gbl_conv_total, Gbl_copy_bias_total;
 #endif
 
-LIBXSMM_INLINE void zero_buf(float* buf, size_t size) {
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < (int)size; ++i) {
-    buf[i] = 0.0f;
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_add(int size, float *a, float *b, float *c)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < size; i++) {
-    c[i] = a[i] + b[i];
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_eltwise_mult(int size, float *a, float *b, float *c)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < size; i++) {
-    c[i] = a[i] * b[i];
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_eltwise_mult_ld_a(int m, int n, int ld, float *a, float *b, float *c)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < m*n; i++) {
-    int row = i / m;
-    int col = i % m;
-    c[i] = a[row*ld + col] * b[i];
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_eltwise_mult_ld_ab(int m, int n, int ld, float *a, float *b, float *c)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < m*n; i++) {
-    int row = i / m;
-    int col = i % m;
-    c[i] = a[row*ld + col] * b[row*ld + col];
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_eltwise_mult_ld_c(int m, int n, int ld, float *a, float *b, float *c)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < m*n; i++) {
-    int row = i / m;
-    int col = i % m;
-    c[row*ld + col] = a[i] * b[i];
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_sigmoid(int size, float *src, float *dst)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < size; i++) {
-    const float exp_value = (float)exp((double) -src[i]);
-    dst[i] = 1.0f / (1.0f + exp_value);
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_sigmoid_ld(int m, int n, int ld, float *src, float *dst)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < m*n; i++) {
-    int row = i / m;
-    int col = i % m;
-    const float exp_value = (float)exp((double) -src[row*ld + col]);
-    dst[row*ld + col] = 1.0f / (1.0f + exp_value);
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_tanh(int size, float *src, float *dst)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < size; i++) {
-    dst[i] = (float)tanh((double)src[i]);
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_tanh_ld(int m, int n, int ld, float *src, float *dst)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < m*n; i++) {
-    int row = i / m;
-    int col = i % m;
-    dst[row*ld + col] = (float)tanh((double)src[row*ld + col]);
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_relu(int size, float *src, float *dst)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < size; i++) {
-    dst[i] = (src[i] > 0.0f) ? src[i] : 0.0f;
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_sigmoid_inverse(int size, float *src, float *dst)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < size; i++) {
-    const float exp_value = (float)exp((double) -src[i]);
-    const float sig_exp = 1.0f / (1.0f + exp_value);
-    dst[i] = (1.0f - sig_exp)*sig_exp;
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_tanh_inverse(int size, float *src, float *dst)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < size; i++) {
-    const float tanh_value = (float)tanh((double)src[i]);
-    dst[i] = 1.0f - (tanh_value * tanh_value);
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_relu_inverse(int size, float *src, float *dst)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < size; i++) {
-    dst[i] = (src[i] > 0.0f) ? 1.0f : 0.0f;
-  }
-}
-
-LIBXSMM_INLINE void matrix_transpose(int rows, int cols, float *src, float *dst)
-{
-  libxsmm_otrans_omp(dst, src, sizeof(float), cols, rows, cols/*ldi*/, rows/*ldo*/);
-}
-
-
-LIBXSMM_INLINE void matrix_copy(int size, float *src, float *dst)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < size; i++) {
-    dst[i] = src[i];
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_copy_ld(int m, int n, int ld, float *src, float *dst)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < m*n; i++) {
-    int row = i / m;
-    int col = i % m;
-    dst[i] = src[row*ld + col];
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_copy_bias(int m, int n, int ld, float *src, float *dst)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < m*n; i++) {
-    int row = i / m;
-    int col = i % m;
-    dst[row*ld + col] = src[col];
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_copy_forget_bias(int m, int n, int ld, float *src, float *dst, float forget_bias)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < m*n; i++) {
-    int row = i / m;
-    int col = i % m;
-    dst[row*ld + col] = src[col] + forget_bias;
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_complement(int size, float *src, float *dst)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < size; i++) {
-    dst[i] = 1.0f - src[i];
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_complement_ld(int m, int n, int ld, float *src, float *dst)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < m*n; i++) {
-    int row = i / m;
-    int col = i % m;
-    dst[i] = 1.0f - src[row*ld + col];
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_complement_square(int size, float *src, float *dst)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < size; i++) {
-    dst[i] = 1.0f - (src[i] * src[i]);
-  }
-}
-
-
-LIBXSMM_INLINE void matrix_complement_square_ld(int m, int n, int ld, float *src, float *dst)
-{
-  int i;
-#if defined(_OPENMP)
-# pragma omp parallel for private(i)
-#endif
-  for (i = 0; i < m*n; i++) {
-    int row = i / m;
-    int col = i % m;
-    dst[i] = 1.0f - (src[row*ld + col] * src[row*ld + col]);
-  }
-}
-
-
-LIBXSMM_INLINE void convert_ck_c4k(int C, int K, float *src, float *dst)
-{
-  int x, y;
-#if defined(_OPENMP)
-  LIBXSMM_OMP_VAR(x); LIBXSMM_OMP_VAR(y);
-# pragma omp parallel for private(x, y)
-#endif
-  for (y = 0; y < C; y++) {
-    for (x = 0; x < K; x++) {
-      dst[y*4*K + x] = src[y*K + x];
-    }
-  }
-}
-
-
-LIBXSMM_INLINE void convert_c4k_4ck(int C, int K, float *src, float *dst)
-{
-  /* offsets: i--0, c--1, f--2, o--3 */
-  int x, y, offset;
-#if defined(_OPENMP)
-  LIBXSMM_OMP_VAR(x); LIBXSMM_OMP_VAR(y);
-# pragma omp parallel for private(x, y, offset)
-#endif
-  for (offset = 0; offset < 4; offset++) {
-    for (y = 0; y < C; y++) {
-      for (x = 0; x < K; x++) {
-        dst[offset*C*K + y*K + x] = src[y*4*K + offset*K + x];
-      }
-    }
-  }
-}
-
-
-LIBXSMM_INLINE void convert_nk_nck(int N, int K, int CK, float *src, float *dst)
-{
-  int x, y;
-#if defined(_OPENMP)
-  LIBXSMM_OMP_VAR(x);
-# pragma omp parallel for private(x, y)
-#endif
-  for (y = 0; y < N; y++) {
-    for (x = 0; x < K; x++) {
-      dst[y*CK + x] = src[y*K + x];
-    }
-  }
-}
-
-
-LIBXSMM_INLINE void lstm_fwd_copy_bias(int N, int K, float *bigold, float *bcgold, float *bfgold, float *bogold, float forget_bias, float *icfogoldt, int j)
-{
-  LIBXSMM_VLA_DECL(3, float, icfogold, icfogoldt, N, 4 * K);
-  int i, l;
-#if defined(_OPENMP)
-  LIBXSMM_OMP_VAR(i); LIBXSMM_OMP_VAR(l);
-# pragma omp parallel for private(i, l) LIBXSMM_OPENMP_COLLAPSE(2)
-#endif
-  for (i = 0; i < N; i++) {
-    for (l = 0; l < K; l++) {
-      LIBXSMM_VLA_ACCESS(3, icfogold, j, i, l,     N, 4 * K) = bigold[l];
-      LIBXSMM_VLA_ACCESS(3, icfogold, j, i, l+K,   N, 4 * K) = bcgold[l];
-      LIBXSMM_VLA_ACCESS(3, icfogold, j, i, l+2*K, N, 4 * K) = bfgold[l] + forget_bias;
-      LIBXSMM_VLA_ACCESS(3, icfogold, j, i, l+3*K, N, 4 * K) = bogold[l];
-    }
-  }
-}
-
-
-LIBXSMM_INLINE void lstm_fwd_eltwise_merged(int N, int K, float *i, float *c, float *f, float *o, float *csp, float *cs, float *co, float *h)
-{
-  int j;
-#if defined(__AVX512F__)
-  int l;
-  int rem = (K/16)*16;
-  __m512 minus1 = _mm512_set1_ps (-1.0f);
-  __m512 plus1  = _mm512_set1_ps (1.0f);
-#if defined(_OPENMP)
-# pragma omp parallel for private(j, l) LIBXSMM_OPENMP_COLLAPSE(2)
-#endif
-  for (j = 0; j < N; j++) {
-    for (l = 0; l < rem; l+=16) {
-      __m512 iv   = LIBXSMM_INTRINSICS_MM512_LOAD_PS (&(i[j*4*K + l]));
-      __m512 cv   = LIBXSMM_INTRINSICS_MM512_LOAD_PS (&(c[j*4*K + l]));
-      __m512 fv   = LIBXSMM_INTRINSICS_MM512_LOAD_PS (&(f[j*4*K + l]));
-      __m512 ov   = LIBXSMM_INTRINSICS_MM512_LOAD_PS (&(o[j*4*K + l]));
-      __m512 cspv = LIBXSMM_INTRINSICS_MM512_LOAD_PS (&(csp[j*K + l]));
-      __m512 csv, cov, hv;
-      /* i = sigmoid(i) */
-      iv = _mm512_mul_ps (iv, minus1);
-      iv = _mm512_exp_ps (iv);
-      iv = _mm512_add_ps (iv, plus1);
-      iv = _mm512_div_ps (plus1, iv);
-      /* c = tanh(c) */
-      cv = LIBXSMM_INTRINSICS_MM512_TANH_PS (cv);
-      /* f = sigmoid(f) */
-      fv = _mm512_mul_ps (fv, minus1);
-      fv = _mm512_exp_ps (fv);
-      fv = _mm512_add_ps (fv, plus1);
-      fv = _mm512_div_ps (plus1, fv);
-      /* o = sigmoid(o) */
-      ov = _mm512_mul_ps (ov, minus1);
-      ov = _mm512_exp_ps (ov);
-      ov = _mm512_add_ps (ov, plus1);
-      ov = _mm512_div_ps (plus1, ov);
-      /* cs = f.csp + i.c */
-      csv = _mm512_mul_ps (fv, cspv);
-      csv = _mm512_fmadd_ps (iv, cv, csv);
-      /* co = tanh(cs) */
-      cov = LIBXSMM_INTRINSICS_MM512_TANH_PS (csv);
-      /* h = o.co */
-      hv = _mm512_mul_ps (ov, cov);
-      _mm512_store_ps (&(i[j*4*K + l]), iv);
-      _mm512_store_ps (&(c[j*4*K + l]), cv);
-      _mm512_store_ps (&(f[j*4*K + l]), fv);
-      _mm512_store_ps (&(o[j*4*K + l]), ov);
-      _mm512_store_ps (&(cs[j*K + l]),  csv);
-      _mm512_store_ps (&(co[j*K + l]),  cov);
-      _mm512_store_ps (&(h[j*K + l]),   hv);
-    }
-  }
-#if defined(_OPENMP)
-# pragma omp parallel for private(j, l) LIBXSMM_OPENMP_COLLAPSE(2)
-#endif
-  for (j = 0; j < N; j++) {
-    for (l = rem; l < K; l++) {
-      float exp_value;
-      /* i = sigmoid(i) */
-      exp_value = (float)exp((double) -i[j*4*K + l]);
-      i[j*4*K + l] = 1.0f / (1.0f + exp_value);
-      /* c = tanh(c) */
-      c[j*4*K + l] = (float)tanh((double)c[j*4*K + l]);
-      /* f = sigmoid(f) */
-      exp_value = (float)exp((double) -f[j*4*K + l]);
-      f[j*4*K + l] = 1.0f / (1.0f + exp_value);
-      /* o = sigmoid(o) */
-      exp_value = (float)exp((double) -o[j*4*K + l]);
-      o[j*4*K + l] = 1.0f / (1.0f + exp_value);
-      /* cs = f.csp + i.c */
-      cs[j*K + l] = f[j*4*K + l]*csp[j*K + l] + i[j*4*K + l]*c[j*4*K + l];
-      /* co = tanh(cs) */
-      co[j*K + l] = (float)tanh((double)cs[j*K + l]);
-      /* h = o.co */
-      h[j*K + l] = o[j*4*K + l] * co[j*K + l];
-    }
-  }
-#else
-#if defined(_OPENMP)
-# pragma omp parallel for private(j)
-#endif
-  for (j = 0; j < N*K; j++) {
-    const int row = j / K;
-    const int col = j % K;
-    float exp_value;
-    /* i = sigmoid(i) */
-    exp_value = (float)exp((double) -i[row*4*K + col]);
-    i[row*4*K + col] = 1.0f / (1.0f + exp_value);
-    /* c = tanh(c) */
-    c[row*4*K + col] = (float)tanh((double)c[row*4*K + col]);
-    /* f = sigmoid(f) */
-    exp_value = (float)exp((double) -f[row*4*K + col]);
-    f[row*4*K + col] = 1.0f / (1.0f + exp_value);
-    /* o = sigmoid(o) */
-    exp_value = (float)exp((double) -o[row*4*K + col]);
-    o[row*4*K + col] = 1.0f / (1.0f + exp_value);
-    /* cs = f.csp + i.c */
-    cs[j] = f[row*4*K + col]*csp[j] + i[row*4*K + col]*c[row*4*K + col];
-    /* co = tanh(cs) */
-    co[j] = (float)tanh((double)cs[j]);
-    /* h = o.co */
-    h[j] = o[row*4*K + col] * co[j];
-  }
-#endif
-}
-
-
-LIBXSMM_INLINE void lstm_bwd_upd_eltwise_merged(int N, int K, float *i, float *c, float *f, float *o, float *csp, float *co,
-                                                float *dh, float *dout, float *di, float *dc, float *df, float *dp, float *dcsp, float *dcs)
-{
-  int j;
-#if defined(__AVX512F__)
-  int l;
-  int rem = (K/16)*16;
-  __m512 plus1  = _mm512_set1_ps (1.0f);
-#if defined(_OPENMP)
-# pragma omp parallel for private(j, l) LIBXSMM_OPENMP_COLLAPSE(2)
-#endif
-  for (j = 0; j < N; j++) {
-    for (l = 0; l < rem; l+=16) {
-      __m512 iv       = LIBXSMM_INTRINSICS_MM512_LOAD_PS (&(i[j*4*K + l]));
-      __m512 cv       = LIBXSMM_INTRINSICS_MM512_LOAD_PS (&(c[j*4*K + l]));
-      __m512 fv       = LIBXSMM_INTRINSICS_MM512_LOAD_PS (&(f[j*4*K + l]));
-      __m512 ov       = LIBXSMM_INTRINSICS_MM512_LOAD_PS (&(o[j*4*K + l]));
-      __m512 cspv     = LIBXSMM_INTRINSICS_MM512_LOAD_PS (&(csp[j*K + l]));
-      __m512 cov      = LIBXSMM_INTRINSICS_MM512_LOAD_PS (&(co[j*K + l]));
-      __m512 dcsv     = LIBXSMM_INTRINSICS_MM512_LOAD_PS (&(dcs[j*K + l]));
-      __m512 dhv, doutv, div, dcv, dfv, dov, dcspv, deltav, tv;
-      /* compute delta */
-      if (NULL == dout) {
-        deltav = LIBXSMM_INTRINSICS_MM512_LOAD_PS (&(dh[j*K + l]));
-      } else {
-        dhv    = LIBXSMM_INTRINSICS_MM512_LOAD_PS (&(dh[j*K + l]));
-        doutv  = LIBXSMM_INTRINSICS_MM512_LOAD_PS (&(dout[j*K + l]));
-        deltav = _mm512_add_ps (dhv, doutv);
-      }
-      /* compute dcsp */
-      /* dcsp = delta.o.(1 - (co.co)) + dcs */
-      tv    = _mm512_mul_ps (cov, cov);
-      tv    = _mm512_sub_ps (plus1, tv);
-      dcspv = _mm512_mul_ps (deltav, ov);
-      dcspv = _mm512_fmadd_ps (dcspv, tv, dcsv);
-      /* compute di */
-      /* di = dcsp.c.i.(1 - i) */
-      tv  = _mm512_sub_ps (plus1, iv);
-      tv  = _mm512_mul_ps (iv, tv);
-      div = _mm512_mul_ps (dcspv, cv);
-      div = _mm512_mul_ps (div, tv);
-      /* compute dc */
-      /* dc = dcsp.i.(1 - (c.c)) */
-      tv  = _mm512_mul_ps (cv, cv);
-      tv  = _mm512_sub_ps (plus1, tv);
-      dcv = _mm512_mul_ps (dcspv, iv);
-      dcv = _mm512_mul_ps (dcv, tv);
-      /* compute df */
-      /* df = dcsp.csp.f.(1 - f) */
-      tv  = _mm512_sub_ps (plus1, fv);
-      tv  = _mm512_mul_ps (fv, tv);
-      dfv = _mm512_mul_ps (dcspv, cspv);
-      dfv = _mm512_mul_ps (dfv, tv);
-      /* compute do */
-      /* do = delta.co.o.(1 - o) */
-      tv  = _mm512_sub_ps (plus1, ov);
-      tv  = _mm512_mul_ps (ov, tv);
-      dov = _mm512_mul_ps (deltav, cov);
-      dov = _mm512_mul_ps (dov, tv);
-      /* update dcsp */
-      /* dcsp = dcsp.f */
-      dcspv = _mm512_mul_ps (dcspv, fv);
-      _mm512_store_ps (&(di[j*4*K + l]), div);
-      _mm512_store_ps (&(dc[j*4*K + l]), dcv);
-      _mm512_store_ps (&(df[j*4*K + l]), dfv);
-      _mm512_store_ps (&(dp[j*4*K + l]), dov);
-      _mm512_store_ps (&(dcsp[j*K + l]), dcspv);
-    }
-  }
-#if defined(_OPENMP)
-# pragma omp parallel for private(j, l) LIBXSMM_OPENMP_COLLAPSE(2)
-#endif
-  for (j = 0; j < N; j++) {
-    for (l = rem; l < K; l++) {
-      float delta;
-      /* compute delta */
-      if (NULL == dout) {
-        delta = dh[j*K + l];
-      } else {
-        delta = dh[j*K + l] + dout[j*K + l];
-      }
-      /* compute dcsp */
-      dcsp[j*K + l] = delta * o[j*4*K + l] * (1.0f - (co[j*K + l]*co[j*K + l])) + dcs[j*K + l];
-      /* compute di */
-      di[j*4*K + l] = dcsp[j*K + l] * c[j*4*K + l] * i[j*4*K + l] * (1.0f - i[j*4*K + l]);
-      /* compute dc */
-      dc[j*4*K + l] = dcsp[j*K + l] * i[j*4*K + l] * (1.0f - (c[j*4*K + l]*c[j*4*K + l]));
-      /* compute df */
-      df[j*4*K + l] = dcsp[j*K + l] * csp[j*K + l] * f[j*4*K + l] * (1.0f - f[j*4*K + l]);
-      /* compute do */
-      dp[j*4*K + l] = delta * co[j*K + l] * o[j*4*K + l] * (1.0f - o[j*4*K + l]);
-      /* update dcsp */
-      dcsp[j*K + l] = dcsp[j*K + l] * f[j*4*K + l];
-    }
-  }
-#else
-#if defined(_OPENMP)
-# pragma omp parallel for private(j)
-#endif
-  for (j = 0; j < N*K; j++) {
-    const int row = j / K;
-    const int col = j % K;
-    float delta;
-    /* compute delta */
-    if (NULL == dout) {
-      delta = dh[j];
-    } else {
-      delta = dh[j] + dout[j];
-    }
-    /* compute dcsp */
-    dcsp[j] = delta * o[row*4*K + col] * (1.0f - (co[j]*co[j])) + dcs[j];
-    /* compute di */
-    di[row*4*K + col] = dcsp[j] * c[row*4*K + col] * i[row*4*K + col] * (1.0f - i[row*4*K + col]);
-    /* compute dc */
-    dc[row*4*K + col] = dcsp[j] * i[row*4*K + col] * (1.0f - (c[row*4*K + col]*c[row*4*K + col]));
-    /* compute df */
-    df[row*4*K + col] = dcsp[j] * csp[j] * f[row*4*K + col] * (1.0f - f[row*4*K + col]);
-    /* compute do */
-    dp[row*4*K + col] = delta * co[j] * o[row*4*K + col] * (1.0f - o[row*4*K + col]);
-    /* update dcsp */
-    dcsp[j] = dcsp[j] * f[row*4*K + col];
-  }
-#endif
-}
-
-
-void lstm_ref_fwd( int N, int C, int K, int t, float forget_bias,
-                   float *wigold, float *wcgold, float *wfgold, float *wogold,
-                   float *rigold, float *rcgold, float *rfgold, float *rogold,
-                   float *bigold, float *bcgold, float *bfgold, float *bogold,
-                   float *xgoldt, float *cspgold, float *hpgold,
-                   float *csgoldt, float *cogoldt, float *hgoldt,
-                   float *icfogoldt, float *wgold, float *rgold, float *scratch )
-{
-#if !defined(TWO_GEMMS)
-  float *xhgold;
-#endif
-  const char transa = 'N', transb = 'N';   /* no transposes */
-  const float alpha = 1, beta = 1;
-  int j;
-  int K4 = K * 4;
-  int CK = C + K;
-#if !defined(TWO_GEMMS)
-  xhgold = scratch;
-#endif
-  LIBXSMM_VLA_DECL(2, float, xgold, xgoldt, N * C);
-  LIBXSMM_VLA_DECL(2, float, csgold, csgoldt, K * N);
-  LIBXSMM_VLA_DECL(2, float, cogold, cogoldt, K * N);
-  LIBXSMM_VLA_DECL(2, float, hgold, hgoldt, K * N);
-  LIBXSMM_VLA_DECL(3, float, icfogold, icfogoldt, N, 4 * K);
-#if defined(PROFILE)
-  Gbl_conv_start = libxsmm_timer_tick();
-#endif
-#if defined(TWO_GEMMS)
-  convert_ck_c4k(C, K, wigold, wgold);
-  convert_ck_c4k(C, K, wcgold, &(wgold[K]));
-  convert_ck_c4k(C, K, wfgold, &(wgold[2*K]));
-  convert_ck_c4k(C, K, wogold, &(wgold[3*K]));
-  convert_ck_c4k(K, K, rigold, rgold);
-  convert_ck_c4k(K, K, rcgold, &(rgold[K]));
-  convert_ck_c4k(K, K, rfgold, &(rgold[2*K]));
-  convert_ck_c4k(K, K, rogold, &(rgold[3*K]));
-#else
-  LIBXSMM_UNUSED(rgold);
-  convert_ck_c4k(C, K, wigold, wgold);
-  convert_ck_c4k(C, K, wcgold, &(wgold[K]));
-  convert_ck_c4k(C, K, wfgold, &(wgold[2*K]));
-  convert_ck_c4k(C, K, wogold, &(wgold[3*K]));
-  convert_ck_c4k(K, K, rigold, &(wgold[C*K*4]));
-  convert_ck_c4k(K, K, rcgold, &(wgold[C*K*4 + K]));
-  convert_ck_c4k(K, K, rfgold, &(wgold[C*K*4 + 2*K]));
-  convert_ck_c4k(K, K, rogold, &(wgold[C*K*4 + 3*K]));
-#endif
-#if defined(PROFILE)
-  Gbl_conv_end = libxsmm_timer_tick();
-  Gbl_conv_total += libxsmm_timer_duration(Gbl_conv_start, Gbl_conv_end);
-#endif
-  for (j = 0; j < t; ++j) {
-    /* Initialization with bias */
-#if defined(PROFILE)
-    Gbl_copy_bias_start = libxsmm_timer_tick();
-#endif
-    lstm_fwd_copy_bias(N, K, bigold, bcgold, bfgold, bogold, forget_bias, icfogoldt, j);
-#if defined(PROFILE)
-    Gbl_copy_bias_end = libxsmm_timer_tick();
-    Gbl_copy_bias_total += libxsmm_timer_duration(Gbl_copy_bias_start, Gbl_copy_bias_end);
-    Gbl_blas_start = libxsmm_timer_tick();
-#endif
-#if defined(TWO_GEMMS)
-    /* icfo += W * x */
-    LIBXSMM_XBLAS_SYMBOL(float)(&transa, &transb, &K4, &N, &C, &alpha, wgold, &K4, &LIBXSMM_VLA_ACCESS(2, xgold, j, 0, N * C), &C, &beta, &LIBXSMM_VLA_ACCESS(3, icfogold, j, 0, 0, N, 4 * K), &K4);
-    /* icfo += R * h */
-    if (j == 0) {
-      LIBXSMM_XBLAS_SYMBOL(float)(&transa, &transb, &K4, &N, &K, &alpha, rgold, &K4, hpgold, &K, &beta, &LIBXSMM_VLA_ACCESS(3, icfogold, 0, 0, 0, N, 4 * K), &K4);
-    } else {
-      LIBXSMM_XBLAS_SYMBOL(float)(&transa, &transb, &K4, &N, &K, &alpha, rgold, &K4, &LIBXSMM_VLA_ACCESS(2, hgold, j-1, 0, K * N), &K, &beta, &LIBXSMM_VLA_ACCESS(3, icfogold, j, 0, 0, N, 4 * K), &K4);
-    }
-#else
-    /* Concatenate x and h */
-    convert_nk_nck(N, C, C+K, &LIBXSMM_VLA_ACCESS(2, xgold, j, 0, N * C), xhgold);
-    if (j == 0) {
-      convert_nk_nck(N, K, C+K, hpgold, &(xhgold[C]));
-    } else {
-      convert_nk_nck(N, K, C+K, &LIBXSMM_VLA_ACCESS(2, hgold, j-1, 0, K * N), &(xhgold[C]));
-    }
-    /* icfo += (W * x) + (R * h) */
-    LIBXSMM_XBLAS_SYMBOL(float)(&transa, &transb, &K4, &N, &CK, &alpha, wgold, &K4, xhgold, &CK, &beta, &LIBXSMM_VLA_ACCESS(3, icfogold, j, 0, 0, N, 4 * K), &K4);
-#endif
-#if defined(PROFILE)
-    Gbl_blas_end = libxsmm_timer_tick();
-    Gbl_blas_total += libxsmm_timer_duration(Gbl_blas_start, Gbl_blas_end);
-    Gbl_eltwise_start = libxsmm_timer_tick();
-#endif
-    if (j == 0) {
-      lstm_fwd_eltwise_merged( N, K,
-                               &LIBXSMM_VLA_ACCESS(3, icfogold, 0, 0, 0,   N, 4 * K),
-                               &LIBXSMM_VLA_ACCESS(3, icfogold, 0, 0, K,   N, 4 * K),
-                               &LIBXSMM_VLA_ACCESS(3, icfogold, 0, 0, 2*K, N, 4 * K),
-                               &LIBXSMM_VLA_ACCESS(3, icfogold, 0, 0, 3*K, N, 4 * K),
-                               cspgold,
-                               &LIBXSMM_VLA_ACCESS(2, csgold, 0, 0, K * N),
-                               &LIBXSMM_VLA_ACCESS(2, cogold, 0, 0, K * N),
-                               &LIBXSMM_VLA_ACCESS(2, hgold, 0, 0, K * N) );
-    } else {
-      lstm_fwd_eltwise_merged( N, K,
-                               &LIBXSMM_VLA_ACCESS(3, icfogold, j, 0, 0,   N, 4 * K),
-                               &LIBXSMM_VLA_ACCESS(3, icfogold, j, 0, K,   N, 4 * K),
-                               &LIBXSMM_VLA_ACCESS(3, icfogold, j, 0, 2*K, N, 4 * K),
-                               &LIBXSMM_VLA_ACCESS(3, icfogold, j, 0, 3*K, N, 4 * K),
-                               &LIBXSMM_VLA_ACCESS(2, csgold, j-1, 0, K * N),
-                               &LIBXSMM_VLA_ACCESS(2, csgold, j, 0, K * N),
-                               &LIBXSMM_VLA_ACCESS(2, cogold, j, 0, K * N),
-                               &LIBXSMM_VLA_ACCESS(2, hgold, j, 0, K * N) );
-    }
-#if defined(PROFILE)
-    Gbl_eltwise_end = libxsmm_timer_tick();
-    Gbl_eltwise_total += libxsmm_timer_duration(Gbl_eltwise_start, Gbl_eltwise_end);
-#endif
-  }
-}
-
-
-void lstm_ref_bwd_upd( int N, int C, int K, int t,
-                       float *xgoldt, float *cspgold, float *hpgold,
-                       float *csgoldt, float *cogoldt, float *hgoldt,
-                       float *icfogoldt, float *wgold, float *rgold,
-                       float *dcsgold, float *dhgoldt,
-                       float *dwgold, float *drgold, float *dbgold,
-                       float *dxgoldt, float *dcspgold, float *dhpgold, float *scratch )
-{
-#if !defined(TWO_GEMMS)
-  float *xhgold, *dxhgold;
-#endif
-  float *dicfogoldt, *doutgoldt;
-  float *dout, *dcs, *csp;
-  const char transa = 'N', transb = 'N';   /* no transposes */
-  const char transaT = 'T', transbT = 'T'; /* transposes */
-  const float alpha = 1, beta = 1, beta0 = 0;
-  int j, l, p;
-  int K4 = K * 4;
-  int CK = C + K;
-  dicfogoldt = scratch;
-  doutgoldt  = &(scratch[K*N*t*4]);
-#if !defined(TWO_GEMMS)
-  xhgold    = &(scratch[K*N*t*5]);
-  dxhgold   = &(scratch[K*N*t*5 + (C+K)*N]);
-#endif
-  LIBXSMM_VLA_DECL(2, float, xgold, xgoldt, N * C);
-  LIBXSMM_VLA_DECL(2, float, csgold, csgoldt, K * N);
-  LIBXSMM_VLA_DECL(2, float, cogold, cogoldt, K * N);
-  LIBXSMM_VLA_DECL(2, float, hgold, hgoldt, K * N);
-  LIBXSMM_VLA_DECL(3, float, icfogold, icfogoldt, N, 4 * K);
-  LIBXSMM_VLA_DECL(2, float, dxgold, dxgoldt, N * C);
-  LIBXSMM_VLA_DECL(2, float, dhgold, dhgoldt, K * N);
-  LIBXSMM_VLA_DECL(3, float, dicfogold, dicfogoldt, N, 4 * K);
-  LIBXSMM_VLA_DECL(2, float, doutgold, doutgoldt, K * N);
-  for (j = t-1; j >= 0; --j) {
-#if defined(PROFILE)
-    Gbl_eltwise_start = libxsmm_timer_tick();
-#endif
-    if (t-1 == j) {
-      dout = NULL;
-      dcs = dcsgold;
-    } else {
-      dout = &LIBXSMM_VLA_ACCESS(2, doutgold, j, 0, K * N);
-      dcs = dcspgold;
-    }
-    if (0 == j) {
-      csp = cspgold;
-    } else {
-      csp = &LIBXSMM_VLA_ACCESS(2, csgold, j-1, 0, K * N);
-    }
-    lstm_bwd_upd_eltwise_merged( N, K,
-                                 &LIBXSMM_VLA_ACCESS(3, icfogold, j, 0, 0,   N, 4 * K),
-                                 &LIBXSMM_VLA_ACCESS(3, icfogold, j, 0, K,   N, 4 * K),
-                                 &LIBXSMM_VLA_ACCESS(3, icfogold, j, 0, 2*K, N, 4 * K),
-                                 &LIBXSMM_VLA_ACCESS(3, icfogold, j, 0, 3*K, N, 4 * K),
-                                 csp,
-                                 &LIBXSMM_VLA_ACCESS(2, cogold, j, 0, K * N),
-                                 &LIBXSMM_VLA_ACCESS(2, dhgold, j, 0, K * N),
-                                 dout,
-                                 &LIBXSMM_VLA_ACCESS(3, dicfogold, j, 0, 0,   N, 4 * K),
-                                 &LIBXSMM_VLA_ACCESS(3, dicfogold, j, 0, K,   N, 4 * K),
-                                 &LIBXSMM_VLA_ACCESS(3, dicfogold, j, 0, 2*K, N, 4 * K),
-                                 &LIBXSMM_VLA_ACCESS(3, dicfogold, j, 0, 3*K, N, 4 * K),
-                                 dcspgold, dcs);
-#if defined(PROFILE)
-    Gbl_eltwise_end = libxsmm_timer_tick();
-    Gbl_eltwise_total += libxsmm_timer_duration(Gbl_eltwise_start, Gbl_eltwise_end);
-    Gbl_blas_start = libxsmm_timer_tick();
-#endif
-#if defined(TWO_GEMMS)
-    if (j > 0) {
-      /* compute dout */
-      LIBXSMM_XBLAS_SYMBOL(float)(&transaT, &transb, &K, &N, &K4, &alpha, rgold, &K4, &LIBXSMM_VLA_ACCESS(3, dicfogold, j, 0, 0, N, 4 * K), &K4, &beta0, &LIBXSMM_VLA_ACCESS(2, doutgold, j-1, 0, K * N), &K);
-    } else {
-      /* compute dhp */
-      LIBXSMM_XBLAS_SYMBOL(float)(&transaT, &transb, &K, &N, &K4, &alpha, rgold, &K4, &LIBXSMM_VLA_ACCESS(3, dicfogold, 0, 0, 0, N, 4 * K), &K4, &beta0, dhpgold, &K);
-    }
-
-    /* compute dx */
-    LIBXSMM_XBLAS_SYMBOL(float)(&transaT, &transb, &C, &N, &K4, &alpha, wgold, &K4, &LIBXSMM_VLA_ACCESS(3, dicfogold, j, 0, 0, N, 4 * K), &K4, &beta, &LIBXSMM_VLA_ACCESS(2, dxgold, j, 0, N * C), &C);
-
-    /* compute dw */
-    LIBXSMM_XBLAS_SYMBOL(float)(&transa, &transbT, &K4, &C, &N, &alpha, &LIBXSMM_VLA_ACCESS(3, dicfogold, j, 0, 0, N, 4 * K), &K4, &LIBXSMM_VLA_ACCESS(2, xgold, j, 0, N * C), &C, &beta, dwgold, &K4);
-
-    /* compute dr */
-    if (j == 0) {
-      LIBXSMM_XBLAS_SYMBOL(float)(&transa, &transbT, &K4, &K, &N, &alpha, &LIBXSMM_VLA_ACCESS(3, dicfogold, j, 0, 0, N, 4 * K), &K4, hpgold, &K, &beta, drgold, &K4);
-    } else {
-      LIBXSMM_XBLAS_SYMBOL(float)(&transa, &transbT, &K4, &K, &N, &alpha, &LIBXSMM_VLA_ACCESS(3, dicfogold, j, 0, 0, N, 4 * K), &K4, &LIBXSMM_VLA_ACCESS(2, hgold, j-1, 0, K * N), &K, &beta, drgold, &K4);
-    }
-#else
-    LIBXSMM_UNUSED(rgold); LIBXSMM_UNUSED(drgold);
-    LIBXSMM_XBLAS_SYMBOL(float)(&transaT, &transb, &CK, &N, &K4, &alpha, wgold, &K4, &LIBXSMM_VLA_ACCESS(3, dicfogold, j, 0, 0, N, 4 * K), &K4, &beta0, dxhgold, &CK);
-    matrix_copy_ld(C, N, C+K, dxhgold, &LIBXSMM_VLA_ACCESS(2, dxgold, j, 0, N * C));
-    if (j > 0) {
-      matrix_copy_ld(K, N, C+K, &(dxhgold[C]), &LIBXSMM_VLA_ACCESS(2, doutgold, j-1, 0, K * N));
-    } else {
-      matrix_copy_ld(K, N, C+K, &(dxhgold[C]), dhpgold);
-    }
-
-    /* Concatenate x and h */
-    convert_nk_nck(N, C, C+K, &LIBXSMM_VLA_ACCESS(2, xgold, j, 0, N * C), xhgold);
-    if (j == 0) {
-      convert_nk_nck(N, K, C+K, hpgold, &(xhgold[C]));
-    } else {
-      convert_nk_nck(N, K, C+K, &LIBXSMM_VLA_ACCESS(2, hgold, j-1, 0, K * N), &(xhgold[C]));
-    }
-    LIBXSMM_XBLAS_SYMBOL(float)(&transa, &transbT, &K4, &CK, &N, &alpha, &LIBXSMM_VLA_ACCESS(3, dicfogold, j, 0, 0, N, 4 * K), &K4, xhgold, &CK, &beta, dwgold, &K4);
-#endif
-#if defined(PROFILE)
-    Gbl_blas_end = libxsmm_timer_tick();
-    Gbl_blas_total += libxsmm_timer_duration(Gbl_blas_start, Gbl_blas_end);
-#endif
-    /* compute db */
-#if defined(_OPENMP)
-    LIBXSMM_OMP_VAR(p);
-# pragma omp parallel for private(l, p)
-#endif
-    for (l = 0; l < K; l++) {
-      for (p = 0; p < N; p++) {
-        dbgold[l]       += LIBXSMM_VLA_ACCESS(3, dicfogold, j, p, l,       N, 4 * K);
-        dbgold[l + K]   += LIBXSMM_VLA_ACCESS(3, dicfogold, j, p, l + K,   N, 4 * K);
-        dbgold[l + 2*K] += LIBXSMM_VLA_ACCESS(3, dicfogold, j, p, l + 2*K, N, 4 * K);
-        dbgold[l + 3*K] += LIBXSMM_VLA_ACCESS(3, dicfogold, j, p, l + 3*K, N, 4 * K);
-      }
-    }
-  }
-}
-
-
 int main(int argc, char* argv[])
 {
   float *wigold, *wfgold, *wogold, *wcgold, *rigold, *rfgold, *rogold, *rcgold, *bigold, *bfgold, *bogold, *bcgold;
@@ -1039,21 +179,12 @@ int main(int argc, char* argv[])
   dxgoldt   = (float*)libxsmm_aligned_malloc(N*C*t*sizeof(float), 2097152);
   dcspgold  = (float*)libxsmm_aligned_malloc(K*N*sizeof(float), 2097152);
   dhpgold   = (float*)libxsmm_aligned_malloc(K*N*sizeof(float), 2097152);
-#if defined(TWO_GEMMS)
-  wgold     = (float*)libxsmm_aligned_malloc(C*K*4*sizeof(float), 2097152);
-  rgold     = (float*)libxsmm_aligned_malloc(K*K*4*sizeof(float), 2097152);
-  dwgold    = (float*)libxsmm_aligned_malloc(C*K*4*sizeof(float), 2097152);
-  drgold    = (float*)libxsmm_aligned_malloc(K*K*4*sizeof(float), 2097152);
-  scratch_fwd = NULL;
-  scratch_bu  = (float*)libxsmm_aligned_malloc(K*N*t*5*sizeof(float), 2097152);
-#else
   wgold     = (float*)libxsmm_aligned_malloc((C+K)*K*4*sizeof(float), 2097152);
   rgold     = NULL;
   dwgold    = (float*)libxsmm_aligned_malloc((C+K)*K*4*sizeof(float), 2097152);
   drgold    = NULL;
   scratch_fwd = (float*)libxsmm_aligned_malloc((C+K)*N*sizeof(float), 2097152);
   scratch_bu  = (float*)libxsmm_aligned_malloc(((C+K)*N*2 + K*N*t*5)*sizeof(float), 2097152);
-#endif
   dbgold    = (float*)libxsmm_aligned_malloc(K*4*sizeof(float), 2097152);
   dcsgold   = (float*)libxsmm_aligned_malloc(K*N*sizeof(float), 2097152);
   dhgoldt   = (float*)libxsmm_aligned_malloc(K*N*t*sizeof(float), 2097152);
@@ -1114,12 +245,7 @@ int main(int argc, char* argv[])
   zero_buf(dxgoldt,  N*C*t);
   zero_buf(dcspgold, K*N);
   zero_buf(dhpgold,  K*N);
-#if defined(TWO_GEMMS)
-  zero_buf(dwgold, C*K*4);
-  zero_buf(drgold, K*K*4);
-#else
   zero_buf(dwgold, (C+K)*K*4);
-#endif
   zero_buf(dbgold, K*4);
 
   /* first touch LIBXSMM */
@@ -1207,14 +333,14 @@ int main(int argc, char* argv[])
     CHKERR_LIBXSMM_DNN( libxsmm_dnn_rnncell_allocate_forget_bias(libxsmm_handle, forget_bias) );
 
     /* setup LIBXSMM buffers and filter */
-    libxsmm_layout = libxsmm_dnn_rnncell_create_tensor_datalayout( libxsmm_handle, LIBXSMM_DNN_RNN_REGULAR_INPUT, &status ); CHKERR_LIBXSMM_DNN( status );
-    libxsmm_input = libxsmm_dnn_link_tensor( libxsmm_layout, xt, &status ); CHKERR_LIBXSMM_DNN( status );
-    libxsmm_dnn_destroy_tensor_datalayout( libxsmm_layout );
-
     libxsmm_layout = libxsmm_dnn_rnncell_create_tensor_datalayout( libxsmm_handle, LIBXSMM_DNN_RNN_REGULAR_CS_PREV, &status ); CHKERR_LIBXSMM_DNN( status );
     libxsmm_cs_prev = libxsmm_dnn_link_tensor( libxsmm_layout, csp, &status ); CHKERR_LIBXSMM_DNN( status );
     libxsmm_dnn_destroy_tensor_datalayout( libxsmm_layout );
 
+    libxsmm_layout = libxsmm_dnn_rnncell_create_tensor_datalayout( libxsmm_handle, LIBXSMM_DNN_RNN_REGULAR_INPUT, &status ); CHKERR_LIBXSMM_DNN( status );
+    libxsmm_input = libxsmm_dnn_link_tensor( libxsmm_layout, xt, &status ); CHKERR_LIBXSMM_DNN( status );
+    libxsmm_dnn_destroy_tensor_datalayout( libxsmm_layout );
+
     libxsmm_layout = libxsmm_dnn_rnncell_create_tensor_datalayout( libxsmm_handle, LIBXSMM_DNN_RNN_REGULAR_HIDDEN_STATE_PREV, &status ); CHKERR_LIBXSMM_DNN( status );
     libxsmm_hidden_state_prev = libxsmm_dnn_link_tensor( libxsmm_layout, hp, &status ); CHKERR_LIBXSMM_DNN( status );
     libxsmm_dnn_destroy_tensor_datalayout( libxsmm_layout );
@@ -1461,11 +587,7 @@ int main(int argc, char* argv[])
       printf("Check-norm    : %.24f\n", norms_upd_w.normf_rel);
       libxsmm_matdiff_reduce(&diff, &norms_upd_w);
 
-#if defined(TWO_GEMMS)
-      libxsmm_matdiff(&norms_upd_r, LIBXSMM_DATATYPE_F32, K*K*4, 1, drgold, dr, 0, 0);
-#else
       libxsmm_matdiff(&norms_upd_r, LIBXSMM_DATATYPE_F32, K*K*4, 1, &(dwgold[C*K*4]), dr, 0, 0);
-#endif
       printf("Delta recurrent weight\n");
       printf("L1 reference  : %.25g\n", norms_upd_r.l1_ref);
       printf("L1 test       : %.25g\n", norms_upd_r.l1_tst);
@@ -1528,11 +650,7 @@ int main(int argc, char* argv[])
       printf("Check-norm    : %.24f\n", norms_upd_w.normf_rel);
       libxsmm_matdiff_reduce(&diff, &norms_upd_w);
 
-#if defined(TWO_GEMMS)
-      libxsmm_matdiff(&norms_upd_r, LIBXSMM_DATATYPE_F32, K*K*4, 1, drgold, dr, 0, 0);
-#else
       libxsmm_matdiff(&norms_upd_r, LIBXSMM_DATATYPE_F32, K*K*4, 1, &(dwgold[C*K*4]), dr, 0, 0);
-#endif
       printf("Delta recurrent weight\n");
       printf("L1 reference  : %.25g\n", norms_upd_r.l1_ref);
       printf("L1 test       : %.25g\n", norms_upd_r.l1_tst);
@@ -1907,10 +1025,6 @@ int main(int argc, char* argv[])
   libxsmm_free(dcspgold);
   libxsmm_free(dhpgold);
   libxsmm_free(dwgold);
-#if defined(TWO_GEMMS)
-  libxsmm_free(rgold);
-  libxsmm_free(drgold);
-#endif
   libxsmm_free(dbgold);
   libxsmm_free(dcsgold);
   libxsmm_free(dhgoldt);
@@ -1949,6 +1063,6 @@ int main(int argc, char* argv[])
   /* some empty lines at the end */
   printf("\n\n\n");
 
-  return EXIT_SUCCESS;
+  return global_status;
 }
 
