@@ -1,34 +1,15 @@
 /******************************************************************************
-** Copyright (c) 2016-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
 #include "edge_proxy_common.h"
+#include <libxsmm_intrinsics_x86.h>
 #include <libxsmm.h>
 #include <stdlib.h>
 #include <string.h>
@@ -47,7 +28,7 @@
 #include <linux/mman.h>
 #endif
 
-void* edge_hp_malloc( size_t nbytes, size_t alignment ) {
+LIBXSMM_INLINE void* edge_hp_malloc( size_t nbytes, size_t alignment ) {
   void* ret_ptr = NULL;
 #if defined(EDGE_HP_1G)
   size_t num_large_pages = nbytes / (1073741824L);
@@ -81,7 +62,7 @@ void* edge_hp_malloc( size_t nbytes, size_t alignment ) {
   return ret_ptr;
 }
 
-void edge_hp_free( void* ptr,  size_t nbytes ) {
+LIBXSMM_INLINE void edge_hp_free( void* ptr,  size_t nbytes ) {
   LIBXSMM_UNUSED( nbytes );
 #if defined(EDGE_HP_1G)
   /* to be implemented */
@@ -93,21 +74,23 @@ void edge_hp_free( void* ptr,  size_t nbytes ) {
 }
 
 #if defined(__AVX512F__)
-static void matMulFusedAC(       unsigned short  i_r,
-                                 unsigned int    i_m,
-                                 unsigned int    i_n,
-                                 unsigned int    i_k,
-                                 unsigned int    i_ldA,
-                                 unsigned int    i_ldB,
-                                 unsigned int    i_ldC,
-                                 double       i_beta,
-                           const double      *i_a,
-                           const double      *i_b,
-                                 double      *o_c ) {
+LIBXSMM_INLINE void matMulFusedAC(  unsigned short  i_r,
+                                    unsigned int    i_m,
+                                    unsigned int    i_n,
+                                    unsigned int    i_k,
+                                    unsigned int    i_ldA,
+                                    unsigned int    i_ldB,
+                                    unsigned int    i_ldC,
+                                          double    i_beta,
+                                    const double   *i_a,
+                                    const double   *i_b,
+                                          double   *o_c ) {
   unsigned int l_m, l_n, l_k;
+  const __m512d beta = _mm512_set1_pd( i_beta );
+  LIBXSMM_UNUSED(i_r);
   for( l_m = 0; l_m < i_m; l_m++ ) {
     for( l_n = 0; l_n < i_n; l_n++ ) {
-      __m512d vc = (i_beta != 0.0) ? _mm512_mul_pd( _mm512_loadu_pd( &(o_c[l_m*i_ldC*8 + l_n*8 + 0]) ), _mm512_set1_pd( i_beta ) ) : _mm512_setzero_pd();
+      __m512d vc = (i_beta != 0.0) ? _mm512_mul_pd( _mm512_loadu_pd( &(o_c[l_m*i_ldC*8 + l_n*8 + 0]) ), beta ) : _mm512_setzero_pd();
       _mm512_storeu_pd(&(o_c[l_m*i_ldC*8 + l_n*8 + 0]), vc);
     }
   }
@@ -116,7 +99,8 @@ static void matMulFusedAC(       unsigned short  i_r,
     for( l_n = 0; l_n < i_n; l_n++ ) {
       __m512d vc = _mm512_loadu_pd( &(o_c[l_m*i_ldC*8 + l_n*8 + 0]) );
       for( l_k = 0; l_k < i_k; l_k++ ) {
-        vc = _mm512_fmadd_pd( _mm512_set1_pd( i_b[l_k*i_ldB + l_n] ), _mm512_loadu_pd( &(i_a[l_m*i_ldA*8 + l_k*8 + 0]) ), vc);
+        const __m512d alpha = _mm512_set1_pd( i_b[l_k*i_ldB + l_n] );
+        vc = _mm512_fmadd_pd( alpha, _mm512_loadu_pd( &(i_a[l_m*i_ldA*8 + l_k*8 + 0]) ), vc);
       }
       _mm512_storeu_pd( &(o_c[l_m*i_ldC*8 + l_n*8 + 0]), vc );
     }
@@ -124,21 +108,23 @@ static void matMulFusedAC(       unsigned short  i_r,
 }
 
 
-static void matMulFusedBC(        unsigned short  i_r,
-                                  unsigned int    i_m,
-                                  unsigned int    i_n,
-                                  unsigned int    i_k,
-                                  unsigned int    i_ldA,
-                                  unsigned int    i_ldB,
-                                  unsigned int    i_ldC,
-                                  double       i_beta,
-                            const double      *i_a,
-                            const double      *i_b,
-                                  double      *o_c ) {
+LIBXSMM_INLINE void matMulFusedBC(  unsigned short  i_r,
+                                    unsigned int    i_m,
+                                    unsigned int    i_n,
+                                    unsigned int    i_k,
+                                    unsigned int    i_ldA,
+                                    unsigned int    i_ldB,
+                                    unsigned int    i_ldC,
+                                    double          i_beta,
+                              const double         *i_a,
+                              const double         *i_b,
+                                    double         *o_c ) {
   unsigned int l_m, l_n, l_k;
+  const __m512d beta = _mm512_set1_pd( i_beta );
+  LIBXSMM_UNUSED(i_r);
   for( l_m = 0; l_m < i_m; l_m++ ) {
     for( l_n = 0; l_n < i_n; l_n++ ) {
-      __m512d vc = (i_beta != 0.0) ? _mm512_mul_pd( _mm512_loadu_pd( &(o_c[l_m*i_ldC*8 + l_n*8 + 0]) ), _mm512_set1_pd( i_beta ) ) : _mm512_setzero_pd();
+      __m512d vc = (i_beta != 0.0) ? _mm512_mul_pd( _mm512_loadu_pd( &(o_c[l_m*i_ldC*8 + l_n*8 + 0]) ), beta ) : _mm512_setzero_pd();
       _mm512_storeu_pd(&(o_c[l_m*i_ldC*8 + l_n*8 + 0]), vc);
     }
   }
@@ -147,7 +133,8 @@ static void matMulFusedBC(        unsigned short  i_r,
     for( l_n = 0; l_n < i_n; l_n++ ) {
       __m512d vc = _mm512_loadu_pd( &(o_c[l_m*i_ldC*8 + l_n*8 + 0]) );
       for( l_k = 0; l_k < i_k; l_k++ ) {
-        vc = _mm512_fmadd_pd( _mm512_set1_pd( i_a[l_m*i_ldA + l_k] ), _mm512_loadu_pd( &(i_b[l_k*i_ldB*8 + l_n*8 + 0]) ), vc);
+        const __m512d alpha = _mm512_set1_pd( i_a[l_m*i_ldA + l_k] );
+        vc = _mm512_fmadd_pd( alpha, _mm512_loadu_pd( &(i_b[l_k*i_ldB*8 + l_n*8 + 0]) ), vc);
       }
       _mm512_storeu_pd( &(o_c[l_m*i_ldC*8 + l_n*8 + 0]), vc );
     }
@@ -155,7 +142,7 @@ static void matMulFusedBC(        unsigned short  i_r,
 }
 #endif
 
-void amok_detect( const double* i_runtimes, size_t* io_amoks, const size_t i_workers ) {
+LIBXSMM_INLINE void amok_detect( const double* i_runtimes, size_t* io_amoks, const size_t i_workers ) {
   double time_avg;
   size_t i;
   time_avg = 0.0;
@@ -176,7 +163,7 @@ void amok_detect( const double* i_runtimes, size_t* io_amoks, const size_t i_wor
   }
 }
 
-void amok_balance( const size_t* i_amoks, const size_t i_workers, const size_t i_worksize, const size_t i_mytid, size_t* io_chunk, size_t* io_mystart, size_t* io_myend ) {
+LIBXSMM_INLINE void amok_balance( const size_t* i_amoks, const size_t i_workers, const size_t i_worksize, const size_t i_mytid, size_t* io_chunk, size_t* io_mystart, size_t* io_myend ) {
   size_t l_chunk, l_start, l_end;
   size_t l_cur_amoks = i_amoks[8*i_workers];
   size_t l_non_amoks = i_workers - l_cur_amoks;
@@ -313,10 +300,10 @@ int main(int argc, char* argv[])
     num_quants, num_modes, num_modes, num_modes, 0, num_modes, alpha, beta, flags, prefetch);
   l_xgemm_desc_star = libxsmm_dgemm_descriptor_init(&l_xgemm_blob_star,
     num_quants, num_modes, num_quants, 0, num_modes, num_modes, alpha, beta, flags, prefetch);
-  a_kernel =  libxsmm_create_xcsr_soa( l_xgemm_desc_stiff, mat_a_rowptr,  mat_a_colidx,  (const void*)mat_a_values ).dmm;
-  b_kernel =  libxsmm_create_xcsr_soa( l_xgemm_desc_stiff, mat_b_rowptr,  mat_b_colidx,  (const void*)mat_b_values ).dmm;
-  c_kernel =  libxsmm_create_xcsr_soa( l_xgemm_desc_stiff, mat_c_rowptr,  mat_c_colidx,  (const void*)mat_c_values ).dmm;
-  st_kernel = libxsmm_create_xcsr_soa( l_xgemm_desc_star, mat_st_rowptr, mat_st_colidx, (const void*)mat_st_values ).dmm;
+  a_kernel =  libxsmm_create_xcsr_soa( l_xgemm_desc_stiff, mat_a_rowptr,  mat_a_colidx,  (const void*)mat_a_values, num_cfr ).dmm;
+  b_kernel =  libxsmm_create_xcsr_soa( l_xgemm_desc_stiff, mat_b_rowptr,  mat_b_colidx,  (const void*)mat_b_values, num_cfr ).dmm;
+  c_kernel =  libxsmm_create_xcsr_soa( l_xgemm_desc_stiff, mat_c_rowptr,  mat_c_colidx,  (const void*)mat_c_values, num_cfr ).dmm;
+  st_kernel = libxsmm_create_xcsr_soa( l_xgemm_desc_star, mat_st_rowptr, mat_st_colidx, (const void*)mat_st_values, num_cfr ).dmm;
   if ( a_kernel == 0 ) {
     printf("a kernel could not be built -> exit!");
     exit(-1);
@@ -505,3 +492,4 @@ int main(int argc, char* argv[])
 
   return 0;
 }
+
