@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2017-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Hans Pabst (Intel Corp.)
 ******************************************************************************/
@@ -167,9 +147,6 @@ int main(int argc, char* argv[])
         LIBXSMM_MATINIT(OTYPE, 22 + i, c + static_cast<size_t>(csize) * i, m, n, ldc, scale);
       }
 
-#if defined(MKL_ENABLE_AVX512)
-      mkl_enable_instructions(MKL_ENABLE_AVX512);
-#endif
       // initialize LIBXSMM
       libxsmm_init();
 
@@ -195,7 +172,7 @@ int main(int argc, char* argv[])
               LIBXSMM_GEMM_PREFETCH_C(ci + csize));
           }
         }
-        const unsigned long long ncycles = libxsmm_timer_diff(start, libxsmm_timer_tick());
+        const unsigned long long ncycles = libxsmm_timer_ncycles(start, libxsmm_timer_tick());
         const double duration = libxsmm_timer_duration(0, ncycles) / nrepeat;
         if (0 < duration && 0 != ncycles) {
           fprintf(stdout, "\tpseudo-perf.: %.1f %s/cycle\n", (2.0 * k - 1.0) * (static_cast<double>(s) * m * n) / ncycles, ops);
@@ -219,7 +196,7 @@ int main(int argc, char* argv[])
             smm_eigen_dynamic(m, n, k, ai, bi, ci);
           }
         }
-        const unsigned long long ncycles = libxsmm_timer_diff(start, libxsmm_timer_tick());
+        const unsigned long long ncycles = libxsmm_timer_ncycles(start, libxsmm_timer_tick());
         const double duration = libxsmm_timer_duration(0, ncycles) / nrepeat;
         if (0 < duration && 0 != ncycles) {
           fprintf(stdout, "\tpseudo-perf.: %.1f %s/cycle\n", (2.0 * k - 1.0) * (static_cast<double>(s) * m * n) / ncycles, ops);
@@ -245,7 +222,7 @@ int main(int argc, char* argv[])
               LIBXSMM_GEMM_PREFETCH_C(ci + csize));
           }
         }
-        const unsigned long long ncycles = libxsmm_timer_diff(start, libxsmm_timer_tick());
+        const unsigned long long ncycles = libxsmm_timer_ncycles(start, libxsmm_timer_tick());
         const double duration = libxsmm_timer_duration(0, ncycles) / nrepeat;
         if (0 < duration && 0 != ncycles) {
           fprintf(stdout, "\tpseudo-perf.: %.1f %s/cycle\n", (2.0 * k - 1.0) * (static_cast<double>(s) * m * n) / ncycles, ops);
@@ -269,7 +246,7 @@ int main(int argc, char* argv[])
             smm_eigen_dynamic(m, n, k, ai, b, ci);
           }
         }
-        const unsigned long long ncycles = libxsmm_timer_diff(start, libxsmm_timer_tick());
+        const unsigned long long ncycles = libxsmm_timer_ncycles(start, libxsmm_timer_tick());
         const double duration = libxsmm_timer_duration(0, ncycles) / nrepeat;
         if (0 < duration && 0 != ncycles) {
           fprintf(stdout, "\tpseudo-perf.: %.1f %s/cycle\n", (2.0 * k - 1.0) * (static_cast<double>(s) * m * n) / ncycles, ops);
@@ -295,7 +272,7 @@ int main(int argc, char* argv[])
               LIBXSMM_GEMM_PREFETCH_C(ci + csize));
           }
         }
-        const unsigned long long ncycles = libxsmm_timer_diff(start, libxsmm_timer_tick());
+        const unsigned long long ncycles = libxsmm_timer_ncycles(start, libxsmm_timer_tick());
         const double duration = libxsmm_timer_duration(0, ncycles) / nrepeat;
         if (0 < duration && 0 != ncycles) {
           fprintf(stdout, "\tpseudo-perf.: %.1f %s/cycle\n", (2.0 * k - 1.0) * (static_cast<double>(s) * m * n) / ncycles, ops);
@@ -319,7 +296,7 @@ int main(int argc, char* argv[])
             smm_eigen_dynamic(m, n, k, a, bi, ci);
           }
         }
-        const unsigned long long ncycles = libxsmm_timer_diff(start, libxsmm_timer_tick());
+        const unsigned long long ncycles = libxsmm_timer_ncycles(start, libxsmm_timer_tick());
         const double duration = libxsmm_timer_duration(0, ncycles) / nrepeat;
         if (0 < duration && 0 != ncycles) {
           fprintf(stdout, "\tpseudo-perf.: %.1f %s/cycle\n", (2.0 * k - 1.0) * (static_cast<double>(s) * m * n) / ncycles, ops);
@@ -349,7 +326,7 @@ int main(int argc, char* argv[])
               LIBXSMM_GEMM_PREFETCH_C(c + j));
           }
         }
-        const unsigned long long ncycles = libxsmm_timer_diff(start, libxsmm_timer_tick());
+        const unsigned long long ncycles = libxsmm_timer_ncycles(start, libxsmm_timer_tick());
         const double duration = libxsmm_timer_duration(0, ncycles) / nrepeat;
         if (0 < duration && 0 != ncycles) {
           fprintf(stdout, "\tpseudo-perf.: %.1f %s/cycle\n", (2.0 * k - 1.0) * (static_cast<double>(s) * m * n) / ncycles, ops);
@@ -377,7 +354,7 @@ int main(int argc, char* argv[])
             smm_eigen_dynamic(m, n, k, ai, bi, c + j);
           }
         }
-        const unsigned long long ncycles = libxsmm_timer_diff(start, libxsmm_timer_tick());
+        const unsigned long long ncycles = libxsmm_timer_ncycles(start, libxsmm_timer_tick());
         const double duration = libxsmm_timer_duration(0, ncycles) / nrepeat;
         if (0 < duration && 0 != ncycles) {
           fprintf(stdout, "\tpseudo-perf.: %.1f %s/cycle\n", (2.0 * k - 1.0) * (static_cast<double>(s) * m * n) / ncycles, ops);
@@ -406,7 +383,7 @@ int main(int argc, char* argv[])
               LIBXSMM_GEMM_PREFETCH_C(c + j));
           }
         }
-        const unsigned long long ncycles = libxsmm_timer_diff(start, libxsmm_timer_tick());
+        const unsigned long long ncycles = libxsmm_timer_ncycles(start, libxsmm_timer_tick());
         const double duration = libxsmm_timer_duration(0, ncycles) / nrepeat;
         if (0 < duration && 0 != ncycles) {
           fprintf(stdout, "\tpseudo-perf.: %.1f %s/cycle\n", (2.0 * k - 1.0) * (static_cast<double>(s) * m * n) / ncycles, ops);
@@ -432,7 +409,7 @@ int main(int argc, char* argv[])
             smm_eigen_dynamic(m, n, k, a, b, c + j);
           }
         }
-        const unsigned long long ncycles = libxsmm_timer_diff(start, libxsmm_timer_tick());
+        const unsigned long long ncycles = libxsmm_timer_ncycles(start, libxsmm_timer_tick());
         const double duration = libxsmm_timer_duration(0, ncycles) / nrepeat;
         if (0 < duration && 0 != ncycles) {
           fprintf(stdout, "\tpseudo-perf.: %.1f %s/cycle\n", (2.0 * k - 1.0) * (static_cast<double>(s) * m * n) / ncycles, ops);
