@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2015-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
@@ -53,8 +33,8 @@ LIBXSMM_INLINE void print_help(void) {
   printf("    0: unaligned C, otherwise aligned\n");
   printf("    0: A normal, 1: A trans\n");
   printf("    0: B normal, 1: B trans\n");
-  printf("    PREFETCH: nopf (none), pfsigonly, BL2viaC, AL2, curAL2, AL2jpst, AL2_BL2viaC, curAL2_BL2viaC, AL2jpst_BL2viaC, AL1_BL1_CL1\n");
-  printf("    PRECISION: SP, DP, I16I32, I16F32, BF16F32, BF16\n");
+  printf("    PREFETCH: nopf (none), pfsigonly, BL2viaC, AL2, curAL2, AL2_BL2viaC, curAL2_BL2viaC\n");
+  printf("    PRECISION: SP, DP, I16I32, USI8I32, SUI8I32, SUI8UI8, BF16F32, BF16\n");
   printf("    #repetitions\n");
   printf("\n\n");
   printf("2. Usage (dense*dense=dense, performance only option available):\n");
@@ -65,7 +45,7 @@ LIBXSMM_INLINE void print_help(void) {
   printf("    0: unaligned C, otherwise aligned\n");
   printf("    0: A normal, 1: A trans\n");
   printf("    0: B normal, 1: B trans\n");
-  printf("    PRECISION: SP, DP, I16I32, I16F32, BF16F32\n");
+  printf("    PRECISION: SP, DP, I16I32, USI8I32, SUI8I32, SUI8UI8, BF16F32, BF16\n");
   printf("    #repetitions\n");
   printf("    0: no check, otherwise: run check\n");
   printf("\n\n");
@@ -100,7 +80,7 @@ double run_jit_double( const libxsmm_gemm_descriptor* i_xgemm_desc,
   }
 
   /* receive kernel information */
-  libxsmm_get_mmkernel_info(l_test_jit, &l_info, NULL/*code_size*/);
+  libxsmm_get_mmkernel_info(l_test_jit, &l_info);
 
   l_start = libxsmm_timer_tick();
   if ( l_info.prefetch == LIBXSMM_GEMM_PREFETCH_NONE ) {
@@ -151,7 +131,7 @@ double run_jit_float( const libxsmm_gemm_descriptor* i_xgemm_desc,
   }
 
   /* receive kernel information */
-  libxsmm_get_mmkernel_info(l_test_jit, &l_info, NULL/*code_size*/);
+  libxsmm_get_mmkernel_info(l_test_jit, &l_info);
 
   l_start = libxsmm_timer_tick();
   if ( l_info.prefetch == LIBXSMM_GEMM_PREFETCH_NONE ) {
@@ -202,7 +182,7 @@ double run_jit_short_int( const libxsmm_gemm_descriptor* i_xgemm_desc,
   }
 
   /* receive kernel information */
-  libxsmm_get_mmkernel_info(l_test_jit, &l_info, NULL/*code_size*/);
+  libxsmm_get_mmkernel_info(l_test_jit, &l_info);
 
   l_start = libxsmm_timer_tick();
   if (l_info.prefetch == LIBXSMM_GEMM_PREFETCH_NONE ) {
@@ -226,12 +206,11 @@ double run_jit_short_int( const libxsmm_gemm_descriptor* i_xgemm_desc,
 
 
 LIBXSMM_INLINE
-double run_jit_short_float( const libxsmm_gemm_descriptor* i_xgemm_desc,
-                            const short*                   i_a,
-                            const short*                   i_b,
-                            float*                         o_c,
-                            float*                         i_scf,
-                            const unsigned int             i_print_jit_info ) {
+double run_jit_uschar_int( const libxsmm_gemm_descriptor* i_xgemm_desc,
+                           const unsigned char*           i_a,
+                           const char*                    i_b,
+                           int*                           o_c,
+                           const unsigned int             i_print_jit_info ) {
   /* define function pointer */
   libxsmm_xmmfunction l_test_jit;
   libxsmm_timer_tickint l_start;
@@ -254,16 +233,16 @@ double run_jit_short_float( const libxsmm_gemm_descriptor* i_xgemm_desc,
   }
 
   /* receive kernel information */
-  libxsmm_get_mmkernel_info(l_test_jit, &l_info, NULL/*code_size*/);
+  libxsmm_get_mmkernel_info(l_test_jit, &l_info);
 
   l_start = libxsmm_timer_tick();
   if (l_info.prefetch == LIBXSMM_GEMM_PREFETCH_NONE ) {
     for ( l_t = 0; l_t < g_reps; l_t++ ) {
-      l_test_jit.wsmm(i_a, i_b, o_c, NULL, NULL, NULL, i_scf);
+      l_test_jit.usbimm(i_a, i_b, o_c, NULL, NULL, NULL);
     }
   } else {
     for ( l_t = 0; l_t < g_reps; l_t++ ) {
-      l_test_jit.wsmm(i_a, i_b, o_c, i_a, i_b, o_c, i_scf);
+      l_test_jit.usbimm(i_a, i_b, o_c, i_a, i_b, o_c);
     }
   }
   l_runtime = libxsmm_timer_duration(l_start, libxsmm_timer_tick());
@@ -277,6 +256,161 @@ double run_jit_short_float( const libxsmm_gemm_descriptor* i_xgemm_desc,
 }
 
 
+LIBXSMM_INLINE
+double run_jit_suchar_int( const libxsmm_gemm_descriptor* i_xgemm_desc,
+                           const char*                    i_a,
+                           const unsigned char*           i_b,
+                           int*                           o_c,
+                           const unsigned int             i_print_jit_info ) {
+  /* define function pointer */
+  libxsmm_xmmfunction l_test_jit;
+  libxsmm_timer_tickint l_start;
+  libxsmm_mmkernel_info l_info;
+  double l_jittime, l_runtime;
+  int l_t;
+
+  if (0 == i_xgemm_desc) {
+    fprintf(stderr, "JIT: unsupported descriptor arguments or data type!\n");
+    return EXIT_FAILURE;
+  }
+
+  l_start = libxsmm_timer_tick();
+  l_test_jit = libxsmm_xmmdispatch(i_xgemm_desc);
+  l_jittime = libxsmm_timer_duration(l_start, libxsmm_timer_tick());
+
+  if (l_test_jit.xmm == 0) {
+    printf("JIT failed, please run with LIBXSMM_VERBOSE=-1 and/or with debug mode LIBXSMM library!\n");
+    exit(EXIT_FAILURE);
+  }
+
+  /* receive kernel information */
+  libxsmm_get_mmkernel_info(l_test_jit, &l_info);
+
+  l_start = libxsmm_timer_tick();
+  if (l_info.prefetch == LIBXSMM_GEMM_PREFETCH_NONE ) {
+    for ( l_t = 0; l_t < g_reps; l_t++ ) {
+      l_test_jit.subimm(i_a, i_b, o_c, NULL, NULL, NULL);
+    }
+  } else {
+    for ( l_t = 0; l_t < g_reps; l_t++ ) {
+      l_test_jit.subimm(i_a, i_b, o_c, i_a, i_b, o_c);
+    }
+  }
+  l_runtime = libxsmm_timer_duration(l_start, libxsmm_timer_tick());
+
+  if ( i_print_jit_info == 0 ) {
+    printf("function pointer address: %llx\n", (unsigned long long)l_test_jit.xmm);
+    printf("%fs for creating jit\n", l_jittime);
+  }
+
+  return l_runtime;
+}
+
+#if 0
+LIBXSMM_INLINE
+double run_jit_uschar_uchar( const libxsmm_gemm_descriptor* i_xgemm_desc,
+                             const unsigned char*           i_a,
+                             const char*                    i_b,
+                             unsigned char*                 o_c,
+                             const unsigned int             i_print_jit_info ) {
+  /* define function pointer */
+  libxsmm_xmmfunction l_test_jit;
+  libxsmm_timer_tickint l_start;
+  libxsmm_mmkernel_info l_info;
+  double l_jittime, l_runtime;
+  int l_t;
+  float l_scf = 1.0;
+
+  if (0 == i_xgemm_desc) {
+    fprintf(stderr, "JIT: unsupported descriptor arguments or data type!\n");
+    return EXIT_FAILURE;
+  }
+
+  l_start = libxsmm_timer_tick();
+  l_test_jit = libxsmm_xmmdispatch(i_xgemm_desc);
+  l_jittime = libxsmm_timer_duration(l_start, libxsmm_timer_tick());
+
+  if (l_test_jit.xmm == 0) {
+    printf("JIT failed, please run with LIBXSMM_VERBOSE=-1 and/or with debug mode LIBXSMM library!\n");
+    exit(EXIT_FAILURE);
+  }
+
+  /* receive kernel information */
+  libxsmm_get_mmkernel_info(l_test_jit, &l_info);
+
+  l_start = libxsmm_timer_tick();
+  if (l_info.prefetch == LIBXSMM_GEMM_PREFETCH_NONE ) {
+    for ( l_t = 0; l_t < g_reps; l_t++ ) {
+      l_test_jit.usbubmm(i_a, i_b, o_c, &l_scf, NULL, NULL, NULL);
+    }
+  } else {
+    for ( l_t = 0; l_t < g_reps; l_t++ ) {
+      l_test_jit.usbubmm(i_a, i_b, o_c, &l_scf, i_a, i_b, o_c);
+    }
+  }
+  l_runtime = libxsmm_timer_duration(l_start, libxsmm_timer_tick());
+
+  if ( i_print_jit_info == 0 ) {
+    printf("function pointer address: %llx\n", (unsigned long long)l_test_jit.xmm);
+    printf("%fs for creating jit\n", l_jittime);
+  }
+
+  return l_runtime;
+}
+#endif
+
+
+LIBXSMM_INLINE
+double run_jit_suchar_uchar( const libxsmm_gemm_descriptor* i_xgemm_desc,
+                             const char*                    i_a,
+                             const unsigned char*           i_b,
+                             unsigned char*                 o_c,
+                             float                          i_scf,
+                             const unsigned int             i_print_jit_info ) {
+  /* define function pointer */
+  libxsmm_xmmfunction l_test_jit;
+  libxsmm_timer_tickint l_start;
+  libxsmm_mmkernel_info l_info;
+  double l_jittime, l_runtime;
+  int l_t;
+
+  if (0 == i_xgemm_desc) {
+    fprintf(stderr, "JIT: unsupported descriptor arguments or data type!\n");
+    return EXIT_FAILURE;
+  }
+
+  l_start = libxsmm_timer_tick();
+  l_test_jit = libxsmm_xmmdispatch(i_xgemm_desc);
+  l_jittime = libxsmm_timer_duration(l_start, libxsmm_timer_tick());
+
+  if (l_test_jit.xmm == 0) {
+    printf("JIT failed, please run with LIBXSMM_VERBOSE=-1 and/or with debug mode LIBXSMM library!\n");
+    exit(EXIT_FAILURE);
+  }
+
+  /* receive kernel information */
+  libxsmm_get_mmkernel_info(l_test_jit, &l_info);
+
+  l_start = libxsmm_timer_tick();
+  if (l_info.prefetch == LIBXSMM_GEMM_PREFETCH_NONE ) {
+    for ( l_t = 0; l_t < g_reps; l_t++ ) {
+      l_test_jit.sububmm(i_a, i_b, o_c, &i_scf, NULL, NULL, NULL);
+    }
+  } else {
+    for ( l_t = 0; l_t < g_reps; l_t++ ) {
+      l_test_jit.sububmm(i_a, i_b, o_c, &i_scf, i_a, i_b, o_c);
+    }
+  }
+  l_runtime = libxsmm_timer_duration(l_start, libxsmm_timer_tick());
+
+  if ( i_print_jit_info == 0 ) {
+    printf("function pointer address: %llx\n", (unsigned long long)l_test_jit.xmm);
+    printf("%fs for creating jit\n", l_jittime);
+  }
+
+  return l_runtime;
+}
+
 LIBXSMM_INLINE
 double run_jit_bfloat16_float( const libxsmm_gemm_descriptor* i_xgemm_desc,
                                const libxsmm_bfloat16*        i_a,
@@ -305,7 +439,7 @@ double run_jit_bfloat16_float( const libxsmm_gemm_descriptor* i_xgemm_desc,
   }
 
   /* receive kernel information */
-  libxsmm_get_mmkernel_info(l_test_jit, &l_info, NULL/*code_size*/);
+  libxsmm_get_mmkernel_info(l_test_jit, &l_info);
 
   l_start = libxsmm_timer_tick();
   if (l_info.prefetch == LIBXSMM_GEMM_PREFETCH_NONE ) {
@@ -356,7 +490,7 @@ double run_jit_bfloat16( const libxsmm_gemm_descriptor* i_xgemm_desc,
   }
 
   /* receive kernel information */
-  libxsmm_get_mmkernel_info(l_test_jit, &l_info, NULL/*code_size*/);
+  libxsmm_get_mmkernel_info(l_test_jit, &l_info);
 
   l_start = libxsmm_timer_tick();
   if (l_info.prefetch == LIBXSMM_GEMM_PREFETCH_NONE ) {
@@ -408,23 +542,25 @@ int main(int argc, char* argv []) {
   float *l_a_f = 0, *l_b_f = 0, *l_c_f = 0;
   short *l_a_w = 0, *l_b_w = 0;
   libxsmm_bfloat16 *l_a_bf = 0, *l_b_bf = 0, *l_c_bf = 0;
-  unsigned char* l_a_b = 0;
-  char* l_b_b = 0;
-  int* l_c_b = 0;
-  float* l_c_w_f = 0;
+  unsigned char *l_ua_b = 0, *l_ub_b;
+  char *l_sa_b = 0, *l_sb_b = 0;
+  int* l_c_b_i = 0;
   int* l_c_w_i = 0;
+  unsigned char* l_c_b_ub = 0;
+  float* l_c_bf_f = 0;
   /* Gold data */
   double* l_c_gold_d = 0;
   float* l_c_gold_f = 0;
-  int* l_c_gold_b = 0;
-  float* l_c_gold_w_f = 0;
   libxsmm_bfloat16* l_c_gold_bf = 0;
-  unsigned char exp_a = 0, exp_b = 0;
-  float l_scf = libxsmm_sexp2(-1.f*((float)exp_a + (float)exp_b));
-  /*l_scf = 1000;*/
   int* l_c_gold_w_i = 0;
+  int* l_c_gold_b_i = 0;
+  unsigned char* l_c_gold_b_ub = 0;
+  float* l_c_gold_bf_f = 0;
   double l_total_max_error = 0.0;
 
+  /* scaling factor */
+  float l_scf = 1.0;
+
   libxsmm_matdiff_clear(&l_diff);
 
   /* check argument count for a valid range */
@@ -476,15 +612,6 @@ int main(int argc, char* argv []) {
     else if (strcmp("AL2_BL2viaC", argv[13]) == 0) {
       l_prefetch = LIBXSMM_GEMM_PREFETCH_AL2BL2_VIA_C;
     }
-    else if (strcmp("AL2jpst", argv[13]) == 0) {
-      l_prefetch = LIBXSMM_GEMM_PREFETCH_AL2_JPST;
-    }
-    else if (strcmp("AL2jpst_BL2viaC", argv[13]) == 0) {
-      l_prefetch = LIBXSMM_GEMM_PREFETCH_AL2BL2_VIA_C_JPST;
-    }
-    else if (strcmp("AL1_BL1_CL1", argv[13]) == 0) {
-      l_prefetch = LIBXSMM_GEMM_PREFETCH_AL1_BL1_CL1;
-    }
     else {
       print_help();
       return EXIT_FAILURE;
@@ -883,6 +1010,7 @@ int main(int argc, char* argv []) {
         }
         if ( 6 != sscanf( l_line, "%i %i %i %i %i %i", &l_m, &l_n, &l_k, &l_lda, &l_ldb, &l_ldc ) ) exit(EXIT_FAILURE);
       }
+      l_flags |= LIBXSMM_GEMM_FLAG_VNNI_A;
       l_xgemm_desc = libxsmm_gemm_descriptor_dinit2(&l_xgemm_blob,
         LIBXSMM_GEMM_PRECISION_I16, LIBXSMM_GEMM_PRECISION_I32,
         l_m, l_n, l_k, l_lda, l_ldb, l_ldc, l_alpha, l_beta, l_flags, l_prefetch);
@@ -959,8 +1087,8 @@ int main(int argc, char* argv []) {
       libxsmm_free(l_c_w_i);
       libxsmm_free(l_c_gold_w_i);
     } while ( l_keep_going );
-  } else if (strcmp(l_precision, "I16F32") == 0) {
-    const int l_k_block = 2;
+  } else if (strcmp(l_precision, "USI8I32") == 0) {
+    const int l_k_block = 4;
     double l_max_error = 0;
     int l_k2;
     unsigned int l_keep_going = 0;
@@ -975,34 +1103,36 @@ int main(int argc, char* argv []) {
         }
         if ( 6 != sscanf( l_line, "%i %i %i %i %i %i", &l_m, &l_n, &l_k, &l_lda, &l_ldb, &l_ldc ) ) exit(EXIT_FAILURE);
       }
+      l_flags |= LIBXSMM_GEMM_FLAG_A_UNSIGNED | LIBXSMM_GEMM_FLAG_VNNI_A;
       l_xgemm_desc = libxsmm_gemm_descriptor_dinit2(&l_xgemm_blob,
-        LIBXSMM_GEMM_PRECISION_I16, LIBXSMM_GEMM_PRECISION_F32,
+        LIBXSMM_GEMM_PRECISION_I8, LIBXSMM_GEMM_PRECISION_I32,
         l_m, l_n, l_k, l_lda, l_ldb, l_ldc, l_alpha, l_beta, l_flags, l_prefetch);
-      l_a_w = (short*)libxsmm_aligned_malloc((size_t)l_lda * (size_t)l_k * sizeof(short), 64);
-      l_b_w = (short*)libxsmm_aligned_malloc((size_t)l_ldb * (size_t)l_n * sizeof(short), 64);
-      l_c_w_f = (float*)libxsmm_aligned_malloc((size_t)l_ldc * (size_t)l_n * sizeof(float), 64);
-      l_c_gold_w_f = (float*)libxsmm_aligned_malloc((size_t)l_ldc * (size_t)l_n * sizeof(float), 64);
+      l_ua_b = (unsigned char*)libxsmm_aligned_malloc((size_t)l_lda * (size_t)l_k * sizeof(unsigned char), 64);
+      l_sb_b = (char*)libxsmm_aligned_malloc((size_t)l_ldb * (size_t)l_n * sizeof(char), 64);
+      l_c_b_i = (int*)libxsmm_aligned_malloc((size_t)l_ldc * (size_t)l_n * sizeof(int), 64);
+      l_c_gold_b_i = (int*)libxsmm_aligned_malloc((size_t)l_ldc * (size_t)l_n * sizeof(int), 64);
+
       /* touch A */
       for (l_i = 0; l_i < l_lda; l_i++) {
         for (l_j = 0; l_j < l_k; l_j++) {
-          l_a_w[(l_j * l_lda) + l_i] = (short)(libxsmm_rng_f64() * 10.0);
+          l_ua_b[(l_j * l_lda) + l_i] = (unsigned char)(libxsmm_rng_f64() * 5.0);
         }
       }
       /* touch B */
       for (l_i = 0; l_i < l_ldb; l_i++) {
         for (l_j = 0; l_j < l_n; l_j++) {
-          l_b_w[(l_j * l_ldb) + l_i] = (short)(libxsmm_rng_f64() * 10.0);
+          l_sb_b[(l_j * l_ldb) + l_i] = (char)(libxsmm_rng_f64() * 5.0);
         }
       }
       /* touch C */
       for (l_i = 0; l_i < l_ldc; l_i++) {
         for (l_j = 0; l_j < l_n; l_j++) {
-          l_c_w_f[(l_j * l_ldc) + l_i] = 0.0f;
-          l_c_gold_w_f[(l_j * l_ldc) + l_i] = 0.0f;
+          l_c_b_i[(l_j * l_ldc) + l_i] = 0;
+          l_c_gold_b_i[(l_j * l_ldc) + l_i] = 0;
         }
       }
 
-      l_runtime_libxsmm = run_jit_short_float(l_xgemm_desc, l_a_w, l_b_w, l_c_w_f, &l_scf, l_file_input );
+      l_runtime_libxsmm = run_jit_uschar_int(l_xgemm_desc, l_ua_b, l_sb_b, l_c_b_i, l_file_input );
 
       if ( l_run_check == 1 ) {
         l_start = libxsmm_timer_tick();
@@ -1011,9 +1141,7 @@ int main(int argc, char* argv []) {
             for (l_s = 0; l_s < (l_k / l_k_block); l_s++) {
               for (l_i = 0; l_i < l_m; l_i++) {
                 for (l_k2 = 0; l_k2 < l_k_block; l_k2++) {
-                  const int iprod = (int)l_a_w[(l_s * (l_lda*l_k_block)) + (l_i*l_k_block) + l_k2] * (int)l_b_w[(l_j * l_ldb) + (l_s*l_k_block) + l_k2];
-                  const float fprod = (float)iprod;
-                  l_c_gold_w_f[(l_j * l_ldc) + l_i] += fprod * l_scf;
+                  l_c_gold_b_i[(l_j * l_ldc) + l_i] += l_ua_b[(l_s * (l_lda*l_k_block)) + (l_i*l_k_block) + l_k2] * l_sb_b[(l_j * l_ldb) + (l_s*l_k_block) + l_k2];
                 }
               }
             }
@@ -1023,7 +1151,7 @@ int main(int argc, char* argv []) {
         l_max_error = 0;
         for (l_i = 0; l_i < l_m; l_i++) {
           for (l_j = 0; l_j < l_n; l_j++) {
-            const double l_fabs = fabs((double)l_c_gold_w_f[(l_j * l_ldc) + l_i] - (double)l_c_w_f[(l_j * l_ldc) + l_i]);
+            const double l_fabs = fabs((double)l_c_gold_b_i[(l_j * l_ldc) + l_i] - (double)l_c_b_i[(l_j * l_ldc) + l_i]);
             if (l_max_error < l_fabs) l_max_error = l_fabs;
           }
         }
@@ -1047,12 +1175,203 @@ int main(int argc, char* argv []) {
         l_total_max_error = l_max_error;
       }
 
-      libxsmm_free(l_a_w);
-      libxsmm_free(l_b_w);
-      libxsmm_free(l_c_w_f);
-      libxsmm_free(l_c_gold_w_f);
+      libxsmm_free(l_ua_b);
+      libxsmm_free(l_sb_b);
+      libxsmm_free(l_c_b_i);
+      libxsmm_free(l_c_gold_b_i);
     } while ( l_keep_going );
-  } else if (strcmp(l_precision, "BF16F32") == 0) {
+  } else if (strcmp(l_precision, "SUI8I32") == 0) {
+    const int l_k_block = 4;
+    double l_max_error = 0;
+    int l_k2;
+    unsigned int l_keep_going = 0;
+    do {
+      if ( l_file_input != 0 ) {
+        char l_line[512];
+        if ( fgets( l_line, 512, l_file_handle) == NULL ) {
+          l_keep_going = 0;
+          break;
+        } else {
+          l_keep_going = 1;
+        }
+        if ( 6 != sscanf( l_line, "%i %i %i %i %i %i", &l_m, &l_n, &l_k, &l_lda, &l_ldb, &l_ldc ) ) exit(EXIT_FAILURE);
+      }
+      l_flags |= LIBXSMM_GEMM_FLAG_B_UNSIGNED | LIBXSMM_GEMM_FLAG_VNNI_A;
+      l_xgemm_desc = libxsmm_gemm_descriptor_dinit2(&l_xgemm_blob,
+        LIBXSMM_GEMM_PRECISION_I8, LIBXSMM_GEMM_PRECISION_I32,
+        l_m, l_n, l_k, l_lda, l_ldb, l_ldc, l_alpha, l_beta, l_flags, l_prefetch);
+      l_sa_b = (char*)libxsmm_aligned_malloc((size_t)l_lda * (size_t)l_k * sizeof(char), 64);
+      l_ub_b = (unsigned char*)libxsmm_aligned_malloc((size_t)l_ldb * (size_t)l_n * sizeof(unsigned char), 64);
+      l_c_b_i = (int*)libxsmm_aligned_malloc((size_t)l_ldc * (size_t)l_n * sizeof(int), 64);
+      l_c_gold_b_i = (int*)libxsmm_aligned_malloc((size_t)l_ldc * (size_t)l_n * sizeof(int), 64);
+
+      /* touch A */
+      for (l_i = 0; l_i < l_lda; l_i++) {
+        for (l_j = 0; l_j < l_k; l_j++) {
+          l_sa_b[(l_j * l_lda) + l_i] = (char)(libxsmm_rng_f64() * 5.0);
+        }
+      }
+      /* touch B */
+      for (l_i = 0; l_i < l_ldb; l_i++) {
+        for (l_j = 0; l_j < l_n; l_j++) {
+          l_ub_b[(l_j * l_ldb) + l_i] = (unsigned char)(libxsmm_rng_f64() * 5.0);
+        }
+      }
+      /* touch C */
+      for (l_i = 0; l_i < l_ldc; l_i++) {
+        for (l_j = 0; l_j < l_n; l_j++) {
+          l_c_b_i[(l_j * l_ldc) + l_i] = 0;
+          l_c_gold_b_i[(l_j * l_ldc) + l_i] = 0;
+        }
+      }
+
+      l_runtime_libxsmm = run_jit_suchar_int(l_xgemm_desc, l_sa_b, l_ub_b, l_c_b_i, l_file_input );
+
+      if ( l_run_check == 1 ) {
+        l_start = libxsmm_timer_tick();
+        for (l_t = 0; l_t < g_reps; l_t++) {
+          for (l_j = 0; l_j < l_n; l_j++) {
+            for (l_s = 0; l_s < (l_k / l_k_block); l_s++) {
+              for (l_i = 0; l_i < l_m; l_i++) {
+                for (l_k2 = 0; l_k2 < l_k_block; l_k2++) {
+                  l_c_gold_b_i[(l_j * l_ldc) + l_i] += l_sa_b[(l_s * (l_lda*l_k_block)) + (l_i*l_k_block) + l_k2] * l_ub_b[(l_j * l_ldb) + (l_s*l_k_block) + l_k2];
+                }
+              }
+            }
+          }
+        }
+        l_runtime_c = libxsmm_timer_duration(l_start, libxsmm_timer_tick());
+        l_max_error = 0;
+        for (l_i = 0; l_i < l_m; l_i++) {
+          for (l_j = 0; l_j < l_n; l_j++) {
+            const double l_fabs = fabs((double)l_c_gold_b_i[(l_j * l_ldc) + l_i] - (double)l_c_b_i[(l_j * l_ldc) + l_i]);
+            if (l_max_error < l_fabs) l_max_error = l_fabs;
+          }
+        }
+      }
+
+      if ( l_file_input == 0 ) {
+        printf("%fs for C\n", l_runtime_c);
+        printf("%f GFLOPS for C\n", ((double)((double)g_reps * (double)l_m * (double)l_n * (double)l_k) * 2.0) / (l_runtime_c * 1.0e9));
+        printf("%fs for libxsmm\n", l_runtime_libxsmm);
+        printf("%f GFLOPS for libxsmm\n", ((double)((double)g_reps * (double)l_m * (double)l_n * (double)l_k) * 2.0) / (l_runtime_libxsmm * 1.0e9));
+        printf("max. error: %f\n", l_max_error);
+      } else {
+        if ( l_run_check == 1 ) {
+          printf("%i %i %i %i %i %i %f %f\n", l_m, l_n, l_k, l_lda, l_ldb, l_ldc, ((double)((double)g_reps * (double)l_m * (double)l_n * (double)l_k) * 2.0) / (l_runtime_libxsmm * 1.0e9), l_max_error );
+        } else {
+          printf("%i %i %i %i %i %i %f\n", l_m, l_n, l_k, l_lda, l_ldb, l_ldc, ((double)((double)g_reps * (double)l_m * (double)l_n * (double)l_k) * 2.0) / (l_runtime_libxsmm * 1.0e9) );
+        }
+      }
+
+      if ( (l_total_max_error < l_max_error) && (l_run_check == 1) ) {
+        l_total_max_error = l_max_error;
+      }
+
+      libxsmm_free(l_sa_b);
+      libxsmm_free(l_ub_b);
+      libxsmm_free(l_c_b_i);
+      libxsmm_free(l_c_gold_b_i);
+    } while ( l_keep_going );
+  } else if (strcmp(l_precision, "SUI8UI8") == 0) {
+    const int l_k_block = 4;
+    double l_max_error = 0;
+    int l_k2;
+    unsigned int l_keep_going = 0;
+    do {
+      if ( l_file_input != 0 ) {
+        char l_line[512];
+        if ( fgets( l_line, 512, l_file_handle) == NULL ) {
+          l_keep_going = 0;
+          break;
+        } else {
+          l_keep_going = 1;
+        }
+        if ( 6 != sscanf( l_line, "%i %i %i %i %i %i", &l_m, &l_n, &l_k, &l_lda, &l_ldb, &l_ldc ) ) exit(EXIT_FAILURE);
+      }
+      l_flags |= LIBXSMM_GEMM_FLAG_B_UNSIGNED | LIBXSMM_GEMM_FLAG_C_UNSIGNED | LIBXSMM_GEMM_FLAG_VNNI_A;
+      l_xgemm_desc = libxsmm_gemm_descriptor_dinit2(&l_xgemm_blob,
+        LIBXSMM_GEMM_PRECISION_I8, LIBXSMM_GEMM_PRECISION_I8,
+        l_m, l_n, l_k, l_lda, l_ldb, l_ldc, l_alpha, l_beta, l_flags, l_prefetch);
+      l_sa_b = (char*)libxsmm_aligned_malloc((size_t)l_lda * (size_t)l_k * sizeof(char), 64);
+      l_ub_b = (unsigned char*)libxsmm_aligned_malloc((size_t)l_ldb * (size_t)l_n * sizeof(unsigned char), 64);
+      l_c_b_ub = (unsigned char*)libxsmm_aligned_malloc((size_t)l_ldc * (size_t)l_n * sizeof(unsigned char), 64);
+      l_c_gold_b_ub = (unsigned char*)libxsmm_aligned_malloc((size_t)l_ldc * (size_t)l_n * sizeof(unsigned char), 64);
+
+      /* touch A */
+      for (l_i = 0; l_i < l_lda; l_i++) {
+        for (l_j = 0; l_j < l_k; l_j++) {
+          l_sa_b[(l_j * l_lda) + l_i] = (char)(libxsmm_rng_f64() * 2.0);
+        }
+      }
+      /* touch B */
+      for (l_i = 0; l_i < l_ldb; l_i++) {
+        for (l_j = 0; l_j < l_n; l_j++) {
+          l_ub_b[(l_j * l_ldb) + l_i] = (unsigned char)(libxsmm_rng_f64() * 2.0);
+        }
+      }
+      /* touch C */
+      for (l_i = 0; l_i < l_ldc; l_i++) {
+        for (l_j = 0; l_j < l_n; l_j++) {
+          l_c_b_ub[(l_j * l_ldc) + l_i] = 0;
+          l_c_gold_b_ub[(l_j * l_ldc) + l_i] = 0;
+        }
+      }
+
+      l_runtime_libxsmm = run_jit_suchar_uchar(l_xgemm_desc, l_sa_b, l_ub_b, l_c_b_ub, l_scf, l_file_input );
+
+      if ( l_run_check == 1 ) {
+        l_start = libxsmm_timer_tick();
+        for (l_t = 0; l_t < g_reps; l_t++) {
+          for (l_j = 0; l_j < l_n; l_j++) {
+            for (l_i = 0; l_i < l_m; l_i++) {
+              int tmp = (int)l_c_gold_b_ub[(l_j * l_ldc) + l_i];
+              float ftmp;
+              for (l_s = 0; l_s < (l_k / l_k_block); l_s++) {
+                for (l_k2 = 0; l_k2 < l_k_block; l_k2++) {
+                  tmp += l_sa_b[(l_s * (l_lda*l_k_block)) + (l_i*l_k_block) + l_k2] * l_ub_b[(l_j * l_ldb) + (l_s*l_k_block) + l_k2];
+                }
+              }
+              ftmp = (float)tmp;
+              ftmp *= l_scf;
+              l_c_gold_b_ub[(l_j * l_ldc) + l_i] = (unsigned char)ftmp;
+            }
+          }
+        }
+        l_runtime_c = libxsmm_timer_duration(l_start, libxsmm_timer_tick());
+        l_max_error = 0;
+        for (l_i = 0; l_i < l_m; l_i++) {
+          for (l_j = 0; l_j < l_n; l_j++) {
+            const double l_fabs = fabs((double)l_c_gold_b_ub[(l_j * l_ldc) + l_i] - (double)l_c_b_ub[(l_j * l_ldc) + l_i]);
+            if (l_max_error < l_fabs) l_max_error = l_fabs;
+          }
+        }
+      }
+
+      if ( l_file_input == 0 ) {
+        printf("%fs for C\n", l_runtime_c);
+        printf("%f GFLOPS for C\n", ((double)((double)g_reps * (double)l_m * (double)l_n * (double)l_k) * 2.0) / (l_runtime_c * 1.0e9));
+        printf("%fs for libxsmm\n", l_runtime_libxsmm);
+        printf("%f GFLOPS for libxsmm\n", ((double)((double)g_reps * (double)l_m * (double)l_n * (double)l_k) * 2.0) / (l_runtime_libxsmm * 1.0e9));
+        printf("max. error: %f\n", l_max_error);
+      } else {
+        if ( l_run_check == 1 ) {
+          printf("%i %i %i %i %i %i %f %f\n", l_m, l_n, l_k, l_lda, l_ldb, l_ldc, ((double)((double)g_reps * (double)l_m * (double)l_n * (double)l_k) * 2.0) / (l_runtime_libxsmm * 1.0e9), l_max_error );
+        } else {
+          printf("%i %i %i %i %i %i %f\n", l_m, l_n, l_k, l_lda, l_ldb, l_ldc, ((double)((double)g_reps * (double)l_m * (double)l_n * (double)l_k) * 2.0) / (l_runtime_libxsmm * 1.0e9) );
+        }
+      }
+
+      if ( (l_total_max_error < l_max_error) && (l_run_check == 1) ) {
+        l_total_max_error = l_max_error;
+      }
+
+      libxsmm_free(l_sa_b);
+      libxsmm_free(l_ub_b);
+      libxsmm_free(l_c_b_ub);
+      libxsmm_free(l_c_gold_b_ub);
+    } while ( l_keep_going );
+   } else if (strcmp(l_precision, "BF16F32") == 0) {
     const int l_k_block = 2;
     double l_max_error = 0;
     int l_k2;
@@ -1068,13 +1387,14 @@ int main(int argc, char* argv []) {
         }
         if ( 6 != sscanf( l_line, "%i %i %i %i %i %i", &l_m, &l_n, &l_k, &l_lda, &l_ldb, &l_ldc ) ) exit(EXIT_FAILURE);
       }
+      l_flags |= LIBXSMM_GEMM_FLAG_VNNI_A;
       l_xgemm_desc = libxsmm_gemm_descriptor_dinit2(&l_xgemm_blob,
         LIBXSMM_GEMM_PRECISION_BF16, LIBXSMM_GEMM_PRECISION_F32,
         l_m, l_n, l_k, l_lda, l_ldb, l_ldc, l_alpha, l_beta, l_flags, l_prefetch);
       l_a_bf = (libxsmm_bfloat16*)libxsmm_aligned_malloc((size_t)l_lda * (size_t)l_k * sizeof(libxsmm_bfloat16), 64);
       l_b_bf = (libxsmm_bfloat16*)libxsmm_aligned_malloc((size_t)l_ldb * (size_t)l_n * sizeof(libxsmm_bfloat16), 64);
-      l_c_w_f = (float*)libxsmm_aligned_malloc((size_t)l_ldc * (size_t)l_n * sizeof(float), 64);
-      l_c_gold_w_f = (float*)libxsmm_aligned_malloc((size_t)l_ldc * (size_t)l_n * sizeof(float), 64);
+      l_c_bf_f = (float*)libxsmm_aligned_malloc((size_t)l_ldc * (size_t)l_n * sizeof(float), 64);
+      l_c_gold_bf_f = (float*)libxsmm_aligned_malloc((size_t)l_ldc * (size_t)l_n * sizeof(float), 64);
       /* touch A */
       for (l_i = 0; l_i < l_lda; l_i++) {
         for (l_j = 0; l_j < l_k; l_j++) {
@@ -1094,12 +1414,12 @@ int main(int argc, char* argv []) {
       /* touch C */
       for (l_i = 0; l_i < l_ldc; l_i++) {
         for (l_j = 0; l_j < l_n; l_j++) {
-          l_c_w_f[(l_j * l_ldc) + l_i] = 0.0f;
-          l_c_gold_w_f[(l_j * l_ldc) + l_i] = 0.0f;
+          l_c_bf_f[(l_j * l_ldc) + l_i] = 0.0f;
+          l_c_gold_bf_f[(l_j * l_ldc) + l_i] = 0.0f;
         }
       }
 
-      l_runtime_libxsmm = run_jit_bfloat16_float(l_xgemm_desc, l_a_bf, l_b_bf, l_c_w_f, l_file_input );
+      l_runtime_libxsmm = run_jit_bfloat16_float(l_xgemm_desc, l_a_bf, l_b_bf, l_c_bf_f, l_file_input );
 
       if ( l_run_check == 1 ) {
         l_start = libxsmm_timer_tick();
@@ -1114,7 +1434,7 @@ int main(int argc, char* argv []) {
                   tmp_a_f.i[0] = 0;
                   tmp_b_f.i[1] = l_b_bf[(l_j * l_ldb) + (l_s*l_k_block) + l_k2];
                   tmp_b_f.i[0] = 0;
-                  l_c_gold_w_f[(l_j * l_ldc) + l_i] += (float)(tmp_a_f.f * tmp_b_f.f);
+                  l_c_gold_bf_f[(l_j * l_ldc) + l_i] += (float)(tmp_a_f.f * tmp_b_f.f);
                 }
               }
             }
@@ -1124,7 +1444,7 @@ int main(int argc, char* argv []) {
         l_max_error = 0;
         for (l_i = 0; l_i < l_m; l_i++) {
           for (l_j = 0; l_j < l_n; l_j++) {
-            const double l_fabs = fabs((double)l_c_gold_w_f[(l_j * l_ldc) + l_i] - (double)l_c_w_f[(l_j * l_ldc) + l_i]);
+            const double l_fabs = fabs((double)l_c_gold_bf_f[(l_j * l_ldc) + l_i] - (double)l_c_bf_f[(l_j * l_ldc) + l_i]);
             if (l_max_error < l_fabs) l_max_error = l_fabs;
           }
         }
@@ -1150,8 +1470,8 @@ int main(int argc, char* argv []) {
 
       libxsmm_free(l_a_bf);
       libxsmm_free(l_b_bf);
-      libxsmm_free(l_c_w_f);
-      libxsmm_free(l_c_gold_w_f);
+      libxsmm_free(l_c_bf_f);
+      libxsmm_free(l_c_gold_bf_f);
     } while ( l_keep_going );
   } else if (strcmp(l_precision, "BF16") == 0) {
     const int l_k_block = 2;
@@ -1169,6 +1489,7 @@ int main(int argc, char* argv []) {
         }
         if ( 6 != sscanf( l_line, "%i %i %i %i %i %i", &l_m, &l_n, &l_k, &l_lda, &l_ldb, &l_ldc ) ) exit(EXIT_FAILURE);
       }
+      l_flags |= LIBXSMM_GEMM_FLAG_VNNI_A;
       l_xgemm_desc = libxsmm_gemm_descriptor_dinit2(&l_xgemm_blob,
         LIBXSMM_GEMM_PRECISION_BF16, LIBXSMM_GEMM_PRECISION_BF16,
         l_m, l_n, l_k, l_lda, l_ldb, l_ldc, l_alpha, l_beta, l_flags, l_prefetch);
@@ -1265,42 +1586,10 @@ int main(int argc, char* argv []) {
 
       libxsmm_free(l_a_bf);
       libxsmm_free(l_b_bf);
-      libxsmm_free(l_c_w_f);
-      libxsmm_free(l_c_gold_w_f);
+      libxsmm_free(l_c_bf);
+      libxsmm_free(l_c_gold_bf);
     } while ( l_keep_going );
   }
-#if 0
-  else if (strcmp(l_precision, "I8") == 0) {
-    l_xgemm_desc = libxsmm_gemm_descriptor_dinit2(&l_xgemm_blob,
-      LIBXSMM_GEMM_PRECISION_I8, LIBXSMM_GEMM_PRECISION_I32,
-      l_m, l_n, l_k, l_lda, l_ldb, l_ldc, l_alpha, l_beta, l_flags, l_prefetch);
-    l_a_b = (unsigned char*)libxsmm_aligned_malloc((size_t)l_lda * (size_t)l_k * sizeof(unsigned char), 64);
-    l_b_b = (char*)libxsmm_aligned_malloc((size_t)l_ldb * (size_t)l_n * sizeof(char), 64);
-    l_c_b = (int*)libxsmm_aligned_malloc((size_t)l_ldc * (size_t)l_n * sizeof(int), 64);
-    l_c_gold_b = (int*)libxsmm_aligned_malloc((size_t)l_ldc * (size_t)l_n * sizeof(int), 64);
-    /* touch A */
-    for (l_i = 0; l_i < l_lda; l_i++) {
-      for (l_j = 0; l_j < (l_k / 2); l_j++) {
-        l_a_b[(l_j * l_lda) + l_i] = (unsigned char)(libxsmm_rng_f64() * 10.0);
-      }
-    }
-    /* touch B */
-    for (l_i = 0; l_i < l_ldb; l_i++) {
-      for (l_j = 0; l_j < l_n; l_j++) {
-        l_b_b[(l_j * l_ldb) + l_i] = (char)(libxsmm_rng_f64() * 10.0);
-      }
-    }
-    /* touch C */
-    for (l_i = 0; l_i < l_ldc; l_i++) {
-      for (l_j = 0; l_j < l_n; l_j++) {
-        l_c_b[(l_j * l_ldc) + l_i] = 0;
-        l_c_gold_b[(l_j * l_ldc) + l_i] = 0;
-      }
-    }
-  }
-#else
-  LIBXSMM_UNUSED(l_c_b); LIBXSMM_UNUSED(l_b_b); LIBXSMM_UNUSED(l_c_gold_b); LIBXSMM_UNUSED(l_a_b);
-#endif
 
   if ( l_file_input != 0 ) {
     fclose( l_file_handle );
