@@ -1,31 +1,11 @@
 #!/usr/bin/env python
 ###############################################################################
-# Copyright (c) 2017-2019, Intel Corporation                                  #
-# All rights reserved.                                                        #
+# Copyright (c) Intel Corporation - All rights reserved.                      #
+# This file is part of the LIBXSMM library.                                   #
 #                                                                             #
-# Redistribution and use in source and binary forms, with or without          #
-# modification, are permitted provided that the following conditions          #
-# are met:                                                                    #
-# 1. Redistributions of source code must retain the above copyright           #
-#    notice, this list of conditions and the following disclaimer.            #
-# 2. Redistributions in binary form must reproduce the above copyright        #
-#    notice, this list of conditions and the following disclaimer in the      #
-#    documentation and/or other materials provided with the distribution.     #
-# 3. Neither the name of the copyright holder nor the names of its            #
-#    contributors may be used to endorse or promote products derived          #
-#    from this software without specific prior written permission.            #
-#                                                                             #
-# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS         #
-# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT           #
-# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR       #
-# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT        #
-# HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,      #
-# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED    #
-# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR      #
-# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF      #
-# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING        #
-# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS          #
-# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                #
+# For information on the license, see the LICENSE file.                       #
+# Further information: https://github.com/hfp/libxsmm/                        #
+# SPDX-License-Identifier: BSD-3-Clause                                       #
 ###############################################################################
 # Hans Pabst (Intel Corp.)
 ###############################################################################
@@ -61,6 +41,7 @@ class XgemmTuner(MeasurementInterface):
         Define the search space by creating a
         ConfigurationManipulator
         """
+        self.dimset = libxsmm_utilities.load_mnklist(self.args.mnk, 0, -1)
         self.granularity = 1
         assert(0 < self.granularity)
         m_max = (64 + self.granularity - 1) / self.granularity
@@ -76,13 +57,15 @@ class XgemmTuner(MeasurementInterface):
         return manipulator
 
     def seed_configurations(self):
-        m_seed = [self.args.n, self.args.m][0 != self.args.m]
-        k_seed = [self.args.m, self.args.k][0 != self.args.k]
-        n_seed = [self.args.k, self.args.n][0 != self.args.n]
-        if 0 == m_seed or 0 == n_seed or k_seed:
+        m_seed = self.args.m
+        n_seed = [self.args.n, m_seed][0 == self.args.n]
+        k_seed = [self.args.k, n_seed][0 == self.args.k]
+        if 0 == m_seed or 0 == n_seed or 0 == k_seed:
             return []
         else:
-            return [{"M": m_seed, "N": n_seed, "K": k_seed}]
+            return [{"M": (m_seed + self.granularity - 1) / self.granularity,
+                     "N": (n_seed + self.granularity - 1) / self.granularity,
+                     "K": (k_seed + self.granularity - 1) / self.granularity}]
 
     def objective(self):
         return opentuner.search.objective.MaximizeAccuracyMinimizeSize()
@@ -99,10 +82,9 @@ class XgemmTuner(MeasurementInterface):
             " LIBXSMM_TGEMM_N=" + str(self.granularity * cfg["N"]) +
             " LIBXSMM_TGEMM_K=" + str(self.granularity * cfg["K"]) +
             " ./xgemm.sh")
-        dimset = libxsmm_utilities.load_mnklist(self.args.mnk, 0, -1)
         geoperf = 0  # geometric mean
         compensation = 0  # see Kahan
-        for dims in dimset:
+        for dims in self.dimset:
             run_result = self.call_program(
                 run_cmd + " " + " ".join(map(str, dims)))
             assert(run_result["returncode"] == 0)
@@ -116,17 +98,16 @@ class XgemmTuner(MeasurementInterface):
             khb = geoperf + kha
             compensation = (khb - geoperf) - kha
             geoperf = khb
-        geoperf = math.exp(geoperf / len(dimset))
+        geoperf = math.exp(geoperf / len(self.dimset))
         geotime = 1000000.0 / geoperf
         mnk = (self.granularity**3) * cfg["M"] * cfg["N"] * cfg["K"]
         return Result(time=geotime, accuracy=geoperf, size=mnk)
 
     def save_final_config(self, configuration):
         """called at the end of tuning"""
-        dimset = libxsmm_utilities.load_mnklist(self.args.mnk, 0, -1)
         matrices = (  # collects requested matrix shapes into string
             "-".join(map(str, map(lambda mnk: "x".join(
-                     map(str, mnk)), dimset))))
+                     map(str, mnk)), self.dimset))))
         filename = "xgemm-" + matrices + time.strftime(
                    "-%Y%m%d-%H%M%S") + ".json"
         print("Optimal block size written to " + filename +
@@ -137,15 +118,15 @@ class XgemmTuner(MeasurementInterface):
 if __name__ == "__main__":
     argparser = opentuner.default_argparser()
     argparser.add_argument(
-        "mnk", metavar="N", nargs="*", default=["1024,1280,1536,1792"],
+        "mnk", nargs="*", default=["1024,1280,1536,1792"],
         help="Set of MNK parameters to be tuned")
     argparser.add_argument(
-        "m", type=int, default=0, nargs='?',
-        help="Initial tile size (M)")
+        "-m", "--initial-m", type=int, default=0, nargs='?',
+        dest="m", help="Initial tile size (M)")
     argparser.add_argument(
-        "n", type=int, default=0, nargs='?',
-        help="Initial tile size (N)")
+        "-n", "--initial-n", type=int, default=0, nargs='?',
+        dest="n", help="Initial tile size (N)")
     argparser.add_argument(
-        "k", type=int, default=0, nargs='?',
-        help="Initial tile size (K)")
+        "-k", "--initial-k", type=int, default=0, nargs='?',
+        dest="k", help="Initial tile size (K)")
     XgemmTuner.main(argparser.parse_args())
