@@ -1,47 +1,16 @@
 /******************************************************************************
-** Copyright (c) 2015-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Alexander Heinecke, Hans Pabst (Intel Corp.)
 ******************************************************************************/
 #include "generator_common.h"
 #include "libxsmm_main.h"
 
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
-#endif
-#include <stdlib.h>
-#include <stdarg.h>
-#include <string.h>
-#include <stdio.h>
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(pop)
-#endif
-
 #if !defined(GENERATOR_COMMON_MAX_ERROR_LENGTH)
 # define GENERATOR_COMMON_MAX_ERROR_LENGTH 511
 #endif
@@ -730,10 +699,13 @@ void libxsmm_reset_x86_gp_reg_mapping( libxsmm_gp_reg_mapping* io_gp_reg_mapping
   io_gp_reg_mapping->gp_reg_c = LIBXSMM_X86_GP_REG_UNDEF;
   io_gp_reg_mapping->gp_reg_a_prefetch = LIBXSMM_X86_GP_REG_UNDEF;
   io_gp_reg_mapping->gp_reg_b_prefetch = LIBXSMM_X86_GP_REG_UNDEF;
-  io_gp_reg_mapping->gp_reg_c_prefetch = LIBXSMM_X86_GP_REG_UNDEF;
+/*  io_gp_reg_mapping->gp_reg_c_prefetch = LIBXSMM_X86_GP_REG_UNDEF;*/
   io_gp_reg_mapping->gp_reg_mloop = LIBXSMM_X86_GP_REG_UNDEF;
   io_gp_reg_mapping->gp_reg_nloop = LIBXSMM_X86_GP_REG_UNDEF;
   io_gp_reg_mapping->gp_reg_kloop = LIBXSMM_X86_GP_REG_UNDEF;
+  io_gp_reg_mapping->gp_reg_reduce_count = LIBXSMM_X86_GP_REG_UNDEF;
+  io_gp_reg_mapping->gp_reg_reduce_loop = LIBXSMM_X86_GP_REG_UNDEF;
+  io_gp_reg_mapping->gp_reg_scf = LIBXSMM_X86_GP_REG_UNDEF;
   io_gp_reg_mapping->gp_reg_help_0 = LIBXSMM_X86_GP_REG_UNDEF;
   io_gp_reg_mapping->gp_reg_help_1 = LIBXSMM_X86_GP_REG_UNDEF;
   io_gp_reg_mapping->gp_reg_help_2 = LIBXSMM_X86_GP_REG_UNDEF;
@@ -792,14 +764,13 @@ void libxsmm_mmfunction_signature( libxsmm_generated_code*         io_generated_
 }
 
 LIBXSMM_API_INTERN
-void libxsmm_generator_isa_check_header( libxsmm_generated_code* io_generated_code,
-                                         const char*             i_arch ) {
+void libxsmm_generator_isa_check_header( libxsmm_generated_code* io_generated_code ) {
   if ( io_generated_code->code_type == 0 ) {
     char l_new_code[512];
     int l_max_code_length = 511;
     int l_code_length = 0;
 
-    if ( (strcmp( i_arch, "wsm" ) == 0) ) {
+    if ( io_generated_code->arch <= LIBXSMM_X86_SSE4 ) {
       l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __SSE3__\n" );
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
       l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __AVX__\n" );
@@ -808,7 +779,7 @@ void libxsmm_generator_isa_check_header( libxsmm_generated_code* io_generated_co
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
       l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#endif\n" );
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-    } else if ( (strcmp( i_arch, "snb" ) == 0) ) {
+    } else if ( io_generated_code->arch == LIBXSMM_X86_AVX ) {
       l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __AVX__\n" );
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
       l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __AVX2__\n" );
@@ -817,7 +788,7 @@ void libxsmm_generator_isa_check_header( libxsmm_generated_code* io_generated_co
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
       l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#endif\n" );
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-    } else if ( (strcmp( i_arch, "hsw" ) == 0) ) {
+    } else if ( io_generated_code->arch == LIBXSMM_X86_AVX2 ) {
       l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __AVX2__\n" );
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
       l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __AVX512F__\n" );
@@ -826,14 +797,10 @@ void libxsmm_generator_isa_check_header( libxsmm_generated_code* io_generated_co
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
       l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#endif\n" );
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-    } else if ( (strcmp( i_arch, "knl" ) == 0) ||
-                (strcmp( i_arch, "knm" ) == 0) ||
-                (strcmp( i_arch, "skx" ) == 0) ||
-                (strcmp( i_arch, "clx" ) == 0) ||
-                (strcmp( i_arch, "cpx" ) == 0)    ) {
+    } else if ( ( io_generated_code->arch >= LIBXSMM_X86_AVX512 ) && ( io_generated_code->arch <= LIBXSMM_X86_ALLFEAT ) ) {
       l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#ifdef __AVX512F__\n" );
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-    } else if ( (strcmp( i_arch, "noarch" ) == 0) ) {
+    } else if ( io_generated_code->arch < LIBXSMM_X86_SSE3 ) {
       l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#pragma message (\"LIBXSMM KERNEL COMPILATION WARNING: compiling arch-independent gemm kernel in: \" __FILE__)\n" );
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
     } else {
@@ -844,21 +811,13 @@ void libxsmm_generator_isa_check_header( libxsmm_generated_code* io_generated_co
 }
 
 LIBXSMM_API_INTERN
-void libxsmm_generator_isa_check_footer( libxsmm_generated_code* io_generated_code,
-                                         const char*             i_arch ) {
+void libxsmm_generator_isa_check_footer( libxsmm_generated_code* io_generated_code ) {
   if ( io_generated_code->code_type == 0 ) {
     char l_new_code[512];
     int l_max_code_length = 511;
     int l_code_length = 0;
 
-    if ( (strcmp( i_arch, "wsm" ) == 0) ||
-         (strcmp( i_arch, "snb" ) == 0) ||
-         (strcmp( i_arch, "hsw" ) == 0) ||
-         (strcmp( i_arch, "knl" ) == 0) ||
-         (strcmp( i_arch, "knm" ) == 0) ||
-         (strcmp( i_arch, "skx" ) == 0) ||
-         (strcmp( i_arch, "clx" ) == 0) ||
-         (strcmp( i_arch, "cpx" ) == 0)    )
+    if ( ( io_generated_code->arch >= LIBXSMM_X86_SSE3 ) && ( io_generated_code->arch <= LIBXSMM_X86_ALLFEAT )  )
     {
       l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#else\n" );
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
@@ -868,7 +827,7 @@ void libxsmm_generator_isa_check_footer( libxsmm_generated_code* io_generated_co
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
       l_code_length = LIBXSMM_SNPRINTF( l_new_code, l_max_code_length, "#endif\n\n" );
       libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-    } else if ( (strcmp( i_arch, "noarch" ) == 0) ) {
+    } else if ( io_generated_code->arch < LIBXSMM_X86_SSE3 ) {
     } else {
       LIBXSMM_HANDLE_ERROR( io_generated_code, LIBXSMM_ERR_ARCH );
       return;
@@ -1059,42 +1018,6 @@ const char* libxsmm_strerror(unsigned int i_error_code) {
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
         "number of elements read differs from number of elements specified in CSR file (error #%u)!", i_error_code );
       break;
-    case LIBXSMM_ERR_UNSUP_CONV_FORMAT:
-      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "invalid activation or filter format was detected during convolution kernel generation (error #%u)!", i_error_code );
-      break;
-    case LIBXSMM_ERR_INVALID_KW_UNROLL:
-      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "invalid kw unroll was detected during convolution kernel generation (error #%u)!", i_error_code );
-      break;
-    case LIBXSMM_ERR_INVALID_KH_UNROLL:
-      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "invalid kh unroll was detected during convolution kernel generation (error #%u)!", i_error_code );
-      break;
-    case LIBXSMM_ERR_INVALID_OFW_UNROLL:
-      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "invalid ofw unroll was detected during convolution kernel generation (error #%u)!", i_error_code );
-      break;
-    case LIBXSMM_ERR_INVALID_OFH_UNROLL:
-      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "invalid ofh unroll was detected during convolution kernel generation (error #%u)!", i_error_code );
-      break;
-    case LIBXSMM_ERR_INVALID_CONV_ACC:
-      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "invalid size of accumulator was detected during convolution kernel generation (error #%u)!", i_error_code );
-      break;
-    case LIBXSMM_ERR_CONV_OFM_VEC:
-      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "forward conv. or weight upd. vect. failed, OFM blocking is not divisible by VLEN (error #%u)!", i_error_code );
-      break;
-    case LIBXSMM_ERR_CONV_IFM_VEC:
-      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "backward conv vectorization failed, IFM blocking is not divisible by VLEN (error #%u)!", i_error_code );
-      break;
-    case LIBXSMM_ERR_CONV_CONT_STRIDE:
-      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
-        "backward conv vectorization failed, stride_h/w need to be 1 (error #%u)!", i_error_code );
-      break;
     case LIBXSMM_ERR_UNSUP_DATATYPE:
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
         "unsupported datatype was requested (error #%u)!", i_error_code );
@@ -1123,60 +1046,33 @@ const char* libxsmm_strerror(unsigned int i_error_code) {
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
         "ldb needs to be greater than or equal to n (error #%u)!", i_error_code );
       break;
+    case LIBXSMM_ERR_VNNI_A:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
+        "A is not provided in supported VNNI format (error #%u)!", i_error_code );
+      break;
+    case LIBXSMM_ERR_VNNI_B:
+      LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
+        "B is not provided in supported VNNI format (error #%u)!", i_error_code );
+      break;
     default: /* we do not know what happened */
       LIBXSMM_SNPRINTF( error_message, GENERATOR_COMMON_MAX_ERROR_LENGTH,
         "an unknown error occurred (error #%u)!", i_error_code );
       break;
-  }
+    }
 
   return error_message;
 }
 
-LIBXSMM_API_INTERN
-void libxsmm_convfunction_signature_fp32( libxsmm_generated_code*         io_generated_code,
-                                          const char*                     i_routine_name ) {
-  char l_new_code[512];
-  int l_max_code_length = 511;
-  int l_code_length = 0;
-
-  if ( io_generated_code->code_type > 1 ) {
-    return;
-  } else if ( io_generated_code->code_type == 1 ) {
-    l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, ".global %s\n.type %s, @function\n%s:\n", i_routine_name, i_routine_name, i_routine_name);
-  } else {
-    l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "void %s(const float* inputptr, const float* weightptr, float* outputptr, const float* inputpfptr, const float* weightpfptr, const float* outputpfptr) {\n", i_routine_name);
-  }
-
-  libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-}
-
-LIBXSMM_API_INTERN
-void libxsmm_convfunction_signature_int16( libxsmm_generated_code*         io_generated_code,
-                                           const char*                     i_routine_name ) {
-  char l_new_code[512];
-  int l_max_code_length = 511;
-  int l_code_length = 0;
-
-  if ( io_generated_code->code_type > 1 ) {
-    return;
-  } else if ( io_generated_code->code_type == 1 ) {
-    l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, ".global %s\n.type %s, @function\n%s:\n", i_routine_name, i_routine_name, i_routine_name);
-  } else {
-    l_code_length = LIBXSMM_SNPRINTF(l_new_code, l_max_code_length, "void %s(const short* inputptr, const short* weightptr, int* outputptr, const short* inputpfptr, const short* weightpfptr, const int* outputpfptr) {\n", i_routine_name);
-  }
-
-  libxsmm_append_code_as_string( io_generated_code, l_new_code, l_code_length );
-}
-
 
 LIBXSMM_API_INTERN unsigned int libxsmm_compute_equalized_blocking(
   unsigned int i_size, unsigned int i_max_block,
   unsigned int* o_range_1, unsigned int* o_block_1,
   unsigned int* o_range_2, unsigned int* o_block_2 )
 {
-  unsigned int l_number_of_chunks = 1+((i_size-1)/i_max_block);
-  unsigned int l_modulo = i_size%l_number_of_chunks;
-  unsigned int l_n2 = i_size/l_number_of_chunks;
+  unsigned int l_size = LIBXSMM_MAX(i_size, 1);
+  unsigned int l_number_of_chunks = ((l_size - 1) / i_max_block) + 1;
+  unsigned int l_modulo = l_size % l_number_of_chunks;
+  unsigned int l_n2 = l_size / l_number_of_chunks;
   unsigned int l_n1 = l_n2 + 1;
   unsigned int l_N2 = 0;
   unsigned int l_N1 = 0;
@@ -1185,7 +1081,7 @@ LIBXSMM_API_INTERN unsigned int libxsmm_compute_equalized_blocking(
 
   /* ranges */
   if (l_n1 > i_max_block) l_n1 = i_max_block;
-  for (l_chunk = 0; l_chunk < l_number_of_chunks; l_chunk++) {
+  for (l_chunk = 0; l_chunk < l_number_of_chunks; ++l_chunk) {
     if (l_chunk < l_modulo) {
       l_N1 += l_n1;
     } else {
@@ -1193,8 +1089,8 @@ LIBXSMM_API_INTERN unsigned int libxsmm_compute_equalized_blocking(
     }
   }
 
-  /* if we have perfect blocking, let's swap n2 and n1, set  */
-  if (l_modulo == 0 ) {
+  /* if we have perfect blocking, swap n2 and n1 */
+  if ( l_modulo == 0 ) {
     l_n1 = l_n2;
     l_N1 = l_N2;
     l_n2 = 0;
@@ -1202,7 +1098,7 @@ LIBXSMM_API_INTERN unsigned int libxsmm_compute_equalized_blocking(
   }
 
   /* some checks */
-  if ( l_N1 % l_n1 != 0 ) {
+  if ( (l_N1 % l_n1) != 0 ) {
     l_ret = 1;
   }
   if ( l_n2 != 0 ) {
