@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2015-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Alexander Heinecke, Greg Henry (Intel Corp.)
 ******************************************************************************/
@@ -102,6 +82,7 @@
 #define LIBXSMM_X86_INSTR_MOVSS          10014
 #define LIBXSMM_X86_INSTR_MOVDDUP        10015
 #define LIBXSMM_X86_INSTR_SHUFPS         10016
+#define LIBXSMM_X86_INSTR_SHUFPD         10017
 
 /* Gather/Scatter instructions */
 #define LIBXSMM_X86_INSTR_VGATHERDPS     11000
@@ -119,6 +100,10 @@
 #define LIBXSMM_X86_INSTR_VSHUFF64X2     12002
 #define LIBXSMM_X86_INSTR_VEXTRACTF32X8  12003
 #define LIBXSMM_X86_INSTR_VEXTRACTF64X4  12004
+#define LIBXSMM_X86_INSTR_VSHUFPD        12005
+#define LIBXSMM_X86_INSTR_VSHUFF32X4     12006
+#define LIBXSMM_X86_INSTR_VSHUFI32X4     12007
+#define LIBXSMM_X86_INSTR_VSHUFI64X2     12008
 
 /* Vector compute instructions */
 /* AVX1,AVX2,AVX512 */
@@ -208,6 +193,14 @@
 #define LIBXSMM_X86_INSTR_VPSRLD         20079
 #define LIBXSMM_X86_INSTR_VPERMT2W       20080
 #define LIBXSMM_X86_INSTR_VPMOVSXWD      20081
+#define LIBXSMM_X86_INSTR_VPMOVDB        20082
+#define LIBXSMM_X86_INSTR_VPMOVSDB       20083
+#define LIBXSMM_X86_INSTR_VPMOVUSDB      20084
+#define LIBXSMM_X86_INSTR_VCVTPS2DQ      20085
+#define LIBXSMM_X86_INSTR_VCVTPS2UDQ     20086
+#define LIBXSMM_X86_INSTR_VPMOVZXWD      20087
+#define LIBXSMM_X86_INSTR_VPMOVSXBD      20088
+#define LIBXSMM_X86_INSTR_VPMOVZXBD      20089
 
 /* AVX512, QUAD MADD, QUAD VNNI and VNNI */
 #define LIBXSMM_X86_INSTR_V4FMADDPS      26000
@@ -305,15 +298,6 @@
 #define LIBXSMM_ERR_CSR_READ_DESC        90037
 #define LIBXSMM_ERR_CSR_READ_ELEMS       90038
 #define LIBXSMM_ERR_CSR_LEN              90039
-#define LIBXSMM_ERR_UNSUP_CONV_FORMAT    90040
-#define LIBXSMM_ERR_INVALID_KW_UNROLL    90041
-#define LIBXSMM_ERR_INVALID_KH_UNROLL    90042
-#define LIBXSMM_ERR_INVALID_OFW_UNROLL   90043
-#define LIBXSMM_ERR_INVALID_OFH_UNROLL   90044
-#define LIBXSMM_ERR_INVALID_CONV_ACC     90045
-#define LIBXSMM_ERR_CONV_OFM_VEC         90046
-#define LIBXSMM_ERR_CONV_IFM_VEC         90047
-#define LIBXSMM_ERR_CONV_CONT_STRIDE     90048
 #define LIBXSMM_ERR_UNSUP_DATATYPE       90049
 #define LIBXSMM_ERR_UNSUP_DT_FORMAT      90050
 #define LIBXSMM_ERR_INVALID_GEMM_CONFIG  90051
@@ -322,13 +306,15 @@
 #define LIBXSMM_ERR_JMPLBL_USED           90054
 #define LIBXSMM_ERR_TRANS_B               90055
 #define LIBXSMM_ERR_LDB_TRANS             90056
+#define LIBXSMM_ERR_VNNI_A                90057
+#define LIBXSMM_ERR_VNNI_B                90058
 
 #if defined(LIBXSMM_HANDLE_ERROR_QUIET)
 # define LIBXSMM_HANDLE_ERROR(GENERATED_CODE, ERROR_CODE)
 # define LIBXSMM_HANDLE_ERROR_VERBOSE(GENERATED_CODE, ERROR_CODE)
 #else
 # define LIBXSMM_HANDLE_ERROR(GENERATED_CODE, ERROR_CODE) libxsmm_handle_error( \
-    GENERATED_CODE, ERROR_CODE, LIBXSMM_FUNCNAME, 0 != libxsmm_ninit ? libxsmm_verbosity : 1)
+    GENERATED_CODE, ERROR_CODE, LIBXSMM_FUNCNAME, 1 < libxsmm_ninit ? libxsmm_verbosity : 1)
 # define LIBXSMM_HANDLE_ERROR_VERBOSE(GENERATED_CODE, ERROR_CODE) libxsmm_handle_error( \
     GENERATED_CODE, ERROR_CODE, LIBXSMM_FUNCNAME, 1)
 #endif
@@ -363,13 +349,16 @@ LIBXSMM_EXTERN_C typedef struct libxsmm_gp_reg_mapping_struct {
   unsigned int gp_reg_b;
   unsigned int gp_reg_c;
   unsigned int gp_reg_a_prefetch;
+  unsigned int gp_reg_a_offset;
   unsigned int gp_reg_b_prefetch;
-  unsigned int gp_reg_c_prefetch;
+  unsigned int gp_reg_b_offset;
+/*  unsigned int gp_reg_c_prefetch;*/
   unsigned int gp_reg_mloop;
   unsigned int gp_reg_nloop;
   unsigned int gp_reg_kloop;
   unsigned int gp_reg_reduce_count;
   unsigned int gp_reg_reduce_loop;
+  unsigned int gp_reg_scf;
   unsigned int gp_reg_help_0;
   unsigned int gp_reg_help_1;
   unsigned int gp_reg_help_2;
@@ -378,78 +367,6 @@ LIBXSMM_EXTERN_C typedef struct libxsmm_gp_reg_mapping_struct {
   unsigned int gp_reg_help_5;
 } libxsmm_gp_reg_mapping;
 
-/* convolution kernel configuration */
-LIBXSMM_EXTERN_C typedef struct libxsmm_convolution_kernel_config_struct {
-  unsigned int instruction_set;
-  unsigned int vector_reg_count;
-  unsigned int vector_length_in;
-  unsigned int datatype_size_in;
-  unsigned int vector_length_out;
-  unsigned int datatype_size_out;
-  unsigned int vector_length_wt;
-  unsigned int datatype_size_wt;
-  unsigned int vmove_instruction;
-  unsigned int vbcst_instruction;
-  unsigned int vfma_instruction;
-  unsigned int vxor_instruction;
-  unsigned int vadd_instruction;
-  unsigned int prefetch_instruction;
-  unsigned int alu_add_instruction;
-  unsigned int alu_mul_instruction;
-  unsigned int alu_sub_instruction;
-  unsigned int alu_cmp_instruction;
-  unsigned int alu_jmp_instruction;
-  unsigned int alu_mov_instruction;
-  char vector_name;
-  unsigned int l_ld_ifm_act;
-  unsigned int l_ld_ofm_act;
-  unsigned int l_ld_ifm_fil;
-  unsigned int l_ld_ofm_fil;
-} libxsmm_convolution_kernel_config;
-
-/* structure for storing the current gp reg mapping for convolution */
-LIBXSMM_EXTERN_C typedef struct libxsmm_convolution_forward_gp_reg_mapping_struct {
-  unsigned int gp_reg_input;
-  unsigned int gp_reg_weight;
-  unsigned int gp_reg_output;
-  unsigned int gp_reg_input_pf;
-  unsigned int gp_reg_input_pf_L2;
-  unsigned int gp_reg_weight_pf;
-  unsigned int gp_reg_weight_pf_L2;
-  unsigned int gp_reg_output_pf;
-  unsigned int gp_reg_kw_loop;
-  unsigned int gp_reg_kh_loop;
-  unsigned int gp_reg_ifmInner_loop;
-  unsigned int gp_reg_ifmOuter_loop;
-  unsigned int gp_reg_help_0;
-  unsigned int gp_reg_help_1;
-  unsigned int gp_reg_help_2;
-  unsigned int gp_reg_help_3;
-  unsigned int gp_reg_help_4;
-  unsigned int gp_reg_help_5;
-  unsigned int gp_reg_help_6;
-} libxsmm_convolution_forward_gp_reg_mapping;
-
-/* structure for storing the current gp reg mapping for convolution */
-LIBXSMM_EXTERN_C typedef struct libxsmm_convolution_weight_update_gp_reg_mapping_struct {
-  unsigned int gp_reg_input;
-  unsigned int gp_reg_weight;
-  unsigned int gp_reg_output;
-  unsigned int gp_reg_input_pf;
-  unsigned int gp_reg_weight_pf;
-  unsigned int gp_reg_output_pf;
-  unsigned int gp_reg_ifmInner_loop;
-  unsigned int gp_reg_oj_loop;
-  unsigned int gp_reg_oi_loop;
-  unsigned int gp_reg_help_0;
-  unsigned int gp_reg_help_1;
-  unsigned int gp_reg_help_2;
-  unsigned int gp_reg_help_3;
-  unsigned int gp_reg_help_4;
-  unsigned int gp_reg_help_5;
-  unsigned int gp_reg_help_6;
-} libxsmm_convolution_weight_update_gp_reg_mapping;
-
 /* structure for storing the current gp reg mapping for matcopy */
 LIBXSMM_EXTERN_C typedef struct libxsmm_matcopy_gp_reg_mapping_struct {
   unsigned int gp_reg_a;
@@ -561,12 +478,10 @@ void libxsmm_mmfunction_signature( libxsmm_generated_code*       io_generated_co
                                   const libxsmm_gemm_descriptor* i_xgemm_desc );
 
 LIBXSMM_API_INTERN
-void libxsmm_generator_isa_check_header( libxsmm_generated_code* io_generated_code,
-                                         const char*             i_arch );
+void libxsmm_generator_isa_check_header( libxsmm_generated_code* io_generated_code );
 
 LIBXSMM_API_INTERN
-void libxsmm_generator_isa_check_footer( libxsmm_generated_code* io_generated_code,
-                                         const char*             i_arch );
+void libxsmm_generator_isa_check_footer( libxsmm_generated_code* io_generated_code );
 
 LIBXSMM_API_INTERN
 void libxsmm_handle_error( libxsmm_generated_code* io_generated_code,
@@ -574,14 +489,6 @@ void libxsmm_handle_error( libxsmm_generated_code* io_generated_code,
                            const char*             context,
                            int emit_message );
 
-LIBXSMM_API_INTERN
-void libxsmm_convfunction_signature_fp32( libxsmm_generated_code*         io_generated_code,
-                                          const char*                     i_routine_name     );
-
-LIBXSMM_API_INTERN
-void libxsmm_convfunction_signature_int16( libxsmm_generated_code*         io_generated_code,
-                                           const char*                     i_routine_name     );
-
 LIBXSMM_API_INTERN unsigned int libxsmm_compute_equalized_blocking(
   unsigned int i_size, unsigned int i_max_block,
   unsigned int* o_range_1, unsigned int* o_block_1,
