@@ -1,49 +1,57 @@
 /******************************************************************************
-** Copyright (c) 2017-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Evangelos Georganas, Alexander Heinecke (Intel Corp.)
 ******************************************************************************/
 #include "generator_matcopy_avx_avx512.h"
 #include "generator_x86_instructions.h"
-#include "generator_convolution_common.h"
 #include "generator_common.h"
 #include "libxsmm_main.h"
 
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
-#endif
-#include <string.h>
-#include <stdlib.h>
-#include <assert.h>
-#include <stdio.h>
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(pop)
-#endif
+LIBXSMM_API_INTERN
+void libxsmm_generator_matcopy_header_m_loop( libxsmm_generated_code*                   io_generated_code,
+                                              libxsmm_loop_label_tracker*               io_loop_label_tracker,
+                                              const libxsmm_matcopy_kernel_config*      i_kernel_config,
+                                              const unsigned int                        i_gp_reg_m_loop ) {
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_kernel_config->alu_mov_instruction, i_gp_reg_m_loop, 0);
+  libxsmm_x86_instruction_register_jump_back_label( io_generated_code, io_loop_label_tracker );
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_kernel_config->alu_add_instruction, i_gp_reg_m_loop, 1);
+}
+
+LIBXSMM_API_INTERN
+void libxsmm_generator_matcopy_footer_m_loop( libxsmm_generated_code*                       io_generated_code,
+                                              libxsmm_loop_label_tracker*                   io_loop_label_tracker,
+                                              const libxsmm_matcopy_kernel_config*          i_kernel_config,
+                                              const unsigned int                            i_gp_reg_m_loop,
+                                              const unsigned int                            i_m ) {
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_kernel_config->alu_cmp_instruction, i_gp_reg_m_loop, i_m );
+  libxsmm_x86_instruction_jump_back_to_label( io_generated_code, i_kernel_config->alu_jmp_instruction, io_loop_label_tracker );
+}
+
+LIBXSMM_API_INTERN
+void libxsmm_generator_matcopy_header_n_loop( libxsmm_generated_code*                  io_generated_code,
+                                              libxsmm_loop_label_tracker*               io_loop_label_tracker,
+                                              const libxsmm_matcopy_kernel_config*      i_kernel_config,
+                                              const unsigned int                        i_gp_reg_n_loop ) {
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_kernel_config->alu_mov_instruction, i_gp_reg_n_loop, 0);
+  libxsmm_x86_instruction_register_jump_back_label( io_generated_code, io_loop_label_tracker );
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_kernel_config->alu_add_instruction, i_gp_reg_n_loop, 1);
+}
+
+LIBXSMM_API_INTERN
+void libxsmm_generator_matcopy_footer_n_loop( libxsmm_generated_code*                      io_generated_code,
+                                              libxsmm_loop_label_tracker*                   io_loop_label_tracker,
+                                              const libxsmm_matcopy_kernel_config*          i_kernel_config,
+                                              const unsigned int                            i_gp_reg_n_loop,
+                                              const unsigned int                            i_n ) {
+  libxsmm_x86_instruction_alu_imm( io_generated_code, i_kernel_config->alu_cmp_instruction, i_gp_reg_n_loop, i_n );
+  libxsmm_x86_instruction_jump_back_to_label( io_generated_code, i_kernel_config->alu_jmp_instruction, io_loop_label_tracker );
+}
 
 LIBXSMM_API_INLINE
 void libxsmm_generator_matcopy_avx_avx512_kernel_initialize_mask( libxsmm_generated_code*               io_generated_code,
@@ -264,13 +272,13 @@ void libxsmm_generator_matcopy_avx_avx512_kernel( libxsmm_generated_code*
 
   if (i_matcopy_desc->n > 1) {
     /* open n loop */
-    libxsmm_generator_convolution_header_n_loop(  io_generated_code, &l_loop_label_tracker,
+    libxsmm_generator_matcopy_header_n_loop(  io_generated_code, &l_loop_label_tracker,
         &l_kernel_config, l_gp_reg_mapping.gp_reg_n_loop );
   }
 
   if (m_trips > 1) {
     /* open m loop */
-    libxsmm_generator_convolution_header_m_loop(  io_generated_code, &l_loop_label_tracker,
+    libxsmm_generator_matcopy_header_m_loop(  io_generated_code, &l_loop_label_tracker,
         &l_kernel_config, l_gp_reg_mapping.gp_reg_m_loop );
   }
 
@@ -336,7 +344,7 @@ void libxsmm_generator_matcopy_avx_avx512_kernel( libxsmm_generated_code*
 
   if (m_trips > 1) {
     /* close m loop */
-    libxsmm_generator_convolution_footer_m_loop(  io_generated_code, &l_loop_label_tracker,
+    libxsmm_generator_matcopy_footer_m_loop(  io_generated_code, &l_loop_label_tracker,
         &l_kernel_config, l_gp_reg_mapping.gp_reg_m_loop, m_trips );
   }
 
@@ -460,7 +468,7 @@ void libxsmm_generator_matcopy_avx_avx512_kernel( libxsmm_generated_code*
       }
     }
     /* close n loop */
-    libxsmm_generator_convolution_footer_n_loop(  io_generated_code, &l_loop_label_tracker,
+    libxsmm_generator_matcopy_footer_n_loop(  io_generated_code, &l_loop_label_tracker,
         &l_kernel_config, l_gp_reg_mapping.gp_reg_n_loop, i_matcopy_desc->n );
   }
 
