@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2017-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Alexander Heinecke, Greg Henry, Hans Pabst, Timothy Costa (Intel Corp.)
 ******************************************************************************/
@@ -34,19 +14,6 @@
 #include "generator_common.h"
 #include "libxsmm_main.h"
 
-#include <libxsmm_intrinsics_x86.h>
-
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
-#endif
-#include <string.h>
-#include <stdlib.h>
-#include <assert.h>
-#include <stdio.h>
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(pop)
-#endif
-
 #if 0
 # define GENERATOR_PACKED_TRSM_DEBUG
 #endif
@@ -54,8 +21,8 @@
 
 LIBXSMM_API_INTERN
 void libxsmm_generator_packed_trsm_avx_avx512_kernel( libxsmm_generated_code*        io_code,
-                                                       const libxsmm_trsm_descriptor* i_packed_trsm_desc,
-                                                       const char*                    i_arch )
+                                                      const libxsmm_trsm_descriptor* i_packed_trsm_desc,
+                                                      const char*                    i_arch )
 {
   unsigned char *const buf = (unsigned char *) io_code->generated_code;
   libxsmm_loop_label_tracker l_loop_label_tracker /*= { 0 }*/;
@@ -117,6 +84,10 @@ void libxsmm_generator_packed_trsm_avx_avx512_kernel( libxsmm_generated_code*
   /* Todo-> I first want this code to work, and verify it works, then I can
    *        convert one instruction at a time to those in
    *        generator_x86_instructions.h. Or add to the existing instructions */
+  if (NULL == buf) {
+    LIBXSMM_HANDLE_ERROR(io_code, LIBXSMM_ERR_BUFFER_TOO_SMALL);
+    return;
+  }
 
   if ( io_code->code_type > 1 )
   {
@@ -192,7 +163,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
         i = io_code->code_size;
         buf[i++] = 0xc3; /* retq */
         io_code->code_size = i;
-        return ;
+        return;
      }
      if ( LIBXSMM_NEQ(1, alpha) )
      {
@@ -210,7 +181,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
               if ( nounit )
               {
                   compact_set_one_ ( io_code, 15, numb, datasz, regset );
-                  for ( i = 1 ; i <= m1 ; i++ )
+                  for ( i = 1; i <= m1; i++ )
                   {
                      compact_load_matrix1_ ( io_code, lda, i, i, 3, numb, datasz, regset );
                      compact_divide_two_nums_ ( io_code, 15, 3, 3 , numb, regset );
@@ -242,7 +213,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
 #endif
                        compact_store_matrix2_ ( io_code, ldb, k, j, 0, numb, datasz, regset );
                     }
-                    for ( i = 1 ; i <= k-1 ; i++ )
+                    for ( i = 1; i <= k-1; i++ )
                     {
                        compact_load_matrix2_ ( io_code, ldb, i, j, 1, numb, datasz, regset );
                        compact_load_matrix1_ ( io_code, lda, i, k, 3, numb, datasz, regset );
@@ -271,7 +242,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
               if ( nounit )
               {
                   compact_set_one_ ( io_code, 15, numb, datasz, regset );
-                  for ( i = 1 ; i <= m1 ; i++ )
+                  for ( i = 1; i <= m1; i++ )
                   {
                      compact_load_matrix1_ ( io_code, lda, i, i, 3, numb, datasz, regset );
                      compact_divide_two_nums_ ( io_code, 15, 3, 3 , numb, regset );
@@ -281,7 +252,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
 
               for ( j = 1; j <= n1; j+=3 )
               {
-                 for ( k = 1 ; k <= m1; k+=2 )
+                 for ( k = 1; k <= m1; k+=2 )
                  {
                     scalealpha = 0;
                     if ( LIBXSMM_NEQ(1, alpha) && (k==1) ) scalealpha = 1;
@@ -342,7 +313,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
                           }
                        }
                     }
-                    for ( i = k+2 ; i <= m1; i++ )
+                    for ( i = k+2; i <= m1; i++ )
                     {
                        compact_load_matrix2_ ( io_code, ldb, i, j, 1, numb, datasz, regset );
                        if ( scalealpha == 1 ) {
@@ -400,7 +371,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
               if ( nounit )
               {
                   compact_set_one_ ( io_code, 15, numb, datasz, regset );
-                  for ( i = 1 ; i <= m1 ; i++ )
+                  for ( i = 1; i <= m1; i++ )
                   {
                      compact_load_matrix1_ ( io_code, lda, i, i, 3, numb, datasz, regset );
                      compact_divide_two_nums_ ( io_code, 15, 3, 3 , numb, regset );
@@ -415,7 +386,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
               for ( j = 1; j <= n1; j++  )
 #endif
               {
-                 for ( i = 1 ; i <= m1; i+=2 )
+                 for ( i = 1; i <= m1; i+=2 )
                  {
                     compact_load_matrix2_ ( io_code, ldb, i, j, 0, numb, datasz, regset );
 #ifdef LUT_N2
@@ -515,7 +486,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
               if ( nounit )
               {
                   compact_set_one_ ( io_code, 15, numb, datasz, regset );
-                  for ( i = 1 ; i <= m1 ; i++ )
+                  for ( i = 1; i <= m1; i++ )
                   {
                      compact_load_matrix1_ ( io_code, lda, i, i, 3, numb, datasz, regset );
                      compact_divide_two_nums_ ( io_code, 15, 3, 3 , numb, regset );
@@ -530,9 +501,9 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
 #endif
               {
 #ifdef LLT_M2
-                 for ( i = m1 ; i >= 1; i-=2 )
+                 for ( i = m1; i >= 1; i-=2 )
 #else
-                 for ( i = m1 ; i >= 1; i-=1 )
+                 for ( i = m1; i >= 1; i-=1 )
 #endif
                  {
                     compact_load_matrix2_ ( io_code, ldb, i, j, 0, numb, datasz, regset );
@@ -662,7 +633,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
               if ( nounit )
               {
                   compact_set_one_ ( io_code, 15, numb, datasz, regset );
-                  for ( i = 1 ; i <= n1 ; i++ )
+                  for ( i = 1; i <= n1; i++ )
                   {
                      compact_load_matrix1_ ( io_code, lda, i, i, 3, numb, datasz, regset );
                      compact_divide_two_nums_ ( io_code, 15, 3, 3 , numb, regset );
@@ -706,7 +677,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
                     if ( j+2 <= n1 ) compact_load_matrix1_ ( io_code, lda, k, j+2, 10, numb, datasz, regset );
                     if ( j+3 <= n1 ) compact_load_matrix1_ ( io_code, lda, k, j+3, 12, numb, datasz, regset );
 #endif
-                    for ( i = 1 ; i <= m1 ; i++ )
+                    for ( i = 1; i <= m1; i++ )
                     {
                        compact_load_matrix2_ ( io_code, ldb, i, j, 1, numb, datasz, regset );
                        if (j+1<=n1) compact_load_matrix2_ ( io_code, ldb, i, j+1, 7, numb, datasz, regset );
@@ -741,7 +712,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
                        compact_load_matrix1_ ( io_code, lda, k, j+1, 6, numb, datasz, regset );
                        if ( j==1 && nounit ) compact_load_matrix3_ ( io_code, n1, j, 1, 9, numb, datasz, regset );
                        if ( nounit ) compact_load_matrix3_ ( io_code, n1, j+1, 1, 1, numb, datasz, regset );
-                       for ( i = 1; i <= m1 ; i++ ) {
+                       for ( i = 1; i <= m1; i++ ) {
                           compact_load_matrix2_ ( io_code, ldb, i, j+1, 7, numb, datasz, regset );
                           compact_load_matrix2_ ( io_code, ldb, i, k, 8, numb, datasz, regset );
                           if (j==1 && nounit) {
@@ -760,7 +731,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
                     for ( k = j; k <= j+1; k++ ) {
                        compact_load_matrix1_ ( io_code, lda, k, j+2, 6, numb, datasz, regset );
                        if ( (k==j+1) && nounit ) compact_load_matrix3_ ( io_code, n1, j+2, 1, 1, numb, datasz, regset );
-                       for ( i = 1; i <= m1 ; i++ ) {
+                       for ( i = 1; i <= m1; i++ ) {
                           compact_load_matrix2_ ( io_code, ldb, i, j+2, 7, numb, datasz, regset );
                           compact_load_matrix2_ ( io_code, ldb, i, k, 8, numb, datasz, regset );
                           compact_fms_cminusab_ ( io_code, 7, 6, 8, numb, regset );
@@ -773,7 +744,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
                     for ( k = j; k <= j+2; k++ ) {
                        compact_load_matrix1_ ( io_code, lda, k, j+3, 6, numb, datasz, regset );
                        if ( (k==j+2) && nounit ) compact_load_matrix3_ ( io_code, n1, j+3, 1, 1, numb, datasz, regset );
-                       for ( i = 1; i <= m1 ; i++ ) {
+                       for ( i = 1; i <= m1; i++ ) {
                           compact_load_matrix2_ ( io_code, ldb, i, j+3, 7, numb, datasz, regset );
                           compact_load_matrix2_ ( io_code, ldb, i, k, 8, numb, datasz, regset );
                           compact_fms_cminusab_ ( io_code, 7, 6, 8, numb, regset );
@@ -800,7 +771,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
                  for ( k = j+1; k <= n1; k++ )
                  {
                     compact_load_matrix1_ ( io_code, lda, k, j, 3, numb, datasz, regset );
-                    for ( i = 1 ; i <= m1 ; i++ )
+                    for ( i = 1; i <= m1; i++ )
                     {
                        compact_load_matrix2_ ( io_code, ldb, i, j, 1, numb, datasz, regset );
                        compact_load_matrix2_ ( io_code, ldb, i, k, 4, numb, datasz, regset );
@@ -812,7 +783,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
                  {
                     compact_load_matrix1_( io_code, lda, j, j, 1, numb, datasz, regset );
                     compact_divide_two_nums_ ( io_code, 5, 1, 1, numb, regset );
-                    for ( i = 1; i <= m1 ; i++ )
+                    for ( i = 1; i <= m1; i++ )
                     {
                        compact_load_matrix2_ ( io_code, ldb, i, j, 3, numb, datasz, regset );
                        compact_mult_two_nums_ ( io_code, 1, 3, 3, numb, regset );
@@ -825,7 +796,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
            if ( (uplo=='U') || (uplo=='u') )
            {
               /* Do RUT* cases: B<- alpha*B *inv(A^T) */
-              for ( k = n1 ; k >= 1; k-- )
+              for ( k = n1; k >= 1; k-- )
               {
                  if ( nounit )
                  {
@@ -838,10 +809,10 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
                        compact_store_matrix2_ ( io_code, ldb, i, k, 3, numb, datasz, regset );
                     }
                  }
-                 for ( j = 1; j <= k-1 ; j++ )
+                 for ( j = 1; j <= k-1; j++ )
                  {
                     compact_load_matrix1_ ( io_code, lda, j, k, 1, numb, datasz, regset );
-                    for ( i = 1 ; i <= m1; i++ )
+                    for ( i = 1; i <= m1; i++ )
                     {
                        compact_load_matrix2_ ( io_code, ldb, i, j, 0, numb, datasz, regset );
                        compact_load_matrix2_ ( io_code, ldb, i, k, 3, numb, datasz, regset );
@@ -861,7 +832,7 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
               }
            } else {
               /* Do RLT* cases: B <- alpha * B *inv(A^T) */
-              for ( k = 1 ; k <= n1; k++ )
+              for ( k = 1; k <= n1; k++ )
               {
                  if ( nounit )
                  {
@@ -874,10 +845,10 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
                        compact_store_matrix2_ ( io_code, ldb, i, k, 3, numb, datasz, regset );
                     }
                  }
-                 for ( j = k+1; j <= n1 ; j++ )
+                 for ( j = k+1; j <= n1; j++ )
                  {
                     compact_load_matrix1_ ( io_code, lda, j, k, 1, numb, datasz, regset );
-                    for ( i = 1 ; i <= m1; i++ )
+                    for ( i = 1; i <= m1; i++ )
                     {
                        compact_load_matrix2_( io_code, ldb, i, j, 0, numb, datasz, regset );
                        compact_load_matrix2_( io_code, ldb, i, k, 3, numb, datasz, regset );
@@ -916,13 +887,12 @@ printf("Inside libxsmm_generator_packed_trsm_avx_avx512_kernel: %c%c%c%c m=%d n=
 #define DEBUG_GIVE_BYTE_CODE_OUTPUT
 #endif
 #ifdef DEBUG_GIVE_BYTE_CODE_OUTPUT
-   buf = (unsigned char *) io_code->generated_code;
-   printf("#Final Routine: \n");
-   for ( i = 0 ; i < io_code->code_size ; i+=8 )
-   {
-      printf("#\tBytes %d-%d\n",i,i+7);
-      printf(".byte 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x\n",buf[i],buf[i+1],buf[i+2],buf[i+3],buf[i+4],buf[i+5],buf[i+6],buf[i+7]);
-   }
+  buf = (unsigned char *) io_code->generated_code;
+  printf("#Final Routine: \n");
+  for ( i = 0; i < io_code->code_size; i+=8 ) {
+    printf("#\tBytes %d-%d\n",i,i+7);
+    printf(".byte 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x, 0x%02x\n",buf[i],buf[i+1],buf[i+2],buf[i+3],buf[i+4],buf[i+5],buf[i+6],buf[i+7]);
+  }
 #endif
 }
 
