@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2015-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Alexander Heinecke, Greg Henry (Intel Corp.)
 ******************************************************************************/
@@ -39,13 +19,11 @@
  *
  * @param io_generated_code pointer to the pointer of the generated code structure
  * @param i_gp_reg_mapping gp register mapping for initialization
- * @param i_arch architecture code was generated for (needed to build clobber)
  * @param i_prefetch prefetch mode which may result in additional gp reg inits
  */
 LIBXSMM_API_INTERN
 void libxsmm_x86_instruction_open_stream( libxsmm_generated_code*       io_generated_code,
                                           const libxsmm_gp_reg_mapping* i_gp_reg_mapping,
-                                          const char*                   i_arch,
                                           unsigned int                  i_prefetch );
 
 /**
@@ -53,13 +31,11 @@ void libxsmm_x86_instruction_open_stream( libxsmm_generated_code*       io_gener
  *
  * @param io_generated_code pointer to the pointer of the generated code structure
  * @param i_gp_reg_mapping gp register mapping for clobbering
- * @param i_arch architecture code was generated for (needed to build clobber)
  * @param i_prefetch prefetch mode which may result in additional gp reg clobbers
  */
 LIBXSMM_API_INTERN
 void libxsmm_x86_instruction_close_stream( libxsmm_generated_code*       io_generated_code,
                                            const libxsmm_gp_reg_mapping* i_gp_reg_mapping,
-                                           const char*                   i_arch,
                                            unsigned int                  i_prefetch );
 
 /**
@@ -352,7 +328,7 @@ LIBXSMM_API_INTERN
 void libxsmm_x86_instruction_alu_imm( libxsmm_generated_code* io_generated_code,
                                       const unsigned int      i_alu_instr,
                                       const unsigned int      i_gp_reg_number,
-                                      const int               i_immediate );
+                                      const long long         i_immediate );
 
 /**
  * Generates regular all instructions with immediates, 64bit
@@ -496,34 +472,18 @@ void libxsmm_x86_instruction_full_vec_load_of_constants ( libxsmm_generated_code
                                                           const char *i_id,
                                                           const char i_vector_name,
                                                           const unsigned int i_vec_reg_number );
-/**
- * @TODO: clean-up
- * Opens the inline assembly section / jit stream for convolutions, this is hacked and should be cleaned up
- *
- * @param io_generated_code pointer to the pointer of the generated code structure
- * @param i_arch architecture code was generated for (needed to build clobber)
- */
-LIBXSMM_API_INTERN
-void libxsmm_x86_instruction_open_stream_convolution( libxsmm_generated_code*                   io_generated_code,
-                                                      const unsigned int                        i_gp_reg_input,
-                                                      const unsigned int                        i_gp_reg_weight,
-                                                      const unsigned int                        i_gp_reg_output,
-                                                      const unsigned int                        i_gp_reg_input_pf,
-                                                      const unsigned int                        i_gp_reg_weight_pf,
-                                                      const unsigned int                        i_gp_reg_output_pf,
-                                                      const char*                               i_arch );
 
 /**
- * @TODO: clean-up
- * Closes the inline assembly section / jit stream for convolutions, this is hacked and should be cleaned up
+ * Generates a sequence to load function arguments from the stack (arguments )
  *
  * @param io_generated_code pointer to the pointer of the generated code structure
- * @param i_arch architecture code was generated for (needed to build clobber)
+ * @param i_arg_number the number of an argument which was passed on the stack
+ * @param i_gp_reg_number the destination register number (rax=0,rcx=1,rdx=2,rbx=3,rsp=4,rbp=5,rsi=6,rdi=7,r8=8,r9=9,r10=10,r11=11,r12=12,r13=13,r14=14,r15=15)
  */
 LIBXSMM_API_INTERN
-void libxsmm_x86_instruction_close_stream_convolution( libxsmm_generated_code*       io_generated_code,
-                                                const char*                   i_arch );
-
+void libxsmm_x86_instruction_load_arg_to_reg( libxsmm_generated_code* io_generated_code,
+                                              const unsigned int      i_arg_number,
+                                              const unsigned int      i_gp_reg_number );
 
 /**
  * @TODO: clean-up
