@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2016-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Hans Pabst (Intel Corp.)
 ******************************************************************************/
@@ -32,18 +12,14 @@
 #include "libxsmm_gemm.h"
 #include "libxsmm_ext.h"
 
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
-#endif
-#include <string.h>
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(pop)
-#endif
-
 #if defined(LIBXSMM_WRAP) && defined(LIBXSMM_BUILD_EXT)
 # include "libxsmm_trace.h"
 #endif
 
+#if !defined(LIBXSMM_EXT_GEMM_PARGROUPS_INFO) && 0
+# define LIBXSMM_EXT_GEMM_PARGROUPS_INFO
+#endif
+
 #if defined(LIBXSMM_WRAP) && defined(LIBXSMM_BUILD_EXT)
 # if !defined(LIBXSMM_EXT_GEMM_MMBATCH_PREFETCH)
 #   define LIBXSMM_EXT_GEMM_MMBATCH_PREFETCH libxsmm_get_gemm_prefetch(LIBXSMM_PREFETCH_AUTO)
@@ -51,9 +27,9 @@
 # if !defined(LIBXSMM_EXT_GEMM_MMBATCH_MAXDEPTH)
 #   define LIBXSMM_EXT_GEMM_MMBATCH_MAXDEPTH 8/*POT*/
 # endif
-LIBXSMM_APIVAR_ARRAY(libxsmm_gemm_descriptor internal_ext_gemm_batchdesc, LIBXSMM_EXT_GEMM_MMBATCH_MAXDEPTH);
-LIBXSMM_APIVAR(unsigned int internal_ext_gemm_batchdepth);
-LIBXSMM_APIVAR(unsigned int internal_ext_gemm_batchsize);
+LIBXSMM_APIVAR_DEFINE(libxsmm_gemm_descriptor internal_ext_gemm_batchdesc[LIBXSMM_EXT_GEMM_MMBATCH_MAXDEPTH]);
+LIBXSMM_APIVAR_DEFINE(unsigned int internal_ext_gemm_batchdepth);
+LIBXSMM_APIVAR_DEFINE(unsigned int internal_ext_gemm_batchsize);
 #endif
 
 
@@ -88,7 +64,7 @@ LIBXSMM_API_INLINE int internal_mmbatch_flush(const libxsmm_gemm_descriptor* bat
           if (0 == omp_get_active_level()) {
             const int max_nthreads = omp_get_max_threads();
             const int nthreads = LIBXSMM_MIN(max_nthreads, nchunks);
-            if (0 >= libxsmm_gemm_taskscale)
+            if (0 == libxsmm_gemm_tasks)
 # else
           if (0 == omp_in_parallel()) {
             const int max_nthreads = omp_get_max_threads();
@@ -104,17 +80,16 @@ LIBXSMM_API_INLINE int internal_mmbatch_flush(const libxsmm_gemm_descriptor* bat
             }
 # if defined(LIBXSMM_EXT_TASKS)
             else { /* internal parallelization with tasks */
-              const int ntasks = nchunks * libxsmm_gemm_taskscale;
 #             pragma omp parallel num_threads(nthreads)
               { /* first thread discovering work will launch all tasks */
 #               pragma omp single nowait /* anyone is good */
-                { int tid; for (tid = 0; tid < ntasks; ++tid) {
+                { int tid; for (tid = 0; tid < nchunks/*ntasks*/; ++tid) {
 #                 pragma omp task untied
                   /*check*/libxsmm_mmbatch_kernel(
                     kernel, 0/*index_base*/, 0/*index_stride*/, &itemsize, &itemsize, &itemsize,
                     &batcharray->value.a, &batcharray->value.b, &batcharray->value.c,
                     0 == (LIBXSMM_MMBATCH_FLAG_SYNCHRONIZED & batchdesc->flags) ? batchsize : -batchsize,
-                    tid, ntasks, itypesize, otypesize, batchdesc->flags);
+                    tid, nchunks/*ntasks*/, itypesize, otypesize, batchdesc->flags);
                   }
                 }
               } /* implicit synchronization (barrier) */
@@ -122,8 +97,7 @@ LIBXSMM_API_INLINE int internal_mmbatch_flush(const libxsmm_gemm_descriptor* bat
 # endif
           }
           else { /* assume external parallelization */
-            const int ntasks = nchunks * (0 < libxsmm_gemm_taskscale ? libxsmm_gemm_taskscale : (LIBXSMM_GEMM_TASKSCALE));
-            int tid; for (tid = 0; tid < ntasks; ++tid) {
+            int tid; for (tid = 0; tid < nchunks/*ntasks*/; ++tid) {
 # if defined(LIBXSMM_EXT_TASKS)
 #             pragma omp task untied
 #endif
@@ -131,7 +105,7 @@ LIBXSMM_API_INLINE int internal_mmbatch_flush(const libxsmm_gemm_descriptor* bat
                 kernel, 0/*index_base*/, 0/*index_stride*/, &itemsize, &itemsize, &itemsize,
                 &batcharray->value.a, &batcharray->value.b, &batcharray->value.c,
                 0 == (LIBXSMM_MMBATCH_FLAG_SYNCHRONIZED & batchdesc->flags) ? batchsize : -batchsize,
-                tid, ntasks, itypesize, otypesize, batchdesc->flags);
+                tid, nchunks/*ntasks*/, itypesize, otypesize, batchdesc->flags);
             }
 # if defined(LIBXSMM_EXT_TASKS)
             if (0 == libxsmm_nosync) { /* allow to omit synchronization */
@@ -171,7 +145,7 @@ LIBXSMM_API_INLINE int internal_mmbatch_flush(const libxsmm_gemm_descriptor* bat
         const libxsmm_blasint m = descriptor.m, n = descriptor.n, k = descriptor.k;
         const char *const symbol = batcharray[i].stat.symbol;
         const unsigned int ci = batcharray[i].stat.count;
-        memset(&batcharray[i], 0, (size_t)itemsize); /* clear */
+        LIBXSMM_MEMZERO127(batcharray + i); /* clear */
         if (threshold < ci && count < limit /* limit printed statistic */
           && 0 < m && 0 < n && 0 < k)
         {
@@ -209,45 +183,69 @@ LIBXSMM_API_INLINE int internal_mmbatch_flush(const libxsmm_gemm_descriptor* bat
 #if defined(LIBXSMM_BUILD) && defined(LIBXSMM_BUILD_EXT)
 
 #if defined(LIBXSMM_BLAS_WRAP_DYNAMIC)
-LIBXSMM_API_EXPORT libxsmm_dgemm_batch_function libxsmm_original_dgemm_batch(void)
+LIBXSMM_API libxsmm_dgemm_batch_function libxsmm_original_dgemm_batch(void)
 {
-  LIBXSMM_BLAS_WRAPPER(double, gemm_batch, libxsmm_original_dgemm_batch_function, libxsmm_original_dgemm_batch/*self*/);
-  LIBXSMM_ASSERT(NULL != libxsmm_original_dgemm_batch_function);
+# if (0 != LIBXSMM_BLAS)
+  LIBXSMM_BLAS_WRAPPER(1, double, gemm_batch, libxsmm_original_dgemm_batch_function, libxsmm_original_dgemm_batch/*self*/);
+  /*LIBXSMM_ASSERT(NULL != libxsmm_original_dgemm_batch_function);*/
+# else
+  LIBXSMM_BLAS_WRAPPER(0, double, gemm_batch, libxsmm_original_dgemm_batch_function, libxsmm_original_dgemm_batch/*self*/);
+# endif
   return libxsmm_original_dgemm_batch_function;
 }
 
-LIBXSMM_API_EXPORT libxsmm_sgemm_batch_function libxsmm_original_sgemm_batch(void)
+LIBXSMM_API libxsmm_sgemm_batch_function libxsmm_original_sgemm_batch(void)
 {
-  LIBXSMM_BLAS_WRAPPER(float, gemm_batch, libxsmm_original_sgemm_batch_function, libxsmm_original_sgemm_batch/*self*/);
-  LIBXSMM_ASSERT(NULL != libxsmm_original_sgemm_batch_function);
+# if (0 != LIBXSMM_BLAS)
+  LIBXSMM_BLAS_WRAPPER(1, float, gemm_batch, libxsmm_original_sgemm_batch_function, libxsmm_original_sgemm_batch/*self*/);
+  /*LIBXSMM_ASSERT(NULL != libxsmm_original_sgemm_batch_function);*/
+# else
+  LIBXSMM_BLAS_WRAPPER(0, float, gemm_batch, libxsmm_original_sgemm_batch_function, libxsmm_original_sgemm_batch/*self*/);
+# endif
   return libxsmm_original_sgemm_batch_function;
 }
 
-LIBXSMM_API_EXPORT libxsmm_dgemm_function libxsmm_original_dgemm(void)
+LIBXSMM_API libxsmm_dgemm_function libxsmm_original_dgemm(void)
 {
-  LIBXSMM_BLAS_WRAPPER(double, gemm, libxsmm_original_dgemm_function, libxsmm_original_dgemm/*self*/);
+# if (0 != LIBXSMM_BLAS)
+  LIBXSMM_BLAS_WRAPPER(1, double, gemm, libxsmm_original_dgemm_function, libxsmm_original_dgemm/*self*/);
   LIBXSMM_ASSERT(NULL != libxsmm_original_dgemm_function);
+# else
+  LIBXSMM_BLAS_WRAPPER(0, double, gemm, libxsmm_original_dgemm_function, libxsmm_original_dgemm/*self*/);
+# endif
   return libxsmm_original_dgemm_function;
 }
 
-LIBXSMM_API_EXPORT libxsmm_sgemm_function libxsmm_original_sgemm(void)
+LIBXSMM_API libxsmm_sgemm_function libxsmm_original_sgemm(void)
 {
-  LIBXSMM_BLAS_WRAPPER(float, gemm, libxsmm_original_sgemm_function, libxsmm_original_sgemm/*self*/);
+# if (0 != LIBXSMM_BLAS)
+  LIBXSMM_BLAS_WRAPPER(1, float, gemm, libxsmm_original_sgemm_function, libxsmm_original_sgemm/*self*/);
   LIBXSMM_ASSERT(NULL != libxsmm_original_sgemm_function);
+# else
+  LIBXSMM_BLAS_WRAPPER(0, float, gemm, libxsmm_original_sgemm_function, libxsmm_original_sgemm/*self*/);
+# endif
   return libxsmm_original_sgemm_function;
 }
 
-LIBXSMM_API_EXPORT libxsmm_dgemv_function libxsmm_original_dgemv(void)
+LIBXSMM_API libxsmm_dgemv_function libxsmm_original_dgemv(void)
 {
-  LIBXSMM_BLAS_WRAPPER(double, gemv, libxsmm_original_dgemv_function, libxsmm_original_dgemv/*self*/);
+# if (0 != LIBXSMM_BLAS)
+  LIBXSMM_BLAS_WRAPPER(1, double, gemv, libxsmm_original_dgemv_function, libxsmm_original_dgemv/*self*/);
   LIBXSMM_ASSERT(NULL != libxsmm_original_dgemv_function);
+# else
+  LIBXSMM_BLAS_WRAPPER(0, double, gemv, libxsmm_original_dgemv_function, libxsmm_original_dgemv/*self*/);
+# endif
   return libxsmm_original_dgemv_function;
 }
 
-LIBXSMM_API_EXPORT libxsmm_sgemv_function libxsmm_original_sgemv(void)
+LIBXSMM_API libxsmm_sgemv_function libxsmm_original_sgemv(void)
 {
-  LIBXSMM_BLAS_WRAPPER(float, gemv, libxsmm_original_sgemv_function, libxsmm_original_sgemv/*self*/);
+# if (0 != LIBXSMM_BLAS)
+  LIBXSMM_BLAS_WRAPPER(1, float, gemv, libxsmm_original_sgemv_function, libxsmm_original_sgemv/*self*/);
   LIBXSMM_ASSERT(NULL != libxsmm_original_sgemv_function);
+# else
+  LIBXSMM_BLAS_WRAPPER(0, float, gemv, libxsmm_original_sgemv_function, libxsmm_original_sgemv/*self*/);
+# endif
   return libxsmm_original_sgemv_function;
 }
 #endif /*defined(LIBXSMM_BLAS_WRAP_DYNAMIC)*/
@@ -337,7 +335,7 @@ LIBXSMM_APIEXT LIBXSMM_ATTRIBUTE_USED void LIBXSMM_FSYMBOL(__wrap_dgemm)(
       || ((unsigned int)*m) != libxsmm_mmbatch_desc.m
       || ((unsigned int)*n) != libxsmm_mmbatch_desc.n
       || ((unsigned int)*k) != libxsmm_mmbatch_desc.k
-      || (flags = LIBXSMM_GEMM_FLAGS(*transa, *transb)) != (LIBXSMM_GEMM_FLAG_TRANS_AB & libxsmm_mmbatch_desc.flags)
+      || (flags = LIBXSMM_GEMM_FLAGS(*transa, *transb)) != (int)(LIBXSMM_GEMM_FLAG_TRANS_AB & libxsmm_mmbatch_desc.flags)
       || LIBXSMM_NEQ(/*0 != (LIBXSMM_GEMM_FLAG_ALPHA_0 & libxsmm_mmbatch_desc.flags) ? 0 : */1, *alpha)
       || LIBXSMM_NEQ(0 != (LIBXSMM_GEMM_FLAG_BETA_0 & libxsmm_mmbatch_desc.flags) ? 0 : 1, *beta)))
 #endif
@@ -474,7 +472,7 @@ LIBXSMM_APIEXT LIBXSMM_ATTRIBUTE_USED void LIBXSMM_FSYMBOL(__wrap_sgemm)(
       || ((unsigned int)*m) != libxsmm_mmbatch_desc.m
       || ((unsigned int)*n) != libxsmm_mmbatch_desc.n
       || ((unsigned int)*k) != libxsmm_mmbatch_desc.k
-      || (flags = LIBXSMM_GEMM_FLAGS(*transa, *transb)) != (LIBXSMM_GEMM_FLAG_TRANS_AB & libxsmm_mmbatch_desc.flags)
+      || (flags = LIBXSMM_GEMM_FLAGS(*transa, *transb)) != (int)(LIBXSMM_GEMM_FLAG_TRANS_AB & libxsmm_mmbatch_desc.flags)
       || LIBXSMM_NEQ(/*0 != (LIBXSMM_GEMM_FLAG_ALPHA_0 & libxsmm_mmbatch_desc.flags) ? 0 : */1, *alpha)
       || LIBXSMM_NEQ(0 != (LIBXSMM_GEMM_FLAG_BETA_0 & libxsmm_mmbatch_desc.flags) ? 0 : 1, *beta)))
 #endif
@@ -594,9 +592,9 @@ LIBXSMM_APIEXT LIBXSMM_ATTRIBUTE_USED void LIBXSMM_FSYMBOL(__wrap_dgemv)(const c
   if ((2 < libxsmm_gemm_wrap || 2 > libxsmm_gemm_wrap) && 1 == *incx && 1 == *incy && LIBXSMM_SMM(*m, 1, *n, 2/*RFO*/, sizeof(double))) {
     if (0 != (libxsmm_gemm_wrap & 1)) { /* sequential */
       const int flags = LIBXSMM_GEMM_FLAGS(*trans, 'N');
-      const libxsmm_dmmfunction xgemv = libxsmm_dmmdispatch(*m, 1, *n, lda, n/*ldb*/, n/*ldc*/, alpha, beta, &flags, NULL);
+      const libxsmm_dmmfunction xgemv = libxsmm_dmmdispatch(*m, 1, *n, lda, n/*ldb*/, m/*ldc*/, alpha, beta, &flags, NULL);
       if (NULL != xgemv) {
-        LIBXSMM_MMCALL_LDX(xgemv, a, x, y, *m, 1, *n, *lda, *n/*ldb*/, *n/*ldc*/);
+        LIBXSMM_MMCALL_LDX(xgemv, a, x, y, *m, 1, *n, *lda, *n/*ldb*/, *m/*ldc*/);
       }
       else {
         LIBXSMM_GEMV_SYMBOL(double)(trans, m, n, alpha, a, lda, x, incx, beta, y, incy);
@@ -621,9 +619,9 @@ LIBXSMM_APIEXT LIBXSMM_ATTRIBUTE_USED void LIBXSMM_FSYMBOL(__wrap_sgemv)(const c
   if ((2 < libxsmm_gemm_wrap || 2 > libxsmm_gemm_wrap) && 1 == *incx && 1 == *incy && LIBXSMM_SMM(*m, 1, *n, 2/*RFO*/, sizeof(float))) {
     if (0 != (libxsmm_gemm_wrap & 1)) { /* sequential */
       const int flags = LIBXSMM_GEMM_FLAGS(*trans, 'N');
-      const libxsmm_smmfunction xgemv = libxsmm_smmdispatch(*m, 1, *n, lda, n/*ldb*/, n/*ldc*/, alpha, beta, &flags, NULL);
+      const libxsmm_smmfunction xgemv = libxsmm_smmdispatch(*m, 1, *n, lda, n/*ldb*/, m/*ldc*/, alpha, beta, &flags, NULL);
       if (NULL != xgemv) {
-        LIBXSMM_MMCALL_LDX(xgemv, a, x, y, *m, 1, *n, *lda, *n/*ldb*/, *n/*ldc*/);
+        LIBXSMM_MMCALL_LDX(xgemv, a, x, y, *m, 1, *n, *lda, *n/*ldb*/, *m/*ldc*/);
       }
       else {
         LIBXSMM_GEMV_SYMBOL(float)(trans, m, n, alpha, a, lda, x, incx, beta, y, incy);
@@ -686,7 +684,7 @@ LIBXSMM_APIEXT void libxsmm_xgemm_omp(libxsmm_gemm_precision iprec, libxsmm_gemm
 #if defined(_OPENMP)
     if (0 == outerpar) { /* enable internal parallelization */
 # if defined(LIBXSMM_EXT_TASKS)
-      if (0 >= libxsmm_gemm_taskscale)
+      if (0 == libxsmm_gemm_tasks)
 # endif
       {
 #       pragma omp parallel num_threads(nthreads)
@@ -694,7 +692,7 @@ LIBXSMM_APIEXT void libxsmm_xgemm_omp(libxsmm_gemm_precision iprec, libxsmm_gemm
       }
 # if defined(LIBXSMM_EXT_TASKS)
       else { /* tasks requested */
-        const int ntasks = nthreads * libxsmm_gemm_taskscale;
+        const int ntasks = nthreads; /* TODO: apply grain-size */
 #       pragma omp parallel num_threads(nthreads)
         { /* first thread discovering work will launch all tasks */
 #         pragma omp single nowait /* anyone is good */
@@ -709,7 +707,7 @@ LIBXSMM_APIEXT void libxsmm_xgemm_omp(libxsmm_gemm_precision iprec, libxsmm_gemm
     }
     else { /* assume external parallelization */
 # if defined(LIBXSMM_EXT_TASKS) /* implies _OPENMP */
-      const int ntasks = nthreads * (0 < libxsmm_gemm_taskscale ? libxsmm_gemm_taskscale : (LIBXSMM_GEMM_TASKSCALE));
+      const int ntasks = nthreads; /* TODO: apply grain-size */
       int tid; for (tid = 0; tid < ntasks; ++tid) {
 #       pragma omp task untied
         libxsmm_gemm_thread(handle, scratch, a, b, c, tid, ntasks);
@@ -771,7 +769,7 @@ LIBXSMM_API_INLINE void internal_gemm_batch_omp(libxsmm_gemm_precision iprec, li
 #endif
     0 != group_count)
   {
-    int result;
+    int result = EXIT_SUCCESS;
     const int max_npargroups = (int)(0 < libxsmm_gemm_npargroups
       ? LIBXSMM_MIN(libxsmm_gemm_npargroups, LIBXSMM_GEMM_NPARGROUPS) : LIBXSMM_GEMM_NPARGROUPS);
     const libxsmm_gemm_prefetch_type prefetch = libxsmm_get_gemm_prefetch(LIBXSMM_PREFETCH_AUTO);
@@ -783,6 +781,9 @@ LIBXSMM_API_INLINE void internal_gemm_batch_omp(libxsmm_gemm_precision iprec, li
     int group = 0, group_next = LIBXSMM_GEMM_NPARGROUPS;
     libxsmm_code_pointer kernel[LIBXSMM_GEMM_NPARGROUPS];
     libxsmm_blasint base[LIBXSMM_GEMM_NPARGROUPS], i;
+#if !defined(LIBXSMM_EXT_GEMM_PARGROUPS_INFO)
+    int kflags[LIBXSMM_GEMM_NPARGROUPS];
+#endif
     int max_nthreads = 1;
 #if defined(_OPENMP)
 # if defined(LIBXSMM_EXT_TASKS)
@@ -792,7 +793,15 @@ LIBXSMM_API_INLINE void internal_gemm_batch_omp(libxsmm_gemm_precision iprec, li
 # endif
     if (0 == outerpar) max_nthreads = omp_get_max_threads();
 #endif
-    for (i = 0; i < max_npargroups; ++i) base[i] = 0;
+    for (i = 0; i < max_npargroups; ++i) {
+#if !defined(NDEBUG)
+      kernel[i].ptr = NULL;
+# if !defined(LIBXSMM_EXT_GEMM_PARGROUPS_INFO)
+      kflags[i] = 0;
+# endif
+#endif
+      base[i] = 0;
+    }
     for (group = 0; group < ngroups; group = group_next, group_next += max_npargroups) {
       const int npargroups = LIBXSMM_MIN(group_next, ngroups);
       libxsmm_blasint size = 0;
@@ -811,19 +820,25 @@ LIBXSMM_API_INLINE void internal_gemm_batch_omp(libxsmm_gemm_precision iprec, li
           const char *const ta = (NULL != transa ? (transa + g) : NULL);
           const char *const tb = (NULL != transb ? (transb + g) : NULL);
           const int flags = LIBXSMM_GEMM_PFLAGS(ta, tb, LIBXSMM_FLAGS);
+          const void **const galpha = &alpha, **const gbeta = &beta;
           libxsmm_descriptor_blob blob;
+          /* coverity[ptr_arith] */
           libxsmm_gemm_descriptor *const desc = libxsmm_gemm_descriptor_init2(&blob, iprec, oprec, im, in, ik,
             NULL != lda ? lda[g] : (0 == (LIBXSMM_GEMM_FLAG_TRANS_A & flags) ? im : ik),
             NULL != ldb ? ldb[g] : (0 == (LIBXSMM_GEMM_FLAG_TRANS_B & flags) ? ik : in),
-            NULL != ldc ? ldc[g] : im, NULL != alpha ? (&alpha)[g] : NULL, NULL != beta ? (&beta)[g] : NULL,
+            NULL != ldc ? ldc[g] : im, NULL != alpha ? galpha[g] : NULL, NULL != beta ? gbeta[g] : NULL,
             flags, prefetch);
           if (NULL != desc) {
             libxsmm_gemm_internal_set_batchflag(desc, c, index_stride, 0 < group_count ? isize : -asize, 1 != max_nthreads);
             kernel[i].xgemm = libxsmm_xmmdispatch(desc);
           }
-          else kernel[i].pmm = NULL;
+          else kernel[i].ptr = NULL;
           if (NULL != kernel[i].ptr_const) {
             if (size < asize) size = asize;
+#if !defined(LIBXSMM_EXT_GEMM_PARGROUPS_INFO)
+            LIBXSMM_ASSERT(NULL != desc); /* coverity[var_deref_op] */
+            kflags[i] = desc->flags;
+#endif
           }
           else {
             suitable = 0;
@@ -837,11 +852,10 @@ LIBXSMM_API_INLINE void internal_gemm_batch_omp(libxsmm_gemm_precision iprec, li
 #if defined(_OPENMP)
         const int nchunks = (int)((size + libxsmm_gemm_taskgrain - 1) / libxsmm_gemm_taskgrain);
         const int ntasks = nchunks * npargroups, nthreads = LIBXSMM_MIN(max_nthreads, ntasks);
-        result = EXIT_SUCCESS;
         if (1 < nthreads) {
           if (0 == outerpar) { /* enable internal parallelization */
 # if defined(LIBXSMM_EXT_TASKS)
-            if (0 >= libxsmm_gemm_taskscale)
+            if (0 == libxsmm_gemm_tasks)
 # endif
             {
 #             pragma omp parallel for num_threads(nthreads) private(i)
@@ -849,11 +863,17 @@ LIBXSMM_API_INLINE void internal_gemm_batch_omp(libxsmm_gemm_precision iprec, li
                 const libxsmm_blasint j = i * libxsmm_gemm_taskgrain, u = j / size, v = j - u * size, g = group + u;
                 const libxsmm_blasint isize = batchsize[g], asize = LIBXSMM_ABS(isize);
                 if (v < asize) {
-                  const libxsmm_descriptor *const kernel_info = libxsmm_get_kernel_info(kernel[g], NULL/*code_size*/);
+#if defined(LIBXSMM_EXT_GEMM_PARGROUPS_INFO)
+                  libxsmm_mmkernel_info kernel_info;
+#endif
                   /*check*/libxsmm_mmbatch_kernel(kernel[g].xgemm, index_base, index_stride, stride_a, stride_b, stride_c,
                     (const char*)a + sa * base[u], (const char*)b + sb * base[u], (char*)c + sc * base[u],
                     0 < group_count ? isize : -asize, (int)i, nchunks, itypesize, otypesize,
-                    kernel_info->gemm.desc.flags);
+#if defined(LIBXSMM_EXT_GEMM_PARGROUPS_INFO)
+                    EXIT_SUCCESS == libxsmm_get_mmkernel_info(kernel[g].xgemm, &kernel_info) ? kernel_info.flags : 0);
+#else
+                    kflags[g]);
+#endif
                 }
               }
             }
@@ -862,17 +882,23 @@ LIBXSMM_API_INLINE void internal_gemm_batch_omp(libxsmm_gemm_precision iprec, li
 #             pragma omp parallel num_threads(nthreads) private(i)
               { /* first thread discovering work will launch all tasks */
 #               pragma omp single nowait /* anyone is good */
-                for (i = 0; i < ntasks * libxsmm_gemm_taskscale; ++i) {
+                for (i = 0; i < ntasks; ++i) {
                   const libxsmm_blasint j = i * libxsmm_gemm_taskgrain, u = j / size, v = j - u * size, g = group + u;
                   const libxsmm_blasint isize = batchsize[g], asize = LIBXSMM_ABS(isize);
                   if (v < asize) {
 #                   pragma omp task
                     {
-                      const libxsmm_descriptor *const kernel_info = libxsmm_get_kernel_info(kernel[g], NULL/*code_size*/);
+#if defined(LIBXSMM_EXT_GEMM_PARGROUPS_INFO)
+                      libxsmm_mmkernel_info kernel_info;
+#endif
                       /*check*/libxsmm_mmbatch_kernel(kernel[g].xgemm, index_base, index_stride, stride_a, stride_b, stride_c,
-                        (const char*)a + sa * base[i], (const char*)b + sb * base[i], (char*)c + sc * base[i],
-                        0 < group_count ? isize : -asize, (int)i, nchunks * libxsmm_gemm_taskscale, itypesize, otypesize,
-                        kernel_info->gemm.desc.flags);
+                        (const char*)a + sa * base[u], (const char*)b + sb * base[u], (char*)c + sc * base[u],
+                        0 < group_count ? isize : -asize, (int)i, nchunks, itypesize, otypesize,
+#if defined(LIBXSMM_EXT_GEMM_PARGROUPS_INFO)
+                        EXIT_SUCCESS == libxsmm_get_mmkernel_info(kernel[g].xgemm, &kernel_info) ? kernel_info.flags : 0);
+#else
+                        kflags[g]);
+#endif
                     }
                   }
                 }
@@ -881,8 +907,7 @@ LIBXSMM_API_INLINE void internal_gemm_batch_omp(libxsmm_gemm_precision iprec, li
 # endif
           }
           else { /* assume external parallelization */
-            const int scale = (0 < libxsmm_gemm_taskscale ? libxsmm_gemm_taskscale : (LIBXSMM_GEMM_TASKSCALE));
-            for (i = 0; i < ((libxsmm_blasint)ntasks * scale); ++i) {
+            for (i = 0; i < (libxsmm_blasint)ntasks; ++i) {
               const libxsmm_blasint j = i * libxsmm_gemm_taskgrain, u = j / size, v = j - u * size, g = group + u;
               const libxsmm_blasint isize = batchsize[g], asize = LIBXSMM_ABS(isize);
               if (v < asize) {
@@ -890,11 +915,17 @@ LIBXSMM_API_INLINE void internal_gemm_batch_omp(libxsmm_gemm_precision iprec, li
 #               pragma omp task
 #endif
                 {
-                  const libxsmm_descriptor *const kernel_info = libxsmm_get_kernel_info(kernel[g], NULL/*code_size*/);
+#if defined(LIBXSMM_EXT_GEMM_PARGROUPS_INFO)
+                  libxsmm_mmkernel_info kernel_info;
+#endif
                   /*check*/libxsmm_mmbatch_kernel(kernel[g].xgemm, index_base, index_stride, stride_a, stride_b, stride_c,
-                    (const char*)a + sa * base[i], (const char*)b + sb * base[i], (char*)c + sc * base[i],
-                    0 < group_count ? isize : -asize, (int)i, nchunks * scale, itypesize, otypesize,
-                    kernel_info->gemm.desc.flags);
+                    (const char*)a + sa * base[u], (const char*)b + sb * base[u], (char*)c + sc * base[u],
+                    0 < group_count ? isize : -asize, (int)i, nchunks, itypesize, otypesize,
+#if defined(LIBXSMM_EXT_GEMM_PARGROUPS_INFO)
+                    EXIT_SUCCESS == libxsmm_get_mmkernel_info(kernel[g].xgemm, &kernel_info) ? kernel_info.flags : 0);
+#else
+                    kflags[g]);
+#endif
                 }
               }
             }
@@ -910,10 +941,17 @@ LIBXSMM_API_INLINE void internal_gemm_batch_omp(libxsmm_gemm_precision iprec, li
         { /* sequential */
           for (i = 0; i < npargroups; ++i) {
             const libxsmm_blasint g = group + i;
-            const libxsmm_descriptor *const kernel_info = libxsmm_get_kernel_info(kernel[i], NULL/*code_size*/);
+#if defined(LIBXSMM_EXT_GEMM_PARGROUPS_INFO)
+            libxsmm_mmkernel_info kernel_info;
+#endif
             libxsmm_mmbatch_kernel(kernel[i].xgemm, index_base, index_stride, stride_a, stride_b, stride_c,
               (const char*)a + sa * base[i], (const char*)b + sb * base[i], (char*)c + sc * base[i], batchsize[g],
-              0/*tid*/, 1/*nthreads*/, itypesize, otypesize, kernel_info->gemm.desc.flags);
+              0/*tid*/, 1/*nthreads*/, itypesize, otypesize,
+#if defined(LIBXSMM_EXT_GEMM_PARGROUPS_INFO)
+              EXIT_SUCCESS == libxsmm_get_mmkernel_info(kernel[i].xgemm, &kernel_info) ? kernel_info.flags : 0);
+#else
+              kflags[i]);
+#endif
           }
         }
       }
@@ -930,7 +968,11 @@ LIBXSMM_API_INLINE void internal_gemm_batch_omp(libxsmm_gemm_precision iprec, li
           const libxsmm_blasint ilda = (NULL != lda ? lda[g] : (0 == (LIBXSMM_GEMM_FLAG_TRANS_A & flags) ? im : ik));
           const libxsmm_blasint ildb = (NULL != ldb ? ldb[g] : (0 == (LIBXSMM_GEMM_FLAG_TRANS_B & flags) ? ik : in));
           const libxsmm_blasint ildc = (NULL != ldc ? ldc[g] : im);
-          const void *const ialpha = (NULL != alpha ? (&alpha)[g] : NULL), *const ibeta = (NULL != beta ? (&beta)[g] : NULL);
+          const void **const galpha = &alpha, **const gbeta = &beta;
+          /* coverity[overrun-local] */
+          const void *const ialpha = (NULL != alpha ? galpha[g] : NULL);
+          /* coverity[overrun-local] */
+          const void *const ibeta = (NULL != beta ? gbeta[g] : NULL);
           if (EXIT_SUCCESS == libxsmm_mmbatch_blas(iprec, oprec, ta, tb, im, in, ik, ialpha,
             (const char*)a + sa * base[i], &ilda, (const char*)b + sb * base[i], &ildb, ibeta, (char*)c + sc * base[i], &ildc,
             index_base, index_stride, stride_a, stride_b, stride_c, batchsize[g]))
@@ -1049,7 +1091,7 @@ LIBXSMM_APIEXT void libxsmm_mmbatch_begin(libxsmm_gemm_precision precision,
       ++internal_ext_gemm_batchdepth;
 
       /* ensure descriptor does not match any GEMM such that... */
-      memset(&libxsmm_mmbatch_desc, 0, sizeof(libxsmm_mmbatch_desc));
+      LIBXSMM_MEMZERO127(&libxsmm_mmbatch_desc);
       /* ...the batch stops and completely flushes */
       if (0 != internal_ext_gemm_batchsize) {
         result = internal_mmbatch_flush(internal_ext_gemm_batchdesc + i,
@@ -1096,7 +1138,8 @@ LIBXSMM_APIEXT void libxsmm_mmbatch_end(void)
 #   pragma warning(push)
 #   pragma warning(disable: 26115) /* try-lock is treated incorrectly by static analysis */
 # endif
-  if (LIBXSMM_LOCK_ACQUIRED(LIBXSMM_LOCK_DEFAULT) == LIBXSMM_LOCK_TRYLOCK(LIBXSMM_LOCK_DEFAULT, &libxsmm_mmbatch_lock)) {
+  /*const*/ int trystate = LIBXSMM_LOCK_TRYLOCK(LIBXSMM_LOCK_DEFAULT, &libxsmm_mmbatch_lock);
+  if (LIBXSMM_LOCK_ACQUIRED(LIBXSMM_LOCK_DEFAULT) == trystate) {
     const unsigned int max_batchsize = (unsigned int)((LIBXSMM_GEMM_MMBATCH_SCALE) * libxsmm_mmbatch_size);
     const libxsmm_gemm_descriptor flushdesc = libxsmm_mmbatch_desc;
     static int error_once = 0;
@@ -1104,7 +1147,7 @@ LIBXSMM_APIEXT void libxsmm_mmbatch_end(void)
     const unsigned int mmbatch_maxdepth = LIBXSMM_UP2POT(LIBXSMM_EXT_GEMM_MMBATCH_MAXDEPTH);
 #endif
     /* ensure descriptor does not match any GEMM such that... */
-    memset(&libxsmm_mmbatch_desc, 0, sizeof(libxsmm_mmbatch_desc));
+    LIBXSMM_MEMZERO127(&libxsmm_mmbatch_desc);
     /* ...the batch stops and completely flushes */
     if (EXIT_SUCCESS == internal_mmbatch_flush(&flushdesc,
       0 != internal_ext_gemm_batchsize ? (((internal_ext_gemm_batchsize - 1) % max_batchsize) + 1) : 0,
@@ -1129,7 +1172,7 @@ LIBXSMM_APIEXT void libxsmm_mmbatch_end(void)
 }
 
 
-#if defined(LIBXSMM_BUILD) && defined(LIBXSMM_BUILD_EXT) && !defined(LIBXSMM_NOFORTRAN)
+#if defined(LIBXSMM_BUILD) && defined(LIBXSMM_BUILD_EXT) && (!defined(LIBXSMM_NOFORTRAN) || defined(__clang_analyzer__))
 
 /* implementation provided for Fortran 77 compatibility */
 LIBXSMM_APIEXT void LIBXSMM_FSYMBOL(libxsmm_xgemm_omp)(const libxsmm_gemm_precision*, const libxsmm_gemm_precision*,
@@ -1221,5 +1264,5 @@ LIBXSMM_APIEXT void LIBXSMM_FSYMBOL(libxsmm_mmbatch_end)(void)
   libxsmm_mmbatch_end();
 }
 
-#endif /*defined(LIBXSMM_BUILD) && defined(LIBXSMM_BUILD_EXT)*/
+#endif /*defined(LIBXSMM_BUILD) && defined(LIBXSMM_BUILD_EXT) && (!defined(LIBXSMM_NOFORTRAN) || defined(__clang_analyzer__))*/
 
