@@ -1,49 +1,33 @@
 /******************************************************************************
-** Copyright (c) 2018-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Hans Pabst (Intel Corp.)
 ******************************************************************************/
 #include "libxsmm_main.h"
 
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
-#endif
-#include <string.h>
-#include <stdio.h>
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(pop)
-#endif
-
 #if !defined(LIBXSMM_PRODUCT_LIMIT)
 # define LIBXSMM_PRODUCT_LIMIT 1024
 #endif
 
 
+#if defined(LIBXSMM_INTRINSICS_AVX512)
+LIBXSMM_APIVAR_PUBLIC_DEF(unsigned int libxsmm_intrinsics_mm512_rng_state0[16]);
+LIBXSMM_APIVAR_PUBLIC_DEF(unsigned int libxsmm_intrinsics_mm512_rng_state1[16]);
+LIBXSMM_APIVAR_PUBLIC_DEF(unsigned int libxsmm_intrinsics_mm512_rng_state2[16]);
+LIBXSMM_APIVAR_PUBLIC_DEF(unsigned int libxsmm_intrinsics_mm512_rng_state3[16]);
+#endif
+
+/* definition of corresponding variables */
+LIBXSMM_APIVAR_PUBLIC_DEF(unsigned int libxsmm_ninit);
+LIBXSMM_APIVAR_PUBLIC_DEF(int libxsmm_target_archid);
+LIBXSMM_APIVAR_PUBLIC_DEF(int libxsmm_verbosity);
+
+
 LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_dgemm_descriptor_init(libxsmm_descriptor_blob* blob,
   libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k, libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
   double alpha, double beta, int flags, int prefetch)
@@ -113,29 +97,6 @@ LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_wigemm_descriptor_init(libxsmm_desc
 }
 
 
-LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_wsgemm_descriptor_init(libxsmm_descriptor_blob* blob,
-  libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k, libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
-  float alpha, float beta, int flags, int prefetch)
-{
-  union {
-    libxsmm_gemm_descriptor* ptr;
-    libxsmm_descriptor_blob* blob;
-  } result;
-  if (LIBXSMM_GEMM_NO_BYPASS(flags, alpha, beta)
-    && LIBXSMM_GEMM_NO_BYPASS_DIMS(lda, ldb, ldc)
-    && LIBXSMM_GEMM_NO_BYPASS_DIMS(m, n, k))
-  {
-    result.blob = blob;
-    LIBXSMM_GEMM_DESCRIPTOR2(*result.ptr, LIBXSMM_GEMM_PRECISION(short), LIBXSMM_GEMM_PRECISION(float),
-      flags, m, n, k, lda, ldb, ldc, alpha, beta, prefetch);
-  }
-  else { /* unsupported */
-    result.ptr = NULL;
-  }
-  return result.ptr;
-}
-
-
 LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_bsgemm_descriptor_init(libxsmm_descriptor_blob* blob,
   libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k, libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
   float alpha, float beta, int flags, int prefetch)
@@ -182,6 +143,52 @@ LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_bgemm_descriptor_init(libxsmm_descr
 }
 
 
+LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_bigemm_descriptor_init(libxsmm_descriptor_blob* blob,
+  libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k, libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
+  int alpha, int beta, int flags, int prefetch)
+{
+  union {
+    libxsmm_gemm_descriptor* ptr;
+    libxsmm_descriptor_blob* blob;
+  } result;
+  if  (LIBXSMM_GEMM_NO_BYPASS(flags, alpha, beta)
+    && LIBXSMM_GEMM_NO_BYPASS_DIMS(lda, ldb, ldc)
+    && LIBXSMM_GEMM_NO_BYPASS_DIMS(m, n, k))
+  {
+    result.blob = blob;
+    LIBXSMM_GEMM_DESCRIPTOR2(*result.ptr, LIBXSMM_GEMM_PRECISION(char), LIBXSMM_GEMM_PRECISION(int),
+      flags, m, n, k, lda, ldb, ldc, alpha, beta, prefetch);
+  }
+  else { /* unsupported */
+    result.ptr = NULL;
+  }
+  return result.ptr;
+}
+
+
+LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_bbgemm_descriptor_init(libxsmm_descriptor_blob* blob,
+  libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k, libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc,
+  int alpha, int beta, int flags, int prefetch)
+{
+  union {
+    libxsmm_gemm_descriptor* ptr;
+    libxsmm_descriptor_blob* blob;
+  } result;
+  if  (LIBXSMM_GEMM_NO_BYPASS(flags, alpha, beta)
+    && LIBXSMM_GEMM_NO_BYPASS_DIMS(lda, ldb, ldc)
+    && LIBXSMM_GEMM_NO_BYPASS_DIMS(m, n, k))
+  {
+    result.blob = blob;
+    LIBXSMM_GEMM_DESCRIPTOR2(*result.ptr, LIBXSMM_GEMM_PRECISION(char), LIBXSMM_GEMM_PRECISION(char),
+      flags, m, n, k, lda, ldb, ldc, alpha, beta, prefetch);
+  }
+  else { /* unsupported */
+    result.ptr = NULL;
+  }
+  return result.ptr;
+}
+
+
 LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_gemm_descriptor_dinit(libxsmm_descriptor_blob* blob,
   libxsmm_gemm_precision precision, libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k,
   libxsmm_blasint lda, libxsmm_blasint ldb, libxsmm_blasint ldc, double alpha, double beta,
@@ -210,14 +217,16 @@ LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_gemm_descriptor_dinit2(libxsmm_desc
         (float)alpha, (float)beta, flags, prefetch);
     } break;
     case LIBXSMM_GEMM_PRECISION_I16: {
+      result = libxsmm_wigemm_descriptor_init(blob, m, n, k, lda, ldb, ldc,
+        (int)alpha, (int)beta, flags, prefetch);
+    } break;
+    case LIBXSMM_GEMM_PRECISION_I8: {
       if (LIBXSMM_GEMM_PRECISION_I32 == oprec) {
-        result = libxsmm_wigemm_descriptor_init(blob, m, n, k, lda, ldb, ldc,
+        result = libxsmm_bigemm_descriptor_init(blob, m, n, k, lda, ldb, ldc,
+          (int)alpha, (int)beta, flags, prefetch);
+      } else if (LIBXSMM_GEMM_PRECISION_I8 == oprec) {
+        result = libxsmm_bbgemm_descriptor_init(blob, m, n, k, lda, ldb, ldc,
           (int)alpha, (int)beta, flags, prefetch);
-      }
-      else {
-        LIBXSMM_ASSERT(LIBXSMM_GEMM_PRECISION_F32 == oprec);
-        result = libxsmm_wsgemm_descriptor_init(blob, m, n, k, lda, ldb, ldc,
-          (float)alpha, (float)beta, flags, prefetch);
       }
     } break;
     case LIBXSMM_GEMM_PRECISION_BF16: {
@@ -286,24 +295,35 @@ LIBXSMM_API libxsmm_gemm_descriptor* libxsmm_gemm_descriptor_init3(libxsmm_descr
       if (NULL != dbeta) *dbeta = (double)bb;
     } break;
     case LIBXSMM_GEMM_PRECISION_I16: {
+      /**
+       * Take alpha and beta as short data although wgemm works on integers.
+       * However, alpha and beta are only JIT-supported for certain values,
+       * and the call-side may not distinct different input and output types
+       * (integer/short), hence it is safer to only read short data.
+       */
+      const short aa = (short)(NULL != alpha ? *((const short*)alpha) : (LIBXSMM_ALPHA));
+      const short bb = (short)(NULL != beta  ? *((const short*)beta)  : (LIBXSMM_BETA));
+      result = libxsmm_wigemm_descriptor_init(blob, m, n, k, lda, ldb, ldc, aa, bb, flags, prefetch);
+      if (NULL != dalpha) *dalpha = (double)aa;
+      if (NULL != dbeta) *dbeta = (double)bb;
+    } break;
+    case LIBXSMM_GEMM_PRECISION_I8: {
+      /**
+       * Take alpha and beta as short data although wgemm works on integers.
+       * However, alpha and beta are only JIT-supported for certain values,
+       * and the call-side may not distinct different input and output types
+       * (integer/short), hence it is safer to only read short data.
+       */
       if (LIBXSMM_GEMM_PRECISION_I32 == oprec) {
-        /**
-         * Take alpha and beta as short data although wgemm works on integers.
-         * However, alpha and beta are only JIT-supported for certain values,
-         * and the call-side may not distinct different input and output types
-         * (integer/short), hence it is safer to only read short data.
-         */
         const short aa = (short)(NULL != alpha ? *((const short*)alpha) : (LIBXSMM_ALPHA));
         const short bb = (short)(NULL != beta  ? *((const short*)beta)  : (LIBXSMM_BETA));
-        result = libxsmm_wigemm_descriptor_init(blob, m, n, k, lda, ldb, ldc, aa, bb, flags, prefetch);
+        result = libxsmm_bigemm_descriptor_init(blob, m, n, k, lda, ldb, ldc, aa, bb, flags, prefetch);
         if (NULL != dalpha) *dalpha = (double)aa;
         if (NULL != dbeta) *dbeta = (double)bb;
-      }
-      else {
-        const float aa = (NULL != alpha ? *((const float*)alpha) : (LIBXSMM_ALPHA));
-        const float bb = (NULL != beta  ? *((const float*)beta)  : (LIBXSMM_BETA));
-        LIBXSMM_ASSERT(LIBXSMM_GEMM_PRECISION_F32 == oprec);
-        result = libxsmm_wsgemm_descriptor_init(blob, m, n, k, lda, ldb, ldc, aa, bb, flags, prefetch);
+      } else if (LIBXSMM_GEMM_PRECISION_I8 == oprec) {
+        const short aa = (short)(NULL != alpha ? *((const short*)alpha) : (LIBXSMM_ALPHA));
+        const short bb = (short)(NULL != beta  ? *((const short*)beta)  : (LIBXSMM_BETA));
+        result = libxsmm_bbgemm_descriptor_init(blob, m, n, k, lda, ldb, ldc, aa, bb, flags, prefetch);
         if (NULL != dalpha) *dalpha = (double)aa;
         if (NULL != dbeta) *dbeta = (double)bb;
       }
@@ -361,21 +381,32 @@ LIBXSMM_API libxsmm_mcopy_descriptor* libxsmm_mcopy_descriptor_init(libxsmm_desc
     libxsmm_mcopy_descriptor* ptr;
     libxsmm_descriptor_blob* blob;
   } result;
-  if (0 == LIBXSMM_MOD2(typesize, 4)) { /* TODO: more general kernel */
-    const unsigned int typescale = typesize / 4;
-    LIBXSMM_DESCRIPTOR_CLEAR(blob);
-    result.blob = blob;
-    result.ptr->unroll_level = (unsigned char)((NULL == unroll || 0 >= *unroll) ? 2/*default*/ : LIBXSMM_MIN(*unroll, 64));
-    result.ptr->typesize = (unsigned char)/*typesize*/4;
-    result.ptr->prefetch = (unsigned char)prefetch;
-    result.ptr->flags = (unsigned char)flags;
-    result.ptr->ldi = ldi * typescale;
-    result.ptr->ldo = ldo * typescale;
-    result.ptr->m = m * typescale;
+  LIBXSMM_DESCRIPTOR_CLEAR(blob);
+  result.blob = blob;
+  result.ptr->prefetch = (unsigned char)prefetch;
+  result.ptr->flags = (unsigned char)flags;
+  /* TODO: backend supports typesize <= 4, but certain AVX1/AVX2-kernels are incorrect */
+  if (4 >= typesize && (LIBXSMM_X86_AVX512 <= libxsmm_target_archid || 32 <= (typesize * m) || ldi == ldo)) {
+    result.ptr->typesize = (unsigned char)typesize;
+    result.ptr->unroll_level = (unsigned char)((NULL == unroll || 0 >= *unroll) ? LIBXSMM_MAX(8 / result.ptr->typesize, 1) : LIBXSMM_MIN(*unroll, 64));
+    result.ptr->ldi = ldi;
+    result.ptr->ldo = ldo;
+    result.ptr->m = m;
     result.ptr->n = n;
   }
-  else {
-    result.ptr = NULL;
+  else { /* fix-up incl. DP-support */
+    result.ptr->typesize = 4;
+    result.ptr->unroll_level = 2;
+    result.ptr->ldi = ldi * typesize / 4; /* scale */
+    result.ptr->ldo = ldo * typesize / 4; /* scale */
+    result.ptr->m = m * typesize / 4; /* scale */
+    result.ptr->n = n;
+    if (((typesize * ldi) != (4 * result.ptr->ldi)
+      || (typesize * ldo) != (4 * result.ptr->ldo)
+      || (typesize * m) != (4 * result.ptr->m)))
+    {
+      result.ptr = NULL;
+    }
   }
   return result.ptr;
 }
