@@ -1,37 +1,23 @@
 /******************************************************************************
-** Copyright (c) 2014-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
-/* Hans Pabst (Intel Corp.), Rajkishore Barik (Intel Corp. )
+/* Hans Pabst (Intel Corp.)
 ******************************************************************************/
 #ifndef LIBXSMM_MAIN_H
 #define LIBXSMM_MAIN_H
 
 #include <libxsmm.h>
+/**
+ * TF includes src/libxsmm_main.h and uses LIBXSMM's sync primitives
+ * without including libxsmm_sync. However, libxsmm_sync.h shall be
+ * an explicit include separate from including libxsmm.h.
+ */
+#include "libxsmm_sync.h"
 
 /** Allow external definition to enable testing corner cases (exhausted registry space). */
 #if !defined(LIBXSMM_CAPACITY_REGISTRY) /* must be POT */
@@ -41,32 +27,59 @@
 # define LIBXSMM_CAPACITY_CACHE 16
 #endif
 
-#if !defined(LIBXSMM_MAX_NTHREADS)
-# define LIBXSMM_MAX_NTHREADS 1024
+#if !defined(LIBXSMM_PAGE_MINSIZE)
+# define LIBXSMM_PAGE_MINSIZE 4096 /* 4 KB */
 #endif
-#if !defined(LIBXSMM_MALLOC_SCRATCH_MAX_NPOOLS)
-# define LIBXSMM_MALLOC_SCRATCH_MAX_NPOOLS LIBXSMM_MAX_NTHREADS
-#endif
-#if !defined(LIBXSMM_MALLOC_SCRATCH_LIMIT)
-# define LIBXSMM_MALLOC_SCRATCH_LIMIT (4ULL << 30) /* 4 GB */
-#endif
-#if !defined(LIBXSMM_MALLOC_MMAP_SCRATCH) && 0
-# define LIBXSMM_MALLOC_MMAP_SCRATCH
-#endif
-#if !defined(LIBXSMM_MALLOC_SCRATCH_SCALE)
-# if defined(LIBXSMM_MALLOC_MMAP_SCRATCH)
-#   define LIBXSMM_MALLOC_SCRATCH_SCALE 1.3
+
+#if !defined(LIBXSMM_NTHREADS_MAX)
+# if (0 != LIBXSMM_SYNC)
+#   define LIBXSMM_NTHREADS_MAX 1024
 # else
-#   define LIBXSMM_MALLOC_SCRATCH_SCALE 1.0
+#   define LIBXSMM_NTHREADS_MAX 1
 # endif
 #endif
+/* code relies on LIBXSMM_NTHREADS_MAX or v/forks */
+#if !defined(LIBXSMM_NTHREADS_USE) && 1
+# define LIBXSMM_NTHREADS_USE
+#endif
+#if !defined(LIBXSMM_MALLOC_SCRATCH_MAX_NPOOLS)
+# define LIBXSMM_MALLOC_SCRATCH_MAX_NPOOLS LIBXSMM_NTHREADS_MAX
+#endif
+#if !defined(LIBXSMM_MALLOC_SCRATCH_SCALE)
+# define LIBXSMM_MALLOC_SCRATCH_SCALE 1.0
+#endif
+#if !defined(LIBXSMM_MALLOC_LIMIT)
+# define LIBXSMM_MALLOC_LIMIT (2U << 20) /* 2 MB */
+#endif
 #if !defined(LIBXSMM_MALLOC_INTERNAL_CALLER_ID)
-# define LIBXSMM_MALLOC_INTERNAL_CALLER_ID ((uintptr_t)-1)
+# define LIBXSMM_MALLOC_INTERNAL_CALLER_ID ((uintptr_t)LIBXSMM_UNLIMITED)
 #endif
 #if !defined(LIBXSMM_MALLOC_INTERNAL_CALLER)
 # define LIBXSMM_MALLOC_INTERNAL_CALLER ((const void*)(LIBXSMM_MALLOC_INTERNAL_CALLER_ID))
 #endif
 
+#if !defined(LIBXSMM_INTERCEPT_DYNAMIC) && defined(LIBXSMM_BUILD) && \
+  (defined(__GNUC__) || defined(_CRAYC)) && !defined(_WIN32) && !defined(__CYGWIN__) && \
+  !(defined(__APPLE__) && defined(__MACH__) && LIBXSMM_VERSION3(6, 1, 0) >= \
+    LIBXSMM_VERSION3(__clang_major__, __clang_minor__, __clang_patchlevel__))
+# define LIBXSMM_INTERCEPT_DYNAMIC
+#endif
+
+#if defined(LIBXSMM_INTERCEPT_DYNAMIC)
+# if defined(LIBXSMM_OFFLOAD_TARGET)
+#   pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
+# endif
+# include <dlfcn.h>
+# if defined(LIBXSMM_OFFLOAD_TARGET)
+#   pragma offload_attribute(pop)
+# endif
+# if !defined(RTLD_NEXT)
+#   define LIBXSMM_RTLD_NEXT ((void*)-1l)
+# else
+#   define LIBXSMM_RTLD_NEXT RTLD_NEXT
+# endif
+#endif
+
 #if !defined(LIBXSMM_VERBOSITY_HIGH)
 # define LIBXSMM_VERBOSITY_HIGH 3 /* secondary warning or info-verbosity */
 #endif
@@ -93,10 +106,7 @@
 
 /** Check if M, N, K, or LDx fits into the descriptor. */
 #if (0 != LIBXSMM_ILP64)
-# define LIBXSMM_GEMM_NO_BYPASS_DIMS(M, N, K) ( \
-    ((unsigned int)(-1)) >= ((unsigned int)(M)) && \
-    ((unsigned int)(-1)) >= ((unsigned int)(N)) && \
-    ((unsigned int)(-1)) >= ((unsigned int)(K)))
+# define LIBXSMM_GEMM_NO_BYPASS_DIMS(M, N, K) (0xFFFFFFFF >= (M) && 0xFFFFFFFF >= (N) && 0xFFFFFFFF >= (K))
 #else /* always fits */
 # define LIBXSMM_GEMM_NO_BYPASS_DIMS(M, N, K) 1
 #endif
@@ -108,23 +118,26 @@
 #endif
 
 #if defined(LIBXSMM_UNPACKED)
-# define LIBXSMM_DESCRIPTOR_CLEAR(BLOB) memset(BLOB, 0, LIBXSMM_DESCRIPTOR_MAXSIZE)
+# define LIBXSMM_DESCRIPTOR_CLEAR_AUX(DST, SIZE) LIBXSMM_MEMSET127(DST, 0, SIZE)
 #else
-# define LIBXSMM_DESCRIPTOR_CLEAR(BLOB) LIBXSMM_ASSERT(BLOB)
+# define LIBXSMM_DESCRIPTOR_CLEAR_AUX(DST, SIZE)
 #endif
+#define LIBXSMM_DESCRIPTOR_CLEAR(BLOB) \
+  LIBXSMM_ASSERT((LIBXSMM_DESCRIPTOR_MAXSIZE) == sizeof(*(BLOB))); \
+  LIBXSMM_DESCRIPTOR_CLEAR_AUX(BLOB, LIBXSMM_DESCRIPTOR_MAXSIZE)
 
 /** Low-level/internal GEMM descriptor initialization. */
 #define LIBXSMM_GEMM_DESCRIPTOR(DESCRIPTOR, DATA_TYPE, FLAGS, M, N, K, LDA, LDB, LDC, ALPHA, BETA, PREFETCH) \
   LIBXSMM_GEMM_DESCRIPTOR_DIM_CHECK(LDA, LDB, LDC); \
   LIBXSMM_GEMM_DESCRIPTOR_DIM_CHECK(M, N, K); \
-  LIBXSMM_DESCRIPTOR_CLEAR(&(DESCRIPTOR)); \
+  LIBXSMM_DESCRIPTOR_CLEAR_AUX(&(DESCRIPTOR), sizeof(DESCRIPTOR)); \
   (DESCRIPTOR).datatype = (unsigned char)(DATA_TYPE); (DESCRIPTOR).prefetch = (unsigned char)(PREFETCH); \
-  (DESCRIPTOR).flags = (unsigned short)((FLAGS) \
+  (DESCRIPTOR).flags = (unsigned int)((FLAGS) \
     /*| (LIBXSMM_NEQ(0, ALPHA) ? 0 : LIBXSMM_GEMM_FLAG_ALPHA_0)*/ \
     | (LIBXSMM_NEQ(0, BETA) ? 0 : LIBXSMM_GEMM_FLAG_BETA_0)); \
   (DESCRIPTOR).m   = (unsigned int)(M);   (DESCRIPTOR).n   = (unsigned int)(N);   (DESCRIPTOR).k   = (unsigned int)(K); \
   (DESCRIPTOR).lda = (unsigned int)(LDA); (DESCRIPTOR).ldb = (unsigned int)(LDB); (DESCRIPTOR).ldc = (unsigned int)(LDC); \
-  (DESCRIPTOR).pad = 0; (DESCRIPTOR).c1 = 0; (DESCRIPTOR).c2 = 0
+  LIBXSMM_PAD((DESCRIPTOR).pad = 0) (DESCRIPTOR).c1 = 0; (DESCRIPTOR).c2 = 0; (DESCRIPTOR).c3 = 0
 
 /** Similar to LIBXSMM_GEMM_DESCRIPTOR, but separately taking the input-/output-precision. */
 #define LIBXSMM_GEMM_DESCRIPTOR2(DESCRIPTOR, IPREC, OPREC, FLAGS, M, N, K, LDA, LDB, LDC, ALPHA, BETA, PREFETCH) \
@@ -155,22 +168,24 @@
 * The size of the structure is padded to LIBXSMM_DESCRIPTOR_MAXSIZE.
 */
 LIBXSMM_EXTERN_C LIBXSMM_PACKED(struct LIBXSMM_RETARGETABLE) libxsmm_gemm_descriptor {
-  /** Denotes the data-type. */
-  unsigned char datatype;
-  /** Prefetch strategy. */
-  unsigned char prefetch;
-  /** Set of flags. */
-  unsigned short flags;
   /** Extents of the matrix. */
   unsigned int m, n, k;
   /** Leading dimensions. */
   unsigned int lda, ldb, ldc;
+  /** Set of flags. */
+  unsigned int flags;
+  /** Prefetch strategy. */
+  unsigned char prefetch;
+  /** Denotes the data-type. */
+  unsigned char datatype;
   /** Ignored entry. */
-  unsigned int pad;
-  /** Description. */
+  LIBXSMM_PAD(unsigned short pad)
+  /** multipurpose 64bit field, currently used for: a) stride_a in brgemm */
   unsigned long long c1;
-  /** Description. */
+  /** multipurpose 64bit field, currently used for: a) stride_b in brgemm */
   unsigned long long c2;
+  /** multipurpose 8bit field, currently used for: a) unroll hint in brgemm */
+  unsigned char c3;
 };
 
 /** Packed structure storing the matcopy argument description. */
@@ -236,6 +251,7 @@ LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE LIBXSMM_MAY_ALIAS libxsmm_c
   const unsigned int* row_ptr;
   const unsigned int* column_idx;
   const void* values;
+  unsigned int packed_width;
 } libxsmm_csr_soa_descriptor;
 
 LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE LIBXSMM_MAY_ALIAS libxsmm_csc_soa_descriptor {
@@ -243,14 +259,17 @@ LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE LIBXSMM_MAY_ALIAS libxsmm_c
   const unsigned int* column_ptr;
   const unsigned int* row_idx;
   const void* values;
+  unsigned int packed_width;
 } libxsmm_csc_soa_descriptor;
 
 LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE LIBXSMM_MAY_ALIAS libxsmm_pgemm_ac_rm_descriptor {
   const libxsmm_gemm_descriptor* gemm;
+  unsigned int packed_width;
 } libxsmm_pgemm_ac_rm_descriptor;
 
 LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE LIBXSMM_MAY_ALIAS libxsmm_pgemm_bc_rm_descriptor {
   const libxsmm_gemm_descriptor* gemm;
+  unsigned int packed_width;
 } libxsmm_pgemm_bc_rm_descriptor;
 
 LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE LIBXSMM_MAY_ALIAS libxsmm_csr_reg_descriptor {
@@ -260,81 +279,15 @@ LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE LIBXSMM_MAY_ALIAS libxsmm_c
   const void* values;
 } libxsmm_csr_reg_descriptor;
 
-/** Function type used for convolutions (single-precision); the actual signature depends on the kind of convolution. */
-LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_sconvfunction)(
-  const float* input1, const float* input2, float* output,
-  const float* ipf1, const float* ipf2, const float* opf, ...);
-
-LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_bf16convfunction)(
-  const libxsmm_bfloat16* input1, const libxsmm_bfloat16* input2, libxsmm_bfloat16* output,
-  const libxsmm_bfloat16* ipf1, const libxsmm_bfloat16* ipf2, const libxsmm_bfloat16* opf, ...);
-
-LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_bf16f32convfunction)(
-  const libxsmm_bfloat16* input1, const float* input2, libxsmm_bfloat16* output,
-  const libxsmm_bfloat16* ipf1, const float* ipf2, const libxsmm_bfloat16* opf, ...);
-
-LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_wconvfunction)(
-  const short* input1, const short* input2, int* output,
-  const short* ipf1, const short* ipf2, const int* opf, ...);
-
-LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_wsconvfunction)(
-  const short* input1, const short* input2, float* output,
-  const short* ipf1, const short* ipf2, const float* opf, ...);
-
-LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_uwsconvfunction)(
-  short* input1, float* input2, short* output,
-  short* ipf1, float* ipf2, short* opf, ...);
-
-LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_bdbconvfunction)(
-  unsigned char* input1, int* input2, unsigned char* output,
-  unsigned char* ipf1, int* ipf2, unsigned char* opf, ...);
-
-LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_busconvfunction)(
-  const unsigned char* input1, const char* input2, short* output,
-  const unsigned char* ipf1, const char* ipf2, const short* opf, ...);
-
-LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_budconvfunction)(
-  const unsigned char* input1, const char* input2, int* output,
-  const unsigned char* ipf1, const char* ipf2, const int* opf, ...);
-
-LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_wconvfunction_bwd)(
-  int* input1, const short* input2, const short* output,
-  const int* ipf1, const short* ipf2, const short* opf, ...);
-
-LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_busconvfunction_bwd)(
-  const unsigned short* input1, const char* input2, const char* output,
-  const unsigned short* ipf1, const char* ipf2, const char* opf, ...);
-
-LIBXSMM_EXTERN_C typedef LIBXSMM_RETARGETABLE void (*libxsmm_budconvfunction_bwd)(
-  const unsigned int* input1, const char* input2, const char* output,
-  const unsigned int* ipf1, const char* ipf2, const char* opf, ...);
-
-/** Function type which is either libxsmm_sconvfunction or libxsmm_wconvfunction (weak-typed). */
-LIBXSMM_EXTERN_C typedef union LIBXSMM_RETARGETABLE libxsmm_xconvfunction {
-  libxsmm_sconvfunction sconv;
-  libxsmm_bf16convfunction bf16conv;
-  libxsmm_bf16f32convfunction bf1632conv;
-  libxsmm_wsconvfunction wsconv;
-  libxsmm_uwsconvfunction uwsconv;
-  libxsmm_wconvfunction wconv;
-  libxsmm_bdbconvfunction bdbconv;
-  libxsmm_busconvfunction busconv;
-  libxsmm_budconvfunction budconv;
-  libxsmm_wconvfunction_bwd wconvb;
-  libxsmm_busconvfunction_bwd busconvb;
-  libxsmm_budconvfunction_bwd budconvb;
-} libxsmm_xconvfunction;
-
 LIBXSMM_EXTERN_C typedef union LIBXSMM_RETARGETABLE libxsmm_code_pointer {
   void (*ptr_fn)(LIBXSMM_VARIADIC);
   const void* ptr_const;
-  void* pmm;
+  void* ptr;
   uintptr_t uval;
   intptr_t ival;
-  libxsmm_xmmfunction xgemm; /* GEMM: smm, dmm, wimm, wsmm, or void-function */
+  libxsmm_xmmfunction xgemm; /* GEMM: smm, dmm, wimm, or void-function */
   libxsmm_xmcopyfunction xmatcopy;
   libxsmm_xtransfunction xtrans;
-  libxsmm_xconvfunction xconv;
   libxsmm_pgemm_xfunction xpgemm;
   libxsmm_getrf_xfunction xgetrf;
   libxsmm_trmm_xfunction xtrmm;
@@ -364,7 +317,6 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_layer {
   libxsmm_dnn_tensor_format filter_format;
   libxsmm_dnn_conv_fuse_op fuse_ops;
   libxsmm_dnn_conv_option options;
-  libxsmm_dnn_internal_format custom_format_type;    /* Specifies internal LIBXSMM format to be used */
 
   /* These are the batchnorm handles in case of fusion */
   libxsmm_dnn_fusedbatchnorm* pre_bn;
@@ -378,17 +330,12 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_layer {
   int ofhp;
   int ofwp;
   int ifmblock;
-  int ifmblock_hp;
   int ofmblock;
-  int ofmblock_lp;
   int blocksifm;
   int blocksofm;
-  int blocksifm_lp;
-  int blocksofm_lp;
   int fwd_ofw_rb;
   int fwd_ofw_rb_2;
   int fwd_ofh_rb;
-  int fwd_ofh_rb_2;
   int bwd_ofw_rb;
   int bwd_ofh_rb;
   int upd_ofw_rb;
@@ -401,43 +348,49 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_layer {
   int nbImg;
   int blocksifm_blocking;
   int blocksofm_blocking;
-  int blocksimg_blocking;
-  int use_accumulation_scratch;
-  int use_nts_fwd;
-  int use_nts_bwd;
-  int use_nts_upd;
-  int use_fwd_for_bwd;
-  int exploit_duality;
-  int qfma_input_pad;
-  int resize_input;
-  int ifhp_resized;
-  int ifwp_resized;
-  int use_fastpath;
-  int use_hybrid_wu_parallelism;
+  int avoid_acc_load;
+  int avoid_acc_load_bwd;
+  int pack_input;
+  int pack_input_bwd;
+  int spread_input_bwd;
   int weight_copies;
-  int compute_batch_stats_in_kernel_fwd;
-  int compute_batch_stats_in_kernel_bwd;
-  int compute_eltwise_in_kernel_bwd;
-  int perform_relu_in_kernel;
-  int compute_max_in_kernel_fwd;
-  int compute_max_in_kernel_bwd;
   int fuse_batchstats_fwd;
   int fuse_batchstats_bwd;
   int fuse_eltwise_bwd;
   int fuse_relu_bwd;
   int use_lp_kernel;
-  int output_lp_padding;
-  int reduce_weights;
   int use_vperm_transposes;
-  int avoid_output_trans;
-  int avoid_input_trans;
-  int enforce_sfma_kernel;
-  int n_variants;
-  int w_variants;
-  int h_variants;
   int loop_order;
-  int f32_bf16_cvt_rne;
-  int fwd_img_par;
+  int use_ofm_parallelization;
+  int use_ifm_parallelization;
+  int avoid_fmas_in_rim;
+  int avoid_init_weights;
+  int upd_use_batchreduce;
+  int upd_pack_input;
+  int upd_img_br_block;
+  int upd_loop_order;
+  int upd_linearized_tasklist;
+  int upd_avoid_rim_fmas;
+  int fwd_flags;
+  int shuffle_filter_accesses;
+  int use_fallback_fwd_loops;
+  int use_fallback_bwd_loops;
+  int input_pixels;
+  int output_pixels;
+  int n_used_pixels;
+  int pixel_blocking;
+  int use_intermediate_f32_wt_tensor;
+  int upd_linearized_pixels;
+  int ifwp_extended;
+  int ofwp_extended;
+  int batchreduce_h_pixels;
+  int on_the_fly_input_packing;
+  int upd_pack_input_upfront;
+  int use_hybrid_imgofm_parallelization;
+  int compute_pixels;
+  int upd_trans_w_only;
+
+  libxsmm_xtransfunction tr_kernel;
 
   /* internal data representation */
   libxsmm_dnn_tensor* reg_input;
@@ -486,15 +439,13 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_layer {
   void* scratchVk;            /* Winograd weight buffer */
   size_t scratchVk_size;
 
+  libxsmm_code_pointer gemm_fwd;     /* ability to hoist forward GEMMs */
+  libxsmm_code_pointer gemm_fwd2;     /* ability to hoist forward GEMMs */
+
+  unsigned long long *A_offsets;
+  unsigned long long *B_offsets;
+
   /* JIT-generated convolution code */
-  int use_fwd_generic;
-  int use_bwd_generic;
-  int use_upd_generic;
-  /*
-  libxsmm_convolution_forward_descriptor       fwd_desc;
-  libxsmm_convolution_forward_descriptor       bwd_desc;
-  libxsmm_convolution_weight_update_descriptor wu_desc;
-  */
   libxsmm_code_pointer code_fwd[3];
   libxsmm_code_pointer code_bwd[3];
   libxsmm_code_pointer code_upd[2];
@@ -504,48 +455,14 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_layer {
   libxsmm_code_pointer matcopy_upd[3];
 
   /* Data structures and metadata related to per-thread private JITing */
-  int trans_ofw_ifm;
-
-  int *n_entries_fwd;
-  int **compute_fwd_indices_ptrs;
-  int **bn_stats_indices_ptrs;
-  int **bn_aux_stats_indices_ptrs;
-  int **bn_aux_input_indices_ptrs;
-  char **kernel_fwd_variant_ptrs;
   int block_fwd_oj;
-  int block_fwd_oi;
   int block_fwd_ifm;
   int block_fwd_ofm;
-  int *n_fwd_code_segments;
-  segment_t **fwd_code_segments;
-  int *ofh_fwd_start;
-  int *ofh_fwd_end;
-
-  int *n_entries_bwd;
-  int **compute_bwd_indices_ptrs;
-  char **kernel_bwd_variant_ptrs;
   int block_bwd_oj;
-  int block_bwd_oi;
   int block_bwd_ifm;
   int block_bwd_ofm;
-  int *n_bwd_code_segments;
-  segment_t **bwd_code_segments;
-  int *n_entries_trans_bwd;
-  int **transpose_bwd_indices_ptrs;
-  int *ofh_bwd_start;
-  int *ofh_bwd_end;
-
-  int *n_entries_upd;
   int block_upd_ifm;
   int block_upd_ofm;
-  int **compute_upd_indices_ptrs;
-  char **kernel_upd_variant_ptrs;
-  int *n_upd_code_segments;
-  segment_t **upd_code_segments;
-  int *n_entries_init_upd;
-  int **init_upd_indices_ptrs;
-  int *n_entries_copy_upd;
-  int **copy_upd_indices_ptrs;
 };
 
 LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_fusedbatchnorm {
@@ -563,16 +480,37 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_fusedbatchnorm {
   libxsmm_dnn_tensor* expvalue;       /* expected value */
   libxsmm_dnn_tensor* rcpstddev;      /* reciprocal of standard derivation */
   libxsmm_dnn_tensor* variance;       /* variance */
+  libxsmm_dnn_tensor* relumask;       /* relumask */
+  libxsmm_barrier* barrier;           /* barrier */
+  int ifmblock;
+  int ofmblock;
+  int blocksifm;
+  int blocksofm;
+  size_t scratch_size;
+  void* scratch;
+};
+
+LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_fusedgroupnorm {
+  libxsmm_dnn_fusedgroupnorm_desc desc;
+  libxsmm_dnn_tensor* reg_input;      /* input tensor */
+  libxsmm_dnn_tensor* reg_output;     /* output tensor */
+  libxsmm_dnn_tensor* grad_input;     /* grad input tensor */
+  libxsmm_dnn_tensor* grad_output;    /* grad output tensor */
+  libxsmm_dnn_tensor* reg_add;        /* elementwise tensor */
+  libxsmm_dnn_tensor* grad_add;       /* grad elementwise tensor */
+  libxsmm_dnn_tensor* reg_beta;       /* beta tensor */
+  libxsmm_dnn_tensor* reg_gamma;      /* gamma tensor */
+  libxsmm_dnn_tensor* grad_beta;      /* grad beta tensor */
+  libxsmm_dnn_tensor* grad_gamma;     /* grad gamma tensor */
+  libxsmm_dnn_tensor* expvalue;       /* expected value */
+  libxsmm_dnn_tensor* rcpstddev;      /* reciprocal of standard derivation */
+  libxsmm_dnn_tensor* variance;       /* variance */
+  libxsmm_dnn_tensor* relumask;       /* relumask */
   libxsmm_barrier* barrier;           /* barrier */
   int ifmblock;
-  int ifmblock_hp;
   int ofmblock;
-  int ofmblock_lp;
   int blocksifm;
   int blocksofm;
-  int blocksifm_lp;  /* not used */
-  int blocksofm_lp;  /* not used */
-  int fm_lp_block;
   size_t scratch_size;
   void* scratch;
 };
@@ -585,21 +523,45 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_fullyconnected {
   libxsmm_dnn_tensor* grad_output;    /* grad output tensor */
   libxsmm_dnn_tensor* reg_filter;     /* filter tensor */
   libxsmm_dnn_tensor* grad_filter;    /* grad filter tensor */
+  libxsmm_dnn_tensor* reg_bias;       /* bias tensor */
+  libxsmm_dnn_tensor* grad_bias;      /* grad bais tensor */
+  libxsmm_dnn_tensor* relumask;       /* relumask */
   libxsmm_barrier* barrier;           /* barrier */
   int ifmblock;
-  int ifmblock_hp;
   int ofmblock;
-  int ofmblock_lp;
   int blocksifm;
   int blocksofm;
-  int blocksifm_lp;  /* not used */
-  int blocksofm_lp;  /* not used */
+  /* Parameters to tune/specialize FC algorithms */
+  int fwd_2d_blocking;
+  int bwd_2d_blocking;
+  int upd_2d_blocking;
+  int fwd_bf;
+  int bwd_bf;
+  int upd_bf;
+  int fwd_row_teams;
+  int fwd_column_teams;
+  int bwd_row_teams;
+  int bwd_column_teams;
+  int upd_row_teams;
+  int upd_column_teams;
+  int ifm_subtasks;
+  int ofm_subtasks;
+
   int fm_lp_block;
   int bn;
   int bk;
   int bc;
   size_t scratch_size;
+  size_t doutput_scratch_mark;
   void* scratch;
+
+  libxsmm_code_pointer gemm_fwd;     /* ability to hoist forward GEMMs */
+  libxsmm_code_pointer gemm_fwd2;    /* ability to hoist forward GEMMs */
+  libxsmm_code_pointer gemm_fwd3;    /* ability to hoist forward GEMMs */
+  libxsmm_code_pointer gemm_bwd;     /* ability to hoist backward GEMMs */
+  libxsmm_code_pointer gemm_bwd2;    /* ability to hoist backward GEMMs */
+  libxsmm_code_pointer gemm_upd;     /* ability to hoist update GEMMs */
+  libxsmm_code_pointer gemm_upd2;    /* ability to hoist update GEMMs */
 };
 
 LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_pooling {
@@ -611,14 +573,9 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_pooling {
   libxsmm_dnn_tensor* mask;           /* elementwise tensor */
   libxsmm_barrier* barrier;           /* barrier */
   int ifmblock;
-  int ifmblock_hp;
   int ofmblock;
-  int ofmblock_lp;
   int blocksifm;
   int blocksofm;
-  int blocksifm_lp;  /* not used */
-  int blocksofm_lp;  /* not used */
-  int fm_lp_block;
   int ofh;
   int ofw;
   size_t scratch_size;
@@ -627,7 +584,6 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_pooling {
 
 LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_rnncell {
   libxsmm_dnn_rnncell_desc desc;
-  libxsmm_dnn_internal_format custom_format_type; /* required only for comparing layouts */
   libxsmm_blasint T;                              /* sequence length, must be smaller than max sequence length in desc */
   libxsmm_blasint bk;
   libxsmm_blasint bn;
@@ -691,10 +647,15 @@ LIBXSMM_EXTERN_C struct LIBXSMM_RETARGETABLE libxsmm_dnn_rnncell {
   void* ot_scratch;
   void* cit_scratch;
   void* cot_scratch;
-  /* options */
-  int fwd_generic;
-  int bwdupd_generic;
-  /* barrier */
+
+  /* Ability to hoist GEMMs */
+  libxsmm_bsmmfunction_reducebatch_strd fwd_kernela;
+  libxsmm_bsmmfunction_reducebatch_strd fwd_kernelb;
+  libxsmm_bsmmfunction_reducebatch_strd bwdupd_kernela;
+  libxsmm_bsmmfunction_reducebatch_strd bwdupd_kernelb;
+  libxsmm_bsmmfunction_reducebatch_strd bwdupd_kernelc;
+  libxsmm_bsmmfunction_reducebatch_strd bwdupd_kerneld;
+
   libxsmm_barrier* barrier; /* barrier */
 };
 
@@ -728,18 +689,17 @@ typedef enum libxsmm_build_kind {
   LIBXSMM_BUILD_KIND_GETRF      = LIBXSMM_KERNEL_KIND_GETRF,
   LIBXSMM_BUILD_KIND_TRMM       = LIBXSMM_KERNEL_KIND_TRMM,
   LIBXSMM_BUILD_KIND_TRSM       = LIBXSMM_KERNEL_KIND_TRSM,
-  LIBXSMM_BUILD_KIND_PGEMMRMAC  = LIBXSMM_KERNEL_KIND_INVALID,
+  LIBXSMM_BUILD_KIND_USER       = LIBXSMM_KERNEL_KIND_USER,
+  LIBXSMM_BUILD_KIND_PGEMMRMAC  = LIBXSMM_KERNEL_UNREGISTERED,
   LIBXSMM_BUILD_KIND_PGEMMRMBC,
   LIBXSMM_BUILD_KIND_SRSOA,
   LIBXSMM_BUILD_KIND_SCSOA,
-  LIBXSMM_BUILD_KIND_SREG,
-  LIBXSMM_BUILD_KIND_CFWD,
-  LIBXSMM_BUILD_KIND_CUPD
+  LIBXSMM_BUILD_KIND_SREG
 } libxsmm_build_kind;
 
 /** Integral type (libxsmm_kernel_kind, libxsmm_build_kind). */
 #if defined(LIBXSMM_UNPACKED)
-typedef unsigned int libxsmm_descriptor_kind;
+typedef size_t libxsmm_descriptor_kind;
 #else
 typedef unsigned char libxsmm_descriptor_kind;
 #endif
@@ -749,6 +709,7 @@ LIBXSMM_EXTERN_C typedef union LIBXSMM_RETARGETABLE libxsmm_descriptor {
   char data[LIBXSMM_DESCRIPTOR_MAXSIZE];
   libxsmm_descriptor_kind kind; /* kind: must be the first member */
   LIBXSMM_REGDESC(LIBXSMM_PACKED(struct) { libxsmm_descriptor_kind /*repeated kind*/ pad; , desc; });
+  LIBXSMM_PACKED(struct) { libxsmm_descriptor_kind /*repeated kind*/ pad; char desc[1]; } user;
 } libxsmm_descriptor;
 
 LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_build_request {
@@ -760,10 +721,10 @@ LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_build_request {
     const libxsmm_pgemm_ac_rm_descriptor* pgemmacrm;
     const libxsmm_pgemm_bc_rm_descriptor* pgemmbcrm;
     const libxsmm_csr_reg_descriptor* sreg;
-    const libxsmm_convolution_forward_descriptor* cfwd;
-    const libxsmm_convolution_weight_update_descriptor* cupd;
   } descriptor;
   libxsmm_build_kind kind;
+  /* used by user-kind */
+  size_t user_size;
 } libxsmm_build_request;
 
 typedef enum libxsmm_malloc_flags {
@@ -777,21 +738,31 @@ typedef enum libxsmm_malloc_flags {
   LIBXSMM_MALLOC_FLAG_X       = 64,
   LIBXSMM_MALLOC_FLAG_RW  = LIBXSMM_MALLOC_FLAG_R | LIBXSMM_MALLOC_FLAG_W,
   LIBXSMM_MALLOC_FLAG_WX  = LIBXSMM_MALLOC_FLAG_X | LIBXSMM_MALLOC_FLAG_W,
-  LIBXSMM_MALLOC_FLAG_RWX = LIBXSMM_MALLOC_FLAG_X | LIBXSMM_MALLOC_FLAG_RW
+  LIBXSMM_MALLOC_FLAG_RWX = LIBXSMM_MALLOC_FLAG_X | LIBXSMM_MALLOC_FLAG_RW,
+  LIBXSMM_MALLOC_FLAG_VALID       = LIBXSMM_MALLOC_FLAG_SCRATCH |
+      LIBXSMM_MALLOC_FLAG_PRIVATE | LIBXSMM_MALLOC_FLAG_REALLOC |
+      LIBXSMM_MALLOC_FLAG_MMAP    | LIBXSMM_MALLOC_FLAG_RWX
 } libxsmm_malloc_flags;
 
+/**
+ * Format for instance an amount of Bytes like libxsmm_format_size(result, sizeof(result), nbytes, "KMGT", "B", 10).
+ * The value returned is in requested/determined unit so that the user can decide about printing the buffer.
+ */
+LIBXSMM_API_INTERN size_t libxsmm_format_size(char buffer[32], int buffer_size, size_t nbytes, const char scale[], const char* unit, int base);
+
+/** Returns the type-name of data-type (can be also libxsmm_gemm_precision). */
+LIBXSMM_API_INTERN const char* libxsmm_typename(libxsmm_datatype datatype);
+
 /** Returns the type-size of data-type (can be also libxsmm_gemm_precision). */
 LIBXSMM_API unsigned char libxsmm_typesize(libxsmm_datatype datatype);
 
-/** Returns the type-name of data-type (can be also libxsmm_gemm_precision). */
-LIBXSMM_API const char* libxsmm_typename(libxsmm_datatype datatype);
-
-/** Determines the generic value given in double-precision. */
-LIBXSMM_API int libxsmm_cast(libxsmm_datatype datatype, double dvalue, void* value);
-
 /** Retrieve internal information about a buffer (default memory domain). */
 LIBXSMM_API int libxsmm_get_malloc_xinfo(const void* memory, size_t* size, int* flags, void** extra);
 
+/** Initializes malloc hooks and other internals. */
+LIBXSMM_API_INTERN void libxsmm_malloc_init(void);
+LIBXSMM_API_INTERN void libxsmm_malloc_finalize(void);
+
 /** Calculates an alignment depending on supposedly allocated size; alignment can be zero ("auto"). */
 LIBXSMM_API_INTERN size_t libxsmm_alignment(size_t size, size_t alignment);
 
@@ -809,6 +780,9 @@ LIBXSMM_API_INTERN int libxsmm_xset_scratch_allocator(LIBXSMM_LOCK_TYPE(LIBXSMM_
 LIBXSMM_API_INTERN int libxsmm_xget_scratch_allocator(LIBXSMM_LOCK_TYPE(LIBXSMM_LOCK)* lock,
   const void** context, libxsmm_malloc_function* malloc_fn, libxsmm_free_function* free_fn);
 
+/** intern function to calculate blockings, that's private API hence it's in this function */
+LIBXSMM_API_INTERN libxsmm_dnn_err_t libxsmm_dnn_get_feature_map_blocks( int C, int K, int* C_block, int* K_block, int* fm_lp_block, libxsmm_dnn_datatype datatype_in, libxsmm_dnn_datatype datatype_out );
+
 /**
  * Attribute memory allocation and protect with only the necessary flags.
  * This procedure is expected to run only one time per buffer, and may
@@ -823,7 +797,10 @@ LIBXSMM_API_INTERN int libxsmm_xmalloc(void** memory, size_t size, size_t alignm
   /* The extra information is stored along with the allocated chunk; can be NULL/zero. */
   const void* extra, size_t extra_size);
 /** Release memory, which was allocated using libxsmm_[*]malloc. */
-LIBXSMM_API_INTERN void libxsmm_xfree(const void* memory);
+LIBXSMM_API_INTERN void libxsmm_xfree(const void* memory, int check);
+
+/** Like libxsmm_release_scratch, but takes a lock (can be NULL). */
+LIBXSMM_API_INTERN void libxsmm_xrelease_scratch(LIBXSMM_LOCK_TYPE(LIBXSMM_LOCK)* lock);
 
 /** Determines the given value in double-precision based on the given type. */
 LIBXSMM_API_INTERN int libxsmm_dvalue(libxsmm_datatype datatype, const void* value, double* dvalue);
@@ -831,68 +808,62 @@ LIBXSMM_API_INTERN int libxsmm_dvalue(libxsmm_datatype datatype, const void* val
 /** Services a build request, and (optionally) registers the code (use regindex=LIBXSMM_CAPACITY_REGISTRY for unmanaged code). */
 LIBXSMM_API_INTERN int libxsmm_build(const libxsmm_build_request* request, unsigned int regindex, libxsmm_code_pointer* code);
 
-/** Attempts to receive information about JIT-generated code. */
-LIBXSMM_API const libxsmm_descriptor* libxsmm_get_kernel_info(libxsmm_code_pointer code, size_t* size);
+LIBXSMM_EXTERN_C typedef struct LIBXSMM_RETARGETABLE libxsmm_kernel_xinfo {
+  /** Non-zero of kernel is registered. */
+  unsigned int registered;
+  /** Number of FLoating Point OPerationS (FLOPS). */
+  unsigned int nflops;
+} libxsmm_kernel_xinfo;
 
-/** Updates counters of the statistic, which is shown at program termination. */
-LIBXSMM_API_INTERN unsigned int libxsmm_update_mmstatistic(libxsmm_gemm_precision precision,
-  libxsmm_blasint m, libxsmm_blasint n, libxsmm_blasint k, unsigned int ntry, unsigned int ncol);
+/** Receive information about JIT-generated code. */
+LIBXSMM_API_INTERN const libxsmm_kernel_xinfo* libxsmm_get_kernel_xinfo(libxsmm_code_pointer code, const libxsmm_descriptor** desc, size_t* code_size);
 
-/** Returns the current tick of a (monotonic) platform-specific counter; not necessarily CPU cycles. */
+/** Calculates duration in seconds from given RTC ticks. */
+LIBXSMM_API_INTERN double libxsmm_timer_duration_rtc(libxsmm_timer_tickint tick0, libxsmm_timer_tickint tick1);
+/** Returns the current tick of platform-specific real-time clock. */
 LIBXSMM_API_INTERN libxsmm_timer_tickint libxsmm_timer_tick_rtc(void);
+/** Returns the current tick of a (monotonic) platform-specific counter. */
+LIBXSMM_API_INTERN libxsmm_timer_tickint libxsmm_timer_tick_tsc(void);
+
+LIBXSMM_API_INTERN void libxsmm_memory_init(int target_arch);
+LIBXSMM_API_INTERN void libxsmm_memory_finalize(void);
 
 LIBXSMM_API_INTERN void libxsmm_dnn_init(int target_arch);
 LIBXSMM_API_INTERN void libxsmm_dnn_finalize(void);
 
-/** Code generation routine for a forward-convolution kernel. Call libxsmm_release_kernel in order to deallocate the JIT'ted code. */
-LIBXSMM_API_INTERN libxsmm_sconvfunction libxsmm_create_sconv_forward(const libxsmm_convolution_forward_descriptor* descriptor);
-
-/** Code generation routine for a convolution kernel as specified by descriptor. */
-LIBXSMM_API_INTERN libxsmm_sconvfunction libxsmm_create_sconv_update_weights(const libxsmm_convolution_weight_update_descriptor* descriptor);
-
-/** Code generation routine for a forward-convolution kernel. Call libxsmm_release_kernel in order to deallocate the JIT'ted code. */
-LIBXSMM_API_INTERN void* libxsmm_create_xconv_forward(const libxsmm_convolution_forward_descriptor* descriptor);
-
-/** Code generation routine for a convolution kernel as specified by descriptor. */
-LIBXSMM_API_INTERN void* libxsmm_create_xconv_update_weights(const libxsmm_convolution_weight_update_descriptor* descriptor);
-
 /** Global lock; create an own lock for an independent domain. */
-LIBXSMM_APIVAR_ALIGNED(LIBXSMM_LOCK_TYPE(LIBXSMM_LOCK) libxsmm_lock_global);
-/** Target architecture (libxsmm_get_target_archid, libxsmm_set_target_archid). */
-LIBXSMM_APIVAR_ALIGNED(int libxsmm_target_archid);
+LIBXSMM_APIVAR_PUBLIC(LIBXSMM_LOCK_TYPE(LIBXSMM_LOCK) libxsmm_lock_global);
 /** Determines whether a threaded implementation is synchronized or not. */
-LIBXSMM_APIVAR_ALIGNED(int libxsmm_nosync);
-/** Number of threads per core. */
-LIBXSMM_APIVAR_ALIGNED(int libxsmm_nt);
+LIBXSMM_APIVAR_PUBLIC(int libxsmm_nosync);
 
 /** Function used to allocate default memory. */
-LIBXSMM_APIVAR(libxsmm_malloc_function libxsmm_default_malloc_fn);
+LIBXSMM_APIVAR_PRIVATE(libxsmm_malloc_function libxsmm_default_malloc_fn);
 /** Function used to allocate scratch memory. */
-LIBXSMM_APIVAR(libxsmm_malloc_function libxsmm_scratch_malloc_fn);
+LIBXSMM_APIVAR_PRIVATE(libxsmm_malloc_function libxsmm_scratch_malloc_fn);
 /** Function used to release default memory. */
-LIBXSMM_APIVAR(libxsmm_free_function libxsmm_default_free_fn);
+LIBXSMM_APIVAR_PRIVATE(libxsmm_free_function libxsmm_default_free_fn);
 /** Function used to release scratch memory. */
-LIBXSMM_APIVAR(libxsmm_free_function libxsmm_scratch_free_fn);
+LIBXSMM_APIVAR_PRIVATE(libxsmm_free_function libxsmm_scratch_free_fn);
 /** If non-NULL, this context is used by the context-form of memory allocation. */
-LIBXSMM_APIVAR(const void* libxsmm_default_allocator_context);
+LIBXSMM_APIVAR_PRIVATE(const void* libxsmm_default_allocator_context);
 /** If non-NULL, this context is used by the context-form of memory allocation. */
-LIBXSMM_APIVAR(const void* libxsmm_scratch_allocator_context);
-/** Number of discovered threads (per libxsmm_get_tid) */
-LIBXSMM_APIVAR(unsigned int libxsmm_threads_count);
+LIBXSMM_APIVAR_PRIVATE(const void* libxsmm_scratch_allocator_context);
 /** Number of scratch memory pools used; clamped against internal maximum. */
-LIBXSMM_APIVAR(unsigned int libxsmm_scratch_pools);
-/** Maximum total size of the scratch memory domain. */
-LIBXSMM_APIVAR(size_t libxsmm_scratch_limit);
+LIBXSMM_APIVAR_PRIVATE(unsigned int libxsmm_scratch_pools);
 /** Growth factor used to scale the scratch memory in case of reallocation. */
-LIBXSMM_APIVAR(double libxsmm_scratch_scale);
-/** even: regular, odd: scratch, >1: intercept */
-LIBXSMM_APIVAR(int libxsmm_malloc_kind);
+LIBXSMM_APIVAR_PRIVATE(double libxsmm_scratch_scale);
+/** Number of seconds per RDTSC-cycle (zero or negative if RDTSC invalid). */
+LIBXSMM_APIVAR_PRIVATE(double libxsmm_timer_scale);
+/** Counts the number of attempts to create an SPMDM-handle. */
+LIBXSMM_APIVAR_PRIVATE(unsigned int libxsmm_statistic_num_spmdm);
+/** Counts the maximum number of thread that have been active. */
+LIBXSMM_APIVAR_PRIVATE(unsigned int libxsmm_thread_count);
+/** Security-enhanced environment. */
+LIBXSMM_APIVAR_PRIVATE(int libxsmm_se);
 
-LIBXSMM_APIVAR(unsigned int libxsmm_statistic_num_spmdm);
-/** Number of seconds per RDTSC-cycle (zero if RDTSC is not used for wall-clock) */
-LIBXSMM_APIVAR(double libxsmm_timer_scale);
-/** Security-enhanced environment */
-LIBXSMM_APIVAR(int libxsmm_se);
+#if (0 != LIBXSMM_SYNC)
+LIBXSMM_APIVAR_PRIVATE(LIBXSMM_TLS_TYPE libxsmm_tlskey);
+#endif
 
 #endif /*LIBXSMM_MAIN_H*/
 
