@@ -1,30 +1,10 @@
 /******************************************************************************
-** Copyright (c) 2009-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Hans Pabst (Intel Corp.)
 ******************************************************************************/
@@ -34,9 +14,6 @@
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
 #endif
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
 #include <ctype.h>
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(pop)
@@ -51,19 +28,23 @@
 #endif
 
 #define LIBXSMM_MHD_MINMAX(TYPE, DATA, NELEMENTS, PMIN_INOUT, PMAX_INOUT) { \
-  size_t libxsmm_mhd_minmax_index_; \
   LIBXSMM_ASSERT(NULL != (PMIN_INOUT) && NULL != (PMAX_INOUT)); \
-  for (libxsmm_mhd_minmax_index_ = 0; libxsmm_mhd_minmax_index_ < (NELEMENTS); ++libxsmm_mhd_minmax_index_) { \
-    TYPE libxsmm_mhd_minmax_value_; \
-    LIBXSMM_ASSERT(NULL != (DATA)); \
-    libxsmm_mhd_minmax_value_ = ((const TYPE*)DATA)[libxsmm_mhd_minmax_index_]; \
-    if (libxsmm_mhd_minmax_value_ < *((const TYPE*)PMIN_INOUT)) { \
-      *((TYPE*)PMIN_INOUT) = libxsmm_mhd_minmax_value_; \
-    } \
-    else if (libxsmm_mhd_minmax_value_ > *((const TYPE*)PMAX_INOUT)) { \
-      *((TYPE*)PMAX_INOUT) = libxsmm_mhd_minmax_value_; \
-    } \
+  if (0 < (NELEMENTS)) { \
+    size_t libxsmm_mhd_minmax_index_ = 0; \
+    do { \
+      TYPE libxsmm_mhd_minmax_value_; \
+      LIBXSMM_ASSERT(NULL != (DATA)); \
+      libxsmm_mhd_minmax_value_ = ((const TYPE*)DATA)[libxsmm_mhd_minmax_index_]; \
+      if (libxsmm_mhd_minmax_value_ < *((const TYPE*)PMIN_INOUT)) { \
+        *((TYPE*)PMIN_INOUT) = libxsmm_mhd_minmax_value_; \
+      } \
+      else if (libxsmm_mhd_minmax_value_ > *((const TYPE*)PMAX_INOUT)) { \
+        *((TYPE*)PMAX_INOUT) = libxsmm_mhd_minmax_value_; \
+      } \
+      ++libxsmm_mhd_minmax_index_; \
+    } while (libxsmm_mhd_minmax_index_ < (NELEMENTS)); \
   } \
+  else *((TYPE*)PMIN_INOUT) = *((TYPE*)PMAX_INOUT) = 0; \
 }
 
 #define LIBXSMM_MHD_TYPE_PROMOTE(DST_TYPE, SRC_TYPE) \
@@ -193,7 +174,7 @@ LIBXSMM_API const char* libxsmm_mhd_typename(libxsmm_mhd_elemtype type, size_t*
     case LIBXSMM_MHD_ELEMTYPE_U8:   { size = 1; mhd_typename = "MET_UCHAR";  c_typename = "unsigned char";      } break;
     default: size = libxsmm_typesize((libxsmm_datatype)type); /* fall-back */
   }
-  assert(size <= LIBXSMM_MHD_MAX_ELEMSIZE);
+  LIBXSMM_ASSERT(size <= LIBXSMM_MHD_MAX_ELEMSIZE);
   if (NULL != ctypename) *ctypename = c_typename;
   if (NULL != typesize) *typesize = size;
   return mhd_typename;
@@ -246,7 +227,7 @@ LIBXSMM_API_INLINE int internal_mhd_readline(char buffer[], char split, size_t*
 
   if (0 != isplit) {
     char* i = isplit;
-    assert(0 != key_end && 0 != value_begin);
+    LIBXSMM_ASSERT(0 != key_end && 0 != value_begin);
     while (buffer != i) { --i;  if (0 == isspace((int)(*i))) break; }
     *key_end = i - buffer + 1;
     i = isplit;
@@ -338,7 +319,7 @@ LIBXSMM_API int libxsmm_mhd_read_header(const char header_filename[], size_t fil
             const size_t len = strlen(header_filename);
             if (0 <= file_position && len < filename_max_length) {
               memcpy(filename, header_filename, len + 1);
-              assert(0 == filename[len]);
+              LIBXSMM_ASSERT(0 == filename[len]);
               *header_size = ftell(file);
             }
             else {
@@ -351,7 +332,7 @@ LIBXSMM_API int libxsmm_mhd_read_header(const char header_filename[], size_t fil
           const size_t len = strlen(value);
           if (len < filename_max_length) {
             memcpy(filename, value, len + 1);
-            assert(0 == filename[len]);
+            LIBXSMM_ASSERT(0 == filename[len]);
           }
           else {
             result = EXIT_FAILURE;
@@ -538,11 +519,12 @@ LIBXSMM_API int libxsmm_mhd_element_comparison(
 }
 
 
+/* coverity[var_deref_op] */
 LIBXSMM_API_INLINE int internal_mhd_minmax(const void* data, size_t nelements,
   libxsmm_mhd_elemtype type, const void* minval, const void* maxval)
 {
   int result;
-  if ((NULL != data || 0 < nelements) && NULL != minval && NULL != maxval) {
+  if ((NULL != data || 0 == nelements) && NULL != minval && NULL != maxval) {
     result = EXIT_SUCCESS;
     switch (type) {
       case LIBXSMM_MHD_ELEMTYPE_F64: {
@@ -584,7 +566,7 @@ LIBXSMM_API_INLINE int internal_mhd_read(FILE* file, void* data, const size_t si
   int result = EXIT_SUCCESS;
   size_t typesize_stored;
 
-  assert(0 != pitch && 0 != typesize);
+  LIBXSMM_ASSERT(0 != pitch && 0 != typesize);
   if (0 != libxsmm_mhd_typename(type_stored, &typesize_stored, NULL/*ctypename*/)) {
     if (1 < ndims) {
       if (size[0] <= pitch[0]) {
@@ -706,7 +688,7 @@ LIBXSMM_API int libxsmm_mhd_read(const char filename[],
           pitch1 *= shape[i];
           size1 *= size[i];
         }
-        assert(size1 <= pitch1);
+        LIBXSMM_ASSERT(size1 <= pitch1);
         if (size1 != pitch1 && 0 == handle_element) {
           memset(data, 0, pitch1 * ncomponents * typesize);
         }
@@ -722,7 +704,8 @@ LIBXSMM_API int libxsmm_mhd_read(const char filename[],
       if (0 != header_size) result = fseek(file, (long)header_size, SEEK_SET); /* set file position to data section */
       if (EXIT_SUCCESS == result && datatype != type_stored) { /* conversion needed */
         if (1 == fread(minmax, typesize, 1, file)) {
-          memcpy(minmax + (LIBXSMM_MHD_MAX_ELEMSIZE), minmax, typesize);
+          LIBXSMM_ASSERT(typesize <= (LIBXSMM_MHD_MAX_ELEMSIZE));
+          LIBXSMM_MEMCPY127(minmax + (LIBXSMM_MHD_MAX_ELEMSIZE), minmax, typesize);
           result = fseek(file, (long)header_size, SEEK_SET); /* reset file position */
           if (EXIT_SUCCESS == result) {
             result = internal_mhd_read(file, NULL/*output*/, size, shape,
@@ -765,7 +748,7 @@ LIBXSMM_API_INLINE int internal_mhd_write(FILE* file, const void* data, const si
 {
   int result = EXIT_SUCCESS;
 
-  assert(0 != pitch);
+  LIBXSMM_ASSERT(0 != pitch);
   if (1 < ndims) {
     if (size[0] <= pitch[0]) {
       const size_t d = ndims - 1;
@@ -912,7 +895,8 @@ LIBXSMM_API int libxsmm_mhd_write(const char filename[],
 
       result = (0 <= file_position ? EXIT_SUCCESS : EXIT_FAILURE);
       if (EXIT_SUCCESS == result && type_data != elemtype) { /* conversion needed */
-        memcpy(minmax, data, typesize_data); memcpy(minmax + (LIBXSMM_MHD_MAX_ELEMSIZE), data, typesize_data); /* initial condition */
+        LIBXSMM_MEMCPY127(minmax, data, typesize_data);
+        LIBXSMM_MEMCPY127(minmax + (LIBXSMM_MHD_MAX_ELEMSIZE), data, typesize_data); /* initial condition */
         result = internal_mhd_write(file, input, size, shape, ndims, ncomponents, type_data, elemtype, typesize_data, typesize,
           1/*search min-max*/, minmax, minmax + (LIBXSMM_MHD_MAX_ELEMSIZE));
       }
