@@ -1,45 +1,22 @@
 /******************************************************************************
-** Copyright (c) 2015-2019, Google Inc.                                      **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Maciej Debski (Google Inc.)
 ******************************************************************************/
 #include "libxsmm_perf.h"
+#include <libxsmm_sync.h>
 
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
 #endif
 #include "perf_jitdump.h"
-#include <inttypes.h>
-#include <stdlib.h>
-#include <stdio.h>
 #if defined(LIBXSMM_PERF_JITDUMP) && !defined(_WIN32)
 # include <sys/mman.h>
-# include <string.h>
 # include <sys/types.h>
 # include <sys/types.h>
 # include <sys/stat.h>
@@ -75,10 +52,10 @@
 #endif
 
 
-LIBXSMM_APIVAR(FILE* internal_perf_fp);
+LIBXSMM_APIVAR_DEFINE(FILE* internal_perf_fp);
 #if defined(LIBXSMM_PERF_JITDUMP) && !defined(_WIN32)
-LIBXSMM_APIVAR(void* internal_perf_marker);
-LIBXSMM_APIVAR(int internal_perf_codeidx);
+LIBXSMM_APIVAR_DEFINE(void* internal_perf_marker);
+LIBXSMM_APIVAR_DEFINE(int internal_perf_codeidx);
 #endif
 
 
@@ -165,7 +142,7 @@ LIBXSMM_API_INTERN void libxsmm_perf_init(void)
     goto error;
   }
 
-  memset(&header, 0, sizeof(header));
+  LIBXSMM_MEMZERO127(&header);
   header.magic      = JITDUMP_MAGIC;
   header.version    = JITDUMP_VERSION;
   header.elf_mach   = 62;  /* EM_X86_64 */
@@ -211,7 +188,7 @@ LIBXSMM_API_INTERN void libxsmm_perf_finalize(void)
     goto error;
   }
 
-  memset(&hdr, 0, sizeof(hdr));
+  LIBXSMM_MEMZERO127(&hdr);
   hdr.id = JITDUMP_CODE_CLOSE;
   hdr.total_size = sizeof(hdr);
   hdr.timestamp = libxsmm_timer_tick();
@@ -239,6 +216,7 @@ error:
 }
 
 
+#if defined(LIBXSMM_PERF_JITDUMP) && !defined(_WIN32)
 /** Utility function to receive the OS-specific thread ID. */
 LIBXSMM_API_INLINE unsigned int internal_perf_get_tid(void)
 {
@@ -248,6 +226,7 @@ LIBXSMM_API_INLINE unsigned int internal_perf_get_tid(void)
   return libxsmm_get_tid();
 #endif
 }
+#endif
 
 
 LIBXSMM_API_INTERN void libxsmm_perf_dump_code(const void* memory, size_t size, const char* name)
@@ -262,8 +241,8 @@ LIBXSMM_API_INTERN void libxsmm_perf_dump_code(const void* memory, size_t size,
     struct jitdump_record_code_load rec;
     size_t name_len = strlen(name) + 1;
 
-    memset(&hdr, 0, sizeof(hdr));
-    memset(&rec, 0, sizeof(rec));
+    LIBXSMM_MEMZERO127(&hdr);
+    LIBXSMM_MEMZERO127(&rec);
 
     hdr.id = JITDUMP_CODE_LOAD;
     hdr.total_size = sizeof(hdr) + sizeof(rec) + name_len + size;
