@@ -1,39 +1,16 @@
 /******************************************************************************
-** Copyright (c) 2016-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Hans Pabst (Intel Corp.)
 ******************************************************************************/
 #include "libxsmm_trace.h"
-#include <libxsmm_sync.h>
+#include "libxsmm_main.h"
 
-#if !defined(LIBXSMM_TRACE_DLINFO) /*&& !defined(__STATIC)*/
-# define LIBXSMM_TRACE_DLINFO
-#endif
 #if !defined(LIBXSMM_TRACE_MINDEPTH) || 0 > (LIBXSMM_TRACE_MINDEPTH)
 # undef LIBXSMM_TRACE_MINDEPTH
 # define LIBXSMM_TRACE_MINDEPTH 1
@@ -46,14 +23,13 @@
 # undef LIBXSMM_TRACE_SYMBOLSIZE
 # define LIBXSMM_TRACE_SYMBOLSIZE 256
 #endif
+#if !defined(LIBXSMM_TRACE_DLINFO) && defined(__USE_GNU)
+# define LIBXSMM_TRACE_DLINFO
+#endif
 
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
 #endif
-#include <inttypes.h>
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
 #if !defined(NDEBUG)
 # include <errno.h>
 #endif
@@ -67,9 +43,9 @@
 # if defined(_MSC_VER)
 #   pragma warning(pop)
 # endif
-LIBXSMM_APIVAR(volatile LONG internal_trace_initialized);
+LIBXSMM_APIVAR_DEFINE(volatile LONG internal_trace_initialized);
 #else
-LIBXSMM_APIVAR(volatile int internal_trace_initialized);
+LIBXSMM_APIVAR_DEFINE(volatile int internal_trace_initialized);
 # include <execinfo.h>
 # if defined(LIBXSMM_TRACE_DLINFO)
 #   include <dlfcn.h>
@@ -80,34 +56,35 @@ LIBXSMM_APIVAR(volatile int internal_trace_initialized);
 #   include <pthread.h>
 #   include <fcntl.h>
 #   if (0 != LIBXSMM_SYNC)
-LIBXSMM_APIVAR(pthread_key_t internal_trace_key);
+LIBXSMM_APIVAR_DEFINE(LIBXSMM_TLS_TYPE internal_trace_key);
+LIBXSMM_APIVAR_DEFINE(void* internal_trace_symbols[LIBXSMM_NTHREADS_MAX]);
 #   endif
 LIBXSMM_API_INLINE void internal_delete(void* value)
 {
   int fd;
-#   if !(defined(__APPLE__) && defined(__MACH__))
+# if !(defined(__APPLE__) && defined(__MACH__))
   LIBXSMM_ASSERT(NULL != value);
-#   endif
+# endif
   fd = *((int*)value);
-#   if defined(NDEBUG)
+# if defined(NDEBUG)
   munmap(value, LIBXSMM_TRACE_SYMBOLSIZE);
-#   else /* library code is expected to be mute */
+# else /* library code is expected to be mute */
   if (0 != munmap(value, LIBXSMM_TRACE_SYMBOLSIZE)) {
     const int error = errno;
     fprintf(stderr, "LIBXSMM ERROR: %s (munmap error #%i at %p)\n",
       strerror(error), error, value);
   }
-#   endif
+# endif
   if (0 <= fd) {
     close(fd);
   }
-#   if !defined(NDEBUG) /* library code is expected to be mute */
+# if !defined(NDEBUG) /* library code is expected to be mute */
   else {
     fprintf(stderr, "LIBXSMM ERROR: invalid file descriptor (%i)\n", fd);
   }
-#   endif
+# endif
 }
-#   if defined(__APPLE__) && defined(__MACH__)
+# if defined(__APPLE__) && defined(__MACH__)
 /* taken from "libtransmission" fdlimit.c */
 LIBXSMM_API_INLINE int posix_fallocate(int fd, off_t offset, off_t length)
 {
@@ -119,29 +96,24 @@ LIBXSMM_API_INLINE int posix_fallocate(int fd, off_t offset, off_t length)
   fst.fst_bytesalloc = 0;
   return fcntl(fd, F_PREALLOCATE, &fst);
 }
-#   elif (!defined(_XOPEN_SOURCE) || 600 > _XOPEN_SOURCE) && \
-         (!defined(_POSIX_C_SOURCE) || 200112L > _POSIX_C_SOURCE)
+# elif (!defined(_XOPEN_SOURCE) || 600 > _XOPEN_SOURCE) && \
+       (!defined(_POSIX_C_SOURCE) || 200112L > _POSIX_C_SOURCE)
 /* C89: avoid warning about posix_fallocate declared implicitly */
 LIBXSMM_EXTERN int posix_fallocate(int, off_t, off_t);
-#   endif
 # endif
+# endif
+LIBXSMM_EXTERN int mkstemp(char*) LIBXSMM_NOTHROW;
 #endif
 #if defined(LIBXSMM_OFFLOAD_TARGET)
 # pragma offload_attribute(pop)
 #endif
 
-
-LIBXSMM_APIVAR(int internal_trace_mindepth);
-LIBXSMM_APIVAR(int internal_trace_threadid);
-LIBXSMM_APIVAR(int internal_trace_maxnsyms);
+LIBXSMM_APIVAR_DEFINE(int internal_trace_mindepth);
+LIBXSMM_APIVAR_DEFINE(int internal_trace_threadid);
+LIBXSMM_APIVAR_DEFINE(int internal_trace_maxnsyms);
 
 
-LIBXSMM_API
-#if defined(__GNUC__)
-LIBXSMM_ATTRIBUTE(no_instrument_function)
-#endif
-int libxsmm_trace_init(int filter_threadid, int filter_mindepth, int filter_maxnsyms);
-
+LIBXSMM_API LIBXSMM_ATTRIBUTE_NO_TRACE int libxsmm_trace_init(int /*filter_threadid*/, int /*filter_mindepth*/, int /*filter_maxnsyms*/);
 LIBXSMM_API int libxsmm_trace_init(int filter_threadid, int filter_mindepth, int filter_maxnsyms)
 {
   int result = EXIT_SUCCESS;
@@ -176,7 +148,7 @@ LIBXSMM_API int libxsmm_trace_init(int filter_threadid, int filter_mindepth, int
       SymSetOptions(SYMOPT_DEFERRED_LOADS | SYMOPT_UNDNAME);
       result = (FALSE != SymInitialize(GetCurrentProcess(), NULL, TRUE) ? EXIT_SUCCESS : GetLastError());
 # elif (0 != LIBXSMM_SYNC) && !defined(LIBXSMM_TRACE_DLINFO)
-      result = pthread_key_create(&internal_trace_key, internal_delete);
+      result = LIBXSMM_TLS_CREATE(&internal_trace_key);
 # endif
       if (EXIT_SUCCESS == result) {
         internal_trace_threadid = filter_threadid;
@@ -197,12 +169,7 @@ LIBXSMM_API int libxsmm_trace_init(int filter_threadid, int filter_mindepth, int
 }
 
 
-LIBXSMM_API
-#if defined(__GNUC__)
-LIBXSMM_ATTRIBUTE(no_instrument_function)
-#endif
-int libxsmm_trace_finalize(void);
-
+LIBXSMM_API LIBXSMM_ATTRIBUTE_NO_TRACE int libxsmm_trace_finalize(void);
 LIBXSMM_API int libxsmm_trace_finalize(void)
 {
   int result;
@@ -213,7 +180,13 @@ LIBXSMM_API int libxsmm_trace_finalize(void)
 # if defined(_WIN32) || defined(__CYGWIN__)
     result = (FALSE != SymCleanup(GetCurrentProcess()) ? EXIT_SUCCESS : GetLastError());
 # elif (0 != LIBXSMM_SYNC) && !defined(LIBXSMM_TRACE_DLINFO)
-    result = pthread_key_delete(internal_trace_key);
+    result = LIBXSMM_TLS_DESTROY(internal_trace_key);
+    { int i = 0;
+      for (; i < LIBXSMM_NTHREADS_MAX; ++i) {
+        void *const buffer = internal_trace_symbols[i];
+        if (NULL != buffer) internal_delete(buffer);
+      }
+    }
 # endif
   }
 #else
@@ -223,12 +196,7 @@ LIBXSMM_API int libxsmm_trace_finalize(void)
 }
 
 
-LIBXSMM_API
-#if defined(__GNUC__)
-LIBXSMM_ATTRIBUTE(no_instrument_function)
-#endif
-unsigned int libxsmm_backtrace(const void* buffer[], unsigned int size, unsigned int skip);
-
+LIBXSMM_API LIBXSMM_ATTRIBUTE_NO_TRACE unsigned int libxsmm_backtrace(const void* /*buffer*/[], unsigned int /*size*/, unsigned int /*skip*/);
 LIBXSMM_API
 #if defined(_WIN32)
 /*TODO: no inline*/
@@ -295,12 +263,9 @@ LIBXSMM_API_INLINE const char* internal_trace_get_symbolname(const void* address
 #endif
 
 
-LIBXSMM_API
-#if defined(__GNUC__)
-LIBXSMM_ATTRIBUTE(no_instrument_function)
-#endif
-const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, const int* filter_threadid,
-  const void* filter_symbol, const int* filter_mindepth, const int* filter_maxnsyms);
+LIBXSMM_API LIBXSMM_ATTRIBUTE_NO_TRACE
+const char* libxsmm_trace_info(unsigned int* /*depth*/, unsigned int* /*threadid*/, const int* /*filter_threadid*/,
+  const void* /*filter_symbol*/, const int* /*filter_mindepth*/, const int* /*filter_maxnsyms*/);
 
 LIBXSMM_API
 #if defined(_WIN32)
@@ -362,7 +327,7 @@ const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, cons
           value->NameLen = 0;
 #   endif
           if (NULL != filter_symbol) {
-            struct { size_t d; int s; } approx = { (size_t)-1, 0 };
+            struct { size_t d; int s; } approx = { (size_t)LIBXSMM_UNLIMITED, 0 };
             while (next < n && (filter_symbol == stacktrace[symbol] ||
 #   if defined(_WIN32) || defined(__CYGWIN__)
               (FALSE != SymFromAddr(process, (DWORD64)stacktrace[symbol], NULL, value) && 0 < value->NameLen)))
@@ -418,7 +383,7 @@ const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, cons
         static char raw_c;
         char */*const*/ raw_value = &raw_c; /* const: avoid warning (below / constant control-flow) */
 #   else
-        char *const raw_value = (char*)pthread_getspecific(internal_trace_key);
+        char *const raw_value = (char*)LIBXSMM_TLS_GETVALUE(internal_trace_key);
 #   endif
         const off_t fdoff = sizeof(int) * 2;
         int* ivalue = NULL, fd = -1;
@@ -440,6 +405,7 @@ const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, cons
         }
         else {
           char filename[] = "/tmp/.libxsmm_map." LIBXSMM_MKTEMP_PATTERN;
+          /* coverity[secure_temp] */
           fd = mkstemp(filename);
           if (0 <= fd) {
             if (0 == unlink(filename) && 0 == posix_fallocate(fd, 0, LIBXSMM_TRACE_SYMBOLSIZE)) {
@@ -450,15 +416,19 @@ const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, cons
                 ivalue[0] = fd; /* valid file descriptor for internal_delete */
                 if (
 #   if (0 != LIBXSMM_SYNC)
-                  0 == pthread_setspecific(internal_trace_key, buffer) &&
+                  0 == LIBXSMM_TLS_SETVALUE(internal_trace_key, buffer) &&
 #   endif
-                      (sizeof(int) * 1) == read(fd, &check, sizeof(int))
-                  && fdoff == lseek(fd, sizeof(int), SEEK_CUR)
-                  && check == fd)
+                  (sizeof(int) * 1) == read(fd, &check, sizeof(int)) &&
+                  fdoff == lseek(fd, sizeof(int), SEEK_CUR) &&
+                  check == fd)
                 {
                   const int tid = LIBXSMM_ATOMIC_ADD_FETCH(&internal_trace_initialized, 0 < init ? 1 : -1, LIBXSMM_ATOMIC_RELAXED);
                   abs_tid = LIBXSMM_ABS(tid) - 1;
                   LIBXSMM_ASSERT(0 < abs_tid);
+#   if (0 != LIBXSMM_SYNC)
+                  LIBXSMM_ASSERT(abs_tid < LIBXSMM_NTHREADS_MAX);
+                  internal_trace_symbols[abs_tid] = buffer;
+#   endif
                   /* use sign bit to flag enabled fall-back for symbol resolution */
                   ivalue[1] = -abs_tid;
                   if (0 > filter || (abs_tid - 1) == filter) {
@@ -490,7 +460,7 @@ const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, cons
         if (NULL != value) {
           int next = symbol + 1;
           if (NULL != filter_symbol) {
-            struct { size_t d; int s; } approx = { (size_t)-1, 0 };
+            struct { size_t d; int s; } approx = { (size_t)LIBXSMM_UNLIMITED, 0 };
             while (next < n && (filter_symbol == stacktrace[symbol] ||
               NULL != internal_trace_get_symbolname(stacktrace[symbol], value, fd, fdoff)))
             {
@@ -542,11 +512,8 @@ const char* libxsmm_trace_info(unsigned int* depth, unsigned int* threadid, cons
 }
 
 
-LIBXSMM_API
-#if defined(__GNUC__)
-LIBXSMM_ATTRIBUTE(no_instrument_function)
-#endif
-void libxsmm_trace(FILE* stream, const int* filter_threadid, const void* filter_symbol, const int* filter_mindepth, const int* filter_maxnsyms);
+LIBXSMM_API LIBXSMM_ATTRIBUTE_NO_TRACE
+void libxsmm_trace(FILE* stream, const int* /*filter_threadid*/, const void* /*filter_symbol*/, const int* /*filter_mindepth*/, const int* /*filter_maxnsyms*/);
 
 LIBXSMM_API void libxsmm_trace(FILE* stream, const int* filter_threadid, const void* filter_symbol, const int* filter_mindepth, const int* filter_maxnsyms)
 {
@@ -574,20 +541,20 @@ LIBXSMM_API void libxsmm_trace(FILE* stream, const int* filter_threadid, const v
 
 #if defined(__TRACE) && defined(__GNUC__) && defined(LIBXSMM_BUILD)
 
-LIBXSMM_API LIBXSMM_ATTRIBUTE(no_instrument_function) void __cyg_profile_func_enter(void* this_fn, void* call_site);
+LIBXSMM_API LIBXSMM_ATTRIBUTE_NO_TRACE void __cyg_profile_func_enter(void* /*this_fn*/, void* /*call_site*/);
 LIBXSMM_API void __cyg_profile_func_enter(void* this_fn, void* call_site)
 {
 #if defined(LIBXSMM_TRACE)
   if (0 > internal_trace_initialized) {
     /* NULL: inherit global settings from libxsmm_trace_init */
-    libxsmm_trace(stderr, NULL/*filter_threadid*/, LIBXSMM_FUNCNAME, NULL, NULL);
+    libxsmm_trace(stderr, NULL/*filter_threadid*/, "__cyg_profile_func_enter"/*LIBXSMM_FUNCNAME*/, NULL, NULL);
   }
 #endif
   LIBXSMM_UNUSED(this_fn); LIBXSMM_UNUSED(call_site);
 }
 
 
-LIBXSMM_API LIBXSMM_ATTRIBUTE(no_instrument_function) void __cyg_profile_func_exit(void* this_fn, void* call_site);
+LIBXSMM_API LIBXSMM_ATTRIBUTE_NO_TRACE void __cyg_profile_func_exit(void* /*this_fn*/, void* /*call_site*/);
 LIBXSMM_API void __cyg_profile_func_exit(void* this_fn, void* call_site)
 {
   LIBXSMM_UNUSED(this_fn); LIBXSMM_UNUSED(call_site); /* suppress warning */
