@@ -1,46 +1,16 @@
 /******************************************************************************
-** Copyright (c) 2016-2019, Intel Corporation                                **
-** All rights reserved.                                                      **
-**                                                                           **
-** Redistribution and use in source and binary forms, with or without        **
-** modification, are permitted provided that the following conditions        **
-** are met:                                                                  **
-** 1. Redistributions of source code must retain the above copyright         **
-**    notice, this list of conditions and the following disclaimer.          **
-** 2. Redistributions in binary form must reproduce the above copyright      **
-**    notice, this list of conditions and the following disclaimer in the    **
-**    documentation and/or other materials provided with the distribution.   **
-** 3. Neither the name of the copyright holder nor the names of its          **
-**    contributors may be used to endorse or promote products derived        **
-**    from this software without specific prior written permission.          **
-**                                                                           **
-** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS       **
-** "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT         **
-** LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR     **
-** A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT      **
-** HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    **
-** SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED  **
-** TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR    **
-** PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF    **
-** LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING      **
-** NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS        **
-** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              **
+* Copyright (c) Intel Corporation - All rights reserved.                      *
+* This file is part of the LIBXSMM library.                                   *
+*                                                                             *
+* For information on the license, see the LICENSE file.                       *
+* Further information: https://github.com/hfp/libxsmm/                        *
+* SPDX-License-Identifier: BSD-3-Clause                                       *
 ******************************************************************************/
 /* Hans Pabst (Intel Corp.)
 ******************************************************************************/
 #include "libxsmm_xcopy.h"
 #include "libxsmm_main.h"
 
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(push,target(LIBXSMM_OFFLOAD_TARGET))
-#endif
-#include <stdlib.h>
-#include <string.h>
-#include <stdio.h>
-#if defined(LIBXSMM_OFFLOAD_TARGET)
-# pragma offload_attribute(pop)
-#endif
-
 #if !defined(LIBXSMM_XCOPY_JIT)
 # if (defined(_WIN32) || defined(__CYGWIN__))
 /* only enable matcopy code generation (workaround issue with taking GP registers correctly) */
@@ -49,9 +19,19 @@
 #   define LIBXSMM_XCOPY_JIT 3
 # endif
 #endif
+#if !defined(LIBXSMM_XCOPY_JIT_TINY) && 0
+# define LIBXSMM_XCOPY_JIT_TINY
+#endif
 
 
-LIBXSMM_API_INTERN void libxsmm_trans_init(int archid)
+/* definition of corresponding variables */
+LIBXSMM_APIVAR_PUBLIC_DEF(int libxsmm_trans_jit);
+LIBXSMM_APIVAR_PUBLIC_DEF(float libxsmm_trans_tile_stretch);
+LIBXSMM_APIVAR_PUBLIC_DEF(unsigned int* libxsmm_trans_mtile);
+LIBXSMM_APIVAR_PUBLIC_DEF(int libxsmm_trans_taskscale);
+
+
+LIBXSMM_API_INTERN void libxsmm_xcopy_init(int archid)
 {
   /* setup tile sizes according to CPUID or environment (LIBXSMM_XCOPY_M, LIBXSMM_XCOPY_N) */
   static unsigned int config_tm[/*config*/][2/*DP/SP*/] = {
@@ -98,7 +78,7 @@ LIBXSMM_API_INTERN void libxsmm_trans_init(int archid)
 }
 
 
-LIBXSMM_API_INTERN void libxsmm_trans_finalize(void)
+LIBXSMM_API_INTERN void libxsmm_xcopy_finalize(void)
 {
 }
 
@@ -133,7 +113,7 @@ LIBXSMM_API void libxsmm_matcopy_thread_internal(void* out, const void* in, unsi
   LIBXSMM_ASSERT_MSG(m0 <= m1 && m1 <= m, "Invalid task size");
   LIBXSMM_ASSERT_MSG(n0 <= n1 && n1 <= n, "Invalid task size");
 
-  if (0 != prefetch && 0 != *prefetch) { /* prefetch */
+  if (NULL != prefetch && 0 != *prefetch) { /* prefetch */
     libxsmm_matcopy_internal_pf(out, in, typesize, ldi, ldo,
       m0, m1, n0, n1, tm, tn, kernel);
   }
@@ -172,7 +152,7 @@ LIBXSMM_API void libxsmm_matcopy_thread(void* out, const void* in, unsigned int
 {
   LIBXSMM_INIT
   if (0 < typesize && m <= ldi && m <= ldo && out != in &&
-    ((0 != out && 0 < m && 0 < n) || (0 == m && 0 == n)) &&
+    ((NULL != out && 0 < m && 0 < n) || (0 == m && 0 == n)) &&
     /* use (signed) integer types, but check sanity of input */
     0 <= tid && tid < nthreads)
   {
@@ -180,7 +160,7 @@ LIBXSMM_API void libxsmm_matcopy_thread(void* out, const void* in, unsigned int
       unsigned int tm = libxsmm_trans_mtile[4 < typesize ? 0 : 1];
       unsigned int tn = (unsigned int)(libxsmm_trans_tile_stretch * tm);
       libxsmm_xmcopyfunction kernel = NULL;
-      if (m < (libxsmm_blasint)tm || n < (libxsmm_blasint)tn) {
+      if ((unsigned int)m < tm || (unsigned int)n < tn) {
         if (1 == nthreads) {
           tm = (unsigned int)m; tn = (unsigned int)n;
         }
@@ -192,12 +172,15 @@ LIBXSMM_API void libxsmm_matcopy_thread(void* out, const void* in, unsigned int
           tm = LIBXSMM_CLMP((unsigned int)m, 1, mm);
         }
       }
-      else if (0 != (1 & libxsmm_trans_jit)) { /* JIT'ted matrix-copy permitted? */
+#if !defined(LIBXSMM_XCOPY_JIT_TINY)
+      else
+#endif
+      if (0 != (1 & libxsmm_trans_jit)) { /* JIT'ted matrix-copy permitted? */
         const int iprefetch = (0 == prefetch ? 0 : *prefetch);
         libxsmm_descriptor_blob blob;
-        kernel = libxsmm_dispatch_mcopy(libxsmm_mcopy_descriptor_init(&blob, typesize,
-          (unsigned int)tm, (unsigned int)tn, (unsigned int)ldo, (unsigned int)ldi,
-          0 != in ? 0 : LIBXSMM_MATCOPY_FLAG_ZERO_SOURCE, iprefetch, NULL/*default unroll*/));
+        kernel = libxsmm_dispatch_mcopy(libxsmm_mcopy_descriptor_init(&blob, typesize, tm, tn,
+          (unsigned int)ldo, (unsigned int)ldi, NULL != in ? 0 : LIBXSMM_MATCOPY_FLAG_ZERO_SOURCE,
+          iprefetch, NULL/*default unroll*/));
       }
       libxsmm_matcopy_thread_internal(out, in, typesize,
         (unsigned int)m, (unsigned int)n, (unsigned int)ldi, (unsigned int)ldo,
@@ -292,7 +275,7 @@ LIBXSMM_API void libxsmm_otrans_thread(void* out, const void* in, unsigned int t
   static int error_once = 0;
   LIBXSMM_INIT
   if (0 < typesize && m <= ldi && n <= ldo &&
-    ((0 != out && 0 != in && 0 < m && 0 < n) || (0 == m && 0 == n)) &&
+    ((NULL != out && NULL != in && 0 < m && 0 < n) || (0 == m && 0 == n)) &&
     /* use (signed) integer types, but check sanity of input */
     0 <= tid && tid < nthreads)
   {
@@ -381,85 +364,75 @@ LIBXSMM_API void libxsmm_itrans(void* inout, unsigned int typesize,
 {
   static int error_once = 0;
   LIBXSMM_INIT
-  if (0 != inout) {
-    if (m == n) { /* some fall-back; still warned as "not implemented" */
-      libxsmm_blasint i, j;
+  if (NULL != inout && 0 < typesize && typesize <= 127 && m <= ld && LIBXSMM_MAX(n, 1) <= ld) {
+    const signed char c = (signed char)typesize;
+    libxsmm_blasint i, j;
+    if (m == n) {
       for (i = 0; i < m; ++i) {
         for (j = 0; j < i; ++j) {
-          char *const a = (char*)inout + ((size_t)i * ld + j) * typesize;
-          char *const b = (char*)inout + ((size_t)j * ld + i) * typesize;
-          unsigned int k;
-          for (k = 0; k < typesize; ++k) {
-            const char tmp = a[k];
-            a[k] = b[k];
-            b[k] = tmp;
-          }
+          char *const a = &((char*)inout)[(i*ld+j)*typesize];
+          char *const b = &((char*)inout)[(j*ld+i)*typesize];
+          signed char k = 0;
+          for (; k < c; ++k) LIBXSMM_ISWAP(a[k], b[k]);
         }
       }
-#if defined(LIBXSMM_XCOPY_CHECK)
-      if ((LIBXSMM_VERBOSITY_WARN <= libxsmm_verbosity || 0 > libxsmm_verbosity) /* library code is expected to be mute */
-        && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
-      {
-        fprintf(stderr, "LIBXSMM WARNING: in-place transpose is not fully implemented!\n");
-      }
-#endif
     }
     else {
-      if (0 != libxsmm_verbosity /* library code is expected to be mute */
+      if ( 0 != libxsmm_verbosity /* library code is expected to be mute */
         && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
       {
-        fprintf(stderr, "LIBXSMM ERROR: in-place transpose is not fully implemented!\n");
+        fprintf(stderr, "LIBXSMM ERROR: in-place transpose is not implemented!\n");
       }
-      LIBXSMM_ASSERT(0/*TODO: proper implementation is pending*/);
+      LIBXSMM_ASSERT_MSG(0, "in-place transpose is not implemented!");
     }
   }
   else if (0 != libxsmm_verbosity /* library code is expected to be mute */
     && 1 == LIBXSMM_ATOMIC_ADD_FETCH(&error_once, 1, LIBXSMM_ATOMIC_RELAXED))
   {
-    fprintf(stderr, "LIBXSMM ERROR: the transpose input/output cannot be NULL!\n");
+    fprintf(stderr, "LIBXSMM ERROR: unsupported or invalid arguments for in-place transpose!\n");
   }
 }
 
 
-#if defined(LIBXSMM_BUILD) && !defined(LIBXSMM_NOFORTRAN)
+#if defined(LIBXSMM_BUILD) && (!defined(LIBXSMM_NOFORTRAN) || defined(__clang_analyzer__))
 
 /* implementation provided for Fortran 77 compatibility */
-LIBXSMM_API void LIBXSMM_FSYMBOL(libxsmm_matcopy)(void* /*out*/, const void* /*in*/, const unsigned int* /*typesize*/,
+LIBXSMM_API void LIBXSMM_FSYMBOL(libxsmm_matcopy)(void* /*out*/, const void* /*in*/, const int* /*typesize*/,
   const libxsmm_blasint* /*m*/, const libxsmm_blasint* /*n*/, const libxsmm_blasint* /*ldi*/, const libxsmm_blasint* /*ldo*/,
   const int* /*prefetch*/);
-LIBXSMM_API void LIBXSMM_FSYMBOL(libxsmm_matcopy)(void* out, const void* in, const unsigned int* typesize,
+LIBXSMM_API void LIBXSMM_FSYMBOL(libxsmm_matcopy)(void* out, const void* in, const int* typesize,
   const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* ldi, const libxsmm_blasint* ldo,
   const int* prefetch)
 {
   libxsmm_blasint ldx;
-  LIBXSMM_ASSERT(0 != typesize && 0 != m);
-  ldx = *(0 != ldi ? ldi : m);
-  libxsmm_matcopy(out, in, *typesize, *m, *(0 != n ? n : m), ldx, 0 != ldo ? *ldo : ldx, prefetch);
+  LIBXSMM_ASSERT(NULL != typesize && 0 < *typesize && NULL != m);
+  ldx = *(NULL != ldi ? ldi : m);
+  libxsmm_matcopy(out, in, (unsigned int)*typesize, *m, *(NULL != n ? n : m), ldx, NULL != ldo ? *ldo : ldx, prefetch);
 }
 
 
 /* implementation provided for Fortran 77 compatibility */
-LIBXSMM_API void LIBXSMM_FSYMBOL(libxsmm_otrans)(void* /*out*/, const void* /*in*/, const unsigned int* /*typesize*/,
+LIBXSMM_API void LIBXSMM_FSYMBOL(libxsmm_otrans)(void* /*out*/, const void* /*in*/, const int* /*typesize*/,
   const libxsmm_blasint* /*m*/, const libxsmm_blasint* /*n*/, const libxsmm_blasint* /*ldi*/, const libxsmm_blasint* /*ldo*/);
-LIBXSMM_API void LIBXSMM_FSYMBOL(libxsmm_otrans)(void* out, const void* in, const unsigned int* typesize,
+LIBXSMM_API void LIBXSMM_FSYMBOL(libxsmm_otrans)(void* out, const void* in, const int* typesize,
   const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* ldi, const libxsmm_blasint* ldo)
 {
   libxsmm_blasint ldx;
-  LIBXSMM_ASSERT(0 != typesize && 0 != m);
-  ldx = *(0 != ldi ? ldi : m);
-  libxsmm_otrans(out, in, *typesize, *m, *(0 != n ? n : m), ldx, 0 != ldo ? *ldo : ldx);
+  LIBXSMM_ASSERT(NULL != typesize && 0 < *typesize && NULL != m);
+  ldx = *(NULL != ldi ? ldi : m);
+  libxsmm_otrans(out, in, (unsigned int)*typesize, *m, *(NULL != n ? n : m), ldx, NULL != ldo ? *ldo : ldx);
 }
 
 
 /* implementation provided for Fortran 77 compatibility */
-LIBXSMM_API void LIBXSMM_FSYMBOL(libxsmm_itrans)(void* /*inout*/, const unsigned int* /*typesize*/,
+LIBXSMM_API void LIBXSMM_FSYMBOL(libxsmm_itrans)(void* /*inout*/, const int* /*typesize*/,
   const libxsmm_blasint* /*m*/, const libxsmm_blasint* /*n*/, const libxsmm_blasint* /*ld*/);
-LIBXSMM_API void LIBXSMM_FSYMBOL(libxsmm_itrans)(void* inout, const unsigned int* typesize,
+LIBXSMM_API void LIBXSMM_FSYMBOL(libxsmm_itrans)(void* inout, const int* typesize,
   const libxsmm_blasint* m, const libxsmm_blasint* n, const libxsmm_blasint* ld)
 {
-  LIBXSMM_ASSERT(0 != typesize && 0 != m);
-  libxsmm_itrans(inout, *typesize, *m, *(0 != n ? n : m), *(0 != ld ? ld : m));
+  LIBXSMM_ASSERT(NULL != typesize && 0 < *typesize && NULL != m);
+  libxsmm_itrans(inout, (unsigned int)*typesize, *m, *(NULL != n ? n : m), *(NULL != ld ? ld : m));
 }
 
-#endif /*defined(LIBXSMM_BUILD)*/
+#endif /*defined(LIBXSMM_BUILD) && (!defined(LIBXSMM_NOFORTRAN) || defined(__clang_analyzer__))*/
 
